"""
Lots of menu's here.
"""


import pygame
from pygame.locals import *
import os, sys, time
from scene import *
from weapons import *
from game.m_FlamingoEntity import *
from pgu import gui, html
from game.m_MedicBox import *
from game.m_Game import Game, GameConfig
from network_game import ClientGameInitiator, ServerGameInitiator, ServerGame, ClientGame
from net import FlameingoIPC



def _write_text_(doc, text, cls='label'):
    for line in text.split("\n"):
        for word in line.split(" "):
            doc.add(gui.Label(word + " ", cls=cls))
        doc.br(22)


class MsgDialog(gui.Dialog):
    def __init__(self, msg_header_text, msg_body_text):
        title = gui.Label(msg_header_text, cls="text_white_16")
        style = {'padding_left': 10,
                 'padding_right': 10,
                 'padding_top': 10,
                 'padding_bottom': 10}
        doc = gui.Document(style=style)
        _write_text_(doc, msg_body_text, cls="text_white_12")
        gui.Dialog.__init__(self,title,doc)



#===============================================================================
# Server Connection Menu
#===============================================================================

class ServerConnectionMenu(gui.Container):
    
    def __init__(self, app):
        gui.Container.__init__(self)
        self.app = app
        
        bg_img = gui.Image(value=\
                os.path.join('data', 'app', 'menu_bg.png'))
        self.add(bg_img, 0, 0)
        self.body = gui.Document(width=460, height=290)
        self.add(self.body, 40, 60)
        
        l = gui.Label(value='Running Server', cls='h1')
        self.body.add(l)
        self.body.br(20)
        self.body.br(1)
        
        self.console = gui.Console(560, 150)
        self.body.add(self.console)
        self.body.br(20)
        self.body.br(1)

        t = gui.Table()
        t.tr()
        self.body.add(t)
        style= {'padding_bottom': 3, 'padding_top': 3}
        td_style = {'padding_left': 30}
        
        b = gui.Button("Stop", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_stop, None)
        t.td(b, style=td_style)

        b = gui.Button("Start", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_continue, None)
        t.td(b, style=td_style)

        self.console.output.write_text('[system]#  Server is running and waiting for connections')
        self.console.handle_input = self.handle_input
        
    def handle_click_stop(self, x):
        if self.app.server_initiator != None:
            self.app.server_initiator.Close()
            self.app.server_initiator = None
        self.app.CloseMenu()
        
    def handle_input(self, text):
        self.app.server_initiator.SendChatMessage(text)
        self.console.output.write_text("[" + self.app.server_initiator.server_game_config.server_player_name + "]#  " + text)
     
    def handle_click_continue(self, x):
        if self.app.server_initiator.GetNumberOfClients() == 0:
            self.console.output.write_text('[system]#  playing alone is not fun')
            return
        self.app.server_initiator.StartGame()
        self.app.server_initiator.Close()
        self.app.server_initiator = None
        self.app.CloseMenu()
        self.app.CloseMenu()
        self.app.CloseMenu()
        
    def handle_new_player(self, player_name):
        self.console.output.write_text('[system]#  ' + player_name + " has connected ")
        self.app.server_initiator.GrantPlayer(player_name, True, "because I'm nice")
    
    def handle_closed_player(self, player_name):
        self.console.output.write_text('[system]#  ' + player_name + " has quit ")
    
    def handle_chat_msg(self, player_name, msg_text):
        self.console.output.write_text('[' + player_name + ']#  ' + msg_text)

    def handle_network_error(self, error_text):
        self.console.output.write_text('[system]#  error: '+ error_text)
        # self.app.server_initiator = None
        # self.app.CloseMenu()
        # dlg = MsgDialog("Oh no!", "network error: " + error_text)
        # dlg.open()


#===============================================================================
# Client Connection Menu
#===============================================================================

class ClientConnectionMenu(gui.Container):
    
    def __init__(self, app):
        gui.Container.__init__(self)
        self.app = app
        bg_img = gui.Image(value=\
                os.path.join('data', 'app', 'menu_bg.png'))
        self.add(bg_img, 0, 0)
        self.body = gui.Document(width=460, height=290)
        self.add(self.body, 40, 50)
        
        l = gui.Label(value='Running Client', cls='h1')
        self.body.add(l)
        self.body.br(20)
        self.body.br(1)
        
        self.console = gui.Console(560, 150)
        self.body.add(self.console)

        self.body.br(0)
        self.body.add(gui.Spacer(30, 10))
        self.body.br(0)

        t = gui.Table()
        t.tr()
        self.body.add(t)
        style= {'padding_bottom': 3, 'padding_top': 3}
        td_style = {'padding_left': 30}
        
        b = gui.Button("Stop", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_stop, None)
        t.td(b, style=td_style)

        self.console.handle_input = self.handle_input
        
    def handle_input(self, text):
        if self.app.client_initiator == None:
            self.console.output.write_text("[system]#  Connections has lost. ")
            self.console.output.write_text('[system]#  click "stop" and try again')
            return
        self.app.client_initiator.SendChatMessage(text)
        self.console.output.write_text("[" + self.app.client_initiator.client_game_config.player_name + "]#  " + text)
        
    def handle_click_stop(self, x):
        if self.app.client_initiator != None:
            self.app.client_initiator.Close()
            self.app.client_initiator = None
        self.app.CloseMenu()

    def handle_chat_msg(self, player_name, msg_text):
        self.console.output.write_text('[' + player_name + ']#  ' + msg_text)

    def handle_connection_status_changed(self, status, text):
        self.console.output.write_text("[system]#  " + text)
        if status == 3: # granted by server
            self.__load_scene__()
            self.app.client_initiator.CreateNewGame()
        elif status == 200: # game started
            self.start_game()
        elif status == -1:
            self.console.output.write_text('[system]#  click "stop" and try again')
            
    def handle_connection_error(self, text):
        if self.app.client_initiator != None:
            self.app.client_initiator.Close()
            self.app.client_initiator = None
        self.console.output.write_text('[system]#  connection closed: ' + text)
        self.console.output.write_text('[system]#  click "stop" and try again')

    def start_game(self):
        self.app.client_initiator.StartGame()
        if self.app.client_initiator != None:
            self.app.client_initiator.Close()
            self.app.client_initiator = None
            self.app.CloseMenu()
            self.app.CloseMenu()
            self.app.CloseMenu()
        else:
            self.app.CloseMenu()
            self.app.OpenMenu(self)
        
    def __load_scene__(self):
        config = self.app.client_initiator.client_game_config
        config.screen = self.app.screen
        res = LevelResource()
        res.Load(config.level_file_name)
        level = res.CreateLevel()
        self.app.scene_rep.CreateLevelRep(level, res)
        level.factory = self.app.entity_factory
        config.scene = Scene(level)
        config.scene_rep = self.app.scene_rep


#===============================================================================
# Client Menu
#===============================================================================

class ClientMenu(gui.Container):
    
    def __init__(self, app):
        gui.Container.__init__(self)
        self.app = app
        self.connection_menu = ClientConnectionMenu(app)
        bg_img = gui.Image(value=os.path.join('data', 'app', 'menu_bg.png'))
        self.add(bg_img, 0, 0)
        self.body = gui.Document(width=460, height=290)
        self.add(self.body, 70, 90)
        
        l = gui.Label(value='Starting Client', cls='h1')
        self.body.add(l)
        self.body.br(30)
        self.body.br(0)
        
        t = gui.Table()
        self.body.add(t)
        
        t.tr()

        l2 = gui.Label(value='Your name: ', cls='h3')
        self.player_name = gui.Input(value='Terry')
        t.td(l2, style={'padding_bottom': 20, 'padding_right': 100}, align=-1)
        t.td(self.player_name, style={'padding_bottom': 20}, align=1)

        t.tr()
        
        l2 = gui.Label(value='Server IP Address:', cls='h3')
        self.ip_address = gui.Input(value='127.0.0.1')
        t.td(l2, style={'padding_bottom': 20}, align=-1)
        t.td(self.ip_address, style={'padding_bottom': 20}, align=1)

        t.tr()
        
        l2 = gui.Label(value='Server Port Number:   ', cls='h3')
        self.port_number = gui.Input(value='52002')
        t.td(l2, style={'padding_bottom': 20}, align=-1)
        t.td(self.port_number, style={'padding_bottom': 20}, align=1)

        t.resize()
        self.body.br(20)
        self.body.br(20)
        
        self.body.add(gui.Spacer(30, 10))
        style= {'padding_bottom': 6, 'padding_top': 6}
        b = gui.Button("Cancel", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_cancle, None)
        self.body.add(b)
        self.body.add(gui.Spacer(30, 10))

        b = gui.Button("Connect", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_connect, None)
        self.body.add(b)
        self.body.add(gui.Spacer(10, 10))

    def handle_click_cancle(self, x):
        self.app.CloseMenu()

    def handle_click_connect(self, x):
        if not self.port_number.value.isdigit():
            MsgDialog("Smart guy, ah?", "the 'port' must be a number").open()
            return
        port = int(self.port_number.value)
        self.app.client_initiator = ClientGameInitiator(self.app.net, self.player_name.value)
        self.app.client_initiator.connection_progress_callbacks = [self.connection_menu.handle_connection_status_changed]
        self.app.client_initiator.error_callbacks = [self.connection_menu.handle_connection_error]
        self.app.client_initiator.player_chat_msg_callbacks = [self.connection_menu.handle_chat_msg]
        retVal = self.app.client_initiator.InitConnection(self.ip_address.value, port)
        if retVal == False:
            MsgDialog("Oh no...", "Sorry, failed connecting to server").open()
            return
        self.app.OpenMenu(self.connection_menu)


#===============================================================================
# Server Menu
#===============================================================================

class ServerMenu(gui.Container):
    
    def __init__(self, app):
        gui.Container.__init__(self)
        self.app = app
        self.connection_menu = ServerConnectionMenu(app)
        bg_img = gui.Image(value=os.path.join('data', 'app', 'menu_bg.png'))
        self.add(bg_img, 0, 0)
        self.body = gui.Document(width=460, height=290)
        self.add(self.body, 70, 90)
        
        l = gui.Label(value='Starting Server', cls='h1')
        self.body.add(l)
        self.body.br(30)
        self.body.br(0)
        
        t = gui.Table()
        self.body.add(t)
        
        t.tr()

        l2 = gui.Label(value='Your name: ', cls='h3')
        self.player_name = gui.Input(value='Frank')
        t.td(l2, style={'padding_bottom': 20, 'padding_right': 100}, align=-1)
        t.td(self.player_name, style={'padding_bottom': 20}, align=1)

        t.tr()
        
        l3 = gui.Label(value='Select level: ', cls='h3')
        self.level_select = gui.Select(width=213)
        path = os.path.join('data', 'levels')
        for fname in os.listdir(path):
            n1, n2 = os.path.splitext(fname)
            if n2 == '.lev':
                self.level_select.add(n1, os.path.join(path, fname))
        t.td(l3, style={'padding_bottom': 20}, align=-1)
        t.td(self.level_select, style={'padding_bottom': 20}, align=1)

        t.tr()

        l2 = gui.Label(value='Port Number:   ', cls='h3')
        self.port_number = gui.Input(value='52002')
        t.td(l2, style={'padding_bottom': 20}, align=-1)
        t.td(self.port_number, style={'padding_bottom': 20}, align=1)

        t.resize()
        self.body.br(30)
        self.body.br(0)
        
        self.body.add(gui.Spacer(30, 10))
        style= {'padding_bottom': 6, 'padding_top': 6}
        b = gui.Button("Cancel", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_cancle, None)
        self.body.add(b)
        self.body.add(gui.Spacer(30, 10))

        b = gui.Button("Continue", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_continue, None)
        self.body.add(b)
        self.body.add(gui.Spacer(10, 10))

    def handle_click_cancle(self, x):
        self.app.CloseMenu()

    def handle_click_continue(self, x):
        if self.level_select.value == None: return
        if not self.port_number.value.isdigit():
            MsgDialog("Smart guy, ah?", "the 'port' must be an integer").open()
            return
        port = int(self.port_number.value)
        if self.app.server_initiator != None:
            self.app.server_initiator.Close()
        self.app.server_initiator = ServerGameInitiator(self.app.net)
        if self.app.server_initiator.Init(port) == False:
            dlg = MsgDialog("Oh no!", "can't listen to port " + str(port))
            dlg.open()
            return
        self.__load_scene__(self.level_select.value)
        self.app.server_initiator.server_game_config.screen = self.app.screen
        self.app.server_initiator.server_game_config.server_player_name = self.player_name.value
        self.app.OpenMenu(self.connection_menu)
        self.app.server_initiator.new_player_callbacks = [self.connection_menu.handle_new_player]
        self.app.server_initiator.player_closed_callbacks = [self.connection_menu.handle_closed_player]
        self.app.server_initiator.player_chat_msg_callbacks = [self.connection_menu.handle_chat_msg]
        self.app.server_initiator.error_callbacks = [self.connection_menu.handle_network_error]
        self.app.server_initiator.CreateNewGame();

    def __load_scene__(self, level_filename):
        res = LevelResource()
        res.Load(level_filename)
        level = res.CreateLevel()
        self.app.scene_rep.CreateLevelRep(level, res)
        level.factory = self.app.entity_factory
        self.app.server_initiator.server_game_config.scene = Scene(level)
        self.app.server_initiator.server_game_config.scene_rep = self.app.scene_rep
        self.app.server_initiator.server_game_config.level_file_name = level_filename
        self.app.server_initiator.server_game_config.level_oid = res.id


#===============================================================================
# Network Game Menu
#===============================================================================

class NetworkGameMenu(gui.Container):
    
    def __init__(self, app):
        gui.Container.__init__(self)
        self.app = app
        self.server_menu = ServerMenu(app)
        self.client_menu = ClientMenu(app)
        bg_img = gui.Image(value=os.path.join('data', 'app', 'menu_bg.png'))
        self.add(bg_img, 0, 0)
        self.body = gui.Document(width=460, height=290)
        self.add(self.body, 70, 120)
        
        l = gui.Label(value='Starting Network Game', cls='h1')
        self.body.add(l)
        self.body.br(20)
        self.body.br(20)
        
        txt = "What would you want to be?"
        _write_text_(self.body, txt)
        self.body.br(20)
        self.body.br(20)

        style= {'padding_bottom': 6, 'padding_top': 6}
        b = gui.Button("Cancel", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_cancle, None)
        self.body.add(b)
        self.body.add(gui.Spacer(10, 10))

        b = gui.Button("Client", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_start_client, None)
        self.body.add(b)
        self.body.add(gui.Spacer(10, 10))

        b = gui.Button("Server", style=style, width=100)
        b.connect(gui.CLICK, self.handle_click_start_server, None)
        self.body.add(b)
        self.body.add(gui.Spacer(10, 10))

    def handle_click_cancle(self, x):
        self.app.CloseMenu()

    def handle_click_start_server(self, x):
        self.app.OpenMenu(self.server_menu)

    def handle_click_start_client(self, x):
        self.app.OpenMenu(self.client_menu)


#===============================================================================
# Game Menu
#===============================================================================

class GameMenu(gui.Container):
    
    def __init__(self, app):
        self.config = GameConfig()
        gui.Container.__init__(self)
        self.app = app
        bg_img = gui.Image(value=os.path.join('data', 'app', 'menu_bg.png'))
        self.add(bg_img, 0, 0)
        self.body = gui.Container(width=460, height=290)
        self.add(self.body, 80, 60)
        
        t = gui.Table(width=440)
        self.body.add(t, 10, 20)

        t.tr()

        l0 = gui.Label(value='2 Players Game', cls='h1')
        t.td(l0, colspan=2, style={'padding_bottom': 30}, align=-1)
        
        t.tr()
        
        l1 = gui.Label(value='Left player name:   ', cls='h3')
        self.name1 = gui.Input(value='Ralf')
        t.td(l1, style={'padding_bottom': 20}, align=-1)
        t.td(self.name1, style={'padding_bottom': 20}, align=1)
        
        t.tr()
        
        l2 = gui.Label(value='Right player name:   ', cls='h3')
        self.name2 = gui.Input(value='Chiko')
        t.td(l2, style={'padding_bottom': 20}, align=-1)
        t.td(self.name2, style={'padding_bottom': 20}, align=1)

        t.tr()
        
        l3 = gui.Label(value='Select level: ', cls='h3')
        self.level_select = gui.Select(width=213)
        path = os.path.join('data', 'levels')
        for fname in os.listdir(path):
            n1, n2 = os.path.splitext(fname)
            if n2 == '.lev':
                self.level_select.add(n1, os.path.join(path, fname))
        t.td(l3, style={'padding_bottom': 20}, align=-1)
        t.td(self.level_select, style={'padding_bottom': 20}, align=1)

        t.tr()
        
        style= {'padding_bottom': 6, 'padding_top': 6}
        b = gui.Button("Cancel", style=style, width=120)
        b.connect(gui.CLICK, self.handle_click_cancle, None)
        t.td(b, style={'padding_top': 20}, align=0)
        b = gui.Button("Start", style=style, width=120)
        b.connect(gui.CLICK, self.handle_click_start, None)
        t.td(b, style={'padding_top': 20}, align=0)

    def handle_click_cancle(self, x):
        self.app.CloseMenu()

    def handle_click_start(self, x):
        if self.level_select.value == None:
            return
        print "loading level"
        self.__load_scene__(self.level_select.value)
        self.config.screen = self.app.screen
        self.config.scene_rep = self.app.scene_rep
        self.config.player1_name = self.name1.value
        self.config.player2_name = self.name2.value
        game = Game(self.config)
        game.Run()
        self.app.CloseMenu()

    def __load_scene__(self, level_filename):
        res = LevelResource()
        res.Load(level_filename)
        level = res.CreateLevel()
        self.app.scene_rep.CreateLevelRep(level, res)
        level.factory = self.app.entity_factory
        self.config.scene = Scene(level)
        

#===============================================================================
# Help Menu
#===============================================================================

class HelpMenu(gui.Container):
    
    page1 = """
    <h1> FLAMEingo Help </h1> 
    <h2> Welcome! </h2> 
    <br>
    This is a game for two players. <br>
    This means that if you don't have little brothers or geek <br>
    friends, you won't enjoy it. <br>
    <br>
    The aim of this game is to blow out your friend to pieces <br>
    using a bazooka.
    <br>
    """
    
    page2 = """
    <h2> Bazooka??? </h2> 
    <br>
    Yes, bazooka. <br>
    You are the green little soldier and you ride on your evil <br>
    flamingo. <br>
    But be ware. Flying is not that easy as it looks... <br>
    <br>
    <img src="data/app/help/this_is_you.png">
    """

    page3 = """
    <h2> Flying a Flamingo </h2> 
    <br>
    Your most important key is the 'jump' key. <br>
    You hit it to jump, and then you hit it again, and again. <br>
    That's the way you fly. <br>
    But watch out! You can't fly for ever. <br>
    Your fat ass is a heavy burden for the poor bird, and it gets <br>
    tired. <br>
    The 'flight' bar at the bottom of the screen tells you how tired <br>
    your flamingo is. <br>
    <br>
    <img src="data/app/help/the_flight_bar.png">
    """
    
    page4 = """
      <h2> The Keys </h2>
      <table>
      <tr>
          <td> <b> Left Player </b> </td>
          <td> <b> Right Player </b> </td>
          <td> <b> Action </b> </td>
      </tr>
      <tr>
          <td> a </td>
          <td> LEFT </td>
          <td> move left </td>
      </tr>
      <tr>
          <td> d </td>
          <td> RIGHT </td>
          <td> move right </td>
      </tr>
      <tr>
          <td> w </td>
          <td> UP </td>
          <td> aim up </td>
      </tr>
      <tr>
          <td> x </td>
          <td> DOWN </td>
          <td> aim down </td>
      </tr>
      <tr>
          <td> q </td>
          <td> . (dot) </td>
          <td> jump </td>
      </tr>
      <tr>
          <td> TAB </td>
          <td> / (slash) </td>
          <td> fire </td>
      </tr>
      </table>
    <br>
    <br>
    You need to hit the 'left' and 'right' keys several times to gain <br>
    some speed.
    """
    
    page5 = """
    <h2> Tip of the Day </h2>
    <br>
    Don't fall through the bottom of the world! <br>
    You'll lose a life immediatlly.
    """

    page6 = """
    <h2> Credits </h2>
    <br>
    <table width=450> <tr> <td align="center">
        <table>
          <tr>
              <td> <img src="data/app/help/small_gal.png"> </td>
              <td valign="center"> 
              Gal Koren <br>
              Animation, programming and everything else.
              </td>
           </tr>
           <tr>
              <td colspan="2"> 
                  <br><br><br>
                  FLAMEingo web site: <br>
                  http://flameingo.sf.net 
              </td>
           </tr>
        </table>
    </td></tr></table>
    """
    pages_txt = [page1, page2, page3, page4, page5, page6]
    
    def __init__(self, app):
        gui.Container.__init__(self)
        self.app = app
        bg_img = gui.Image(value=os.path.join('data', 'app', 'menu_bg.png'))
        self.add(bg_img, 0, 0)
        self.body = gui.Container(width=460, height=290)
        self.add(self.body, 80, 50)
        self.index = 0
        
        self.pages = []
        for txt in HelpMenu.pages_txt:
            self.pages.append( html.HTML(txt) )
        
        b = gui.Button("Prev")
        b.connect(gui.CLICK, self.handle_click_prev, None)
        self.add(b, 100, 330)

        b = gui.Button("Next")
        b.connect(gui.CLICK, self.handle_click_next, None)
        self.add(b, 220, 330)

        b = gui.Button("Close")
        b.connect(gui.CLICK, self.handle_click_close, None)
        self.add(b, 340, 330)
        self.set_page(self.pages[self.index])
    
    def restart(self):
        self.index = 0
        self.set_page(self.pages[self.index])
        
    def handle_click_close(self, x):
        self.app.CloseMenu()
        
    def handle_click_next(self, x):
        if self.index < len(self.pages) - 1:
            self.index += 1
        self.set_page(self.pages[self.index])

    def handle_click_prev(self, x):
        if self.index > 0:
            self.index -= 1
        self.set_page(self.pages[self.index])
        
    def set_page(self, page):
        while len(self.body.widgets) > 0:
            self.body.remove(self.body.widgets[0])
        self.body.add(page, 10, 0)
        self.body.resize()
        self.repaint()
        self.focus()
        

#===============================================================================
# Main Menu
#===============================================================================

class MainMenu(gui.Container):
    def __init__(self, app):
        gui.Container.__init__(self)
        self.app = app
        self.help_menu = HelpMenu(app)
        self.game_menu = GameMenu(app)
        self.network_game_menu = NetworkGameMenu(app)
        value = os.path.join('data', 'app', 'main_bg.png')
        self.add(gui.Image(value), 0, 0)
        self.add(gui.Label(value="Version 2.3", cls="text_white_12"), 550, 50)
        
        td_style = {'padding_top': 5}
        button_style= {'padding_bottom': 9, 'padding_top': 9}
        
        t1 = gui.Table()
        t1.tr()
        b = gui.Button(value="2 Players Game", style=button_style, width=200)
        b.connect(gui.CLICK, self.handle_click_start_game, None)
        t1.td(b, style=td_style)
        t1.tr()
        b = gui.Button(value="Network Game", style=button_style, width=200)
        b.connect(gui.CLICK, self.handle_click_start_network_game, None)
        t1.td(b, style=td_style)
        t1.tr()
        b = gui.Button(value="Help", style=button_style, width=200)
        b.connect(gui.CLICK, self.handle_click_help, None)
        t1.td(b, style=td_style)
        t1.tr()
        b = gui.Button(value="Exit", style=button_style, width=200)
        b.connect(gui.CLICK, self.handle_click_quit, None)
        t1.td(b, style=td_style)
        self.add(t1, 220, 150)
        
    def handle_click_start_game(self, x):
        self.app.OpenMenu(self.game_menu)

    def handle_click_start_network_game(self, x):
        self.app.OpenMenu(self.network_game_menu)

    def handle_click_help(self, x):
        self.app.OpenMenu(self.help_menu)
        
    def handle_click_quit(self, x):
        self.app.Quit()
        

#===============================================================================
# The Application
#===============================================================================

class App:
    """
    This class is responsible for managing the menus, and for
    loading the game resources.
    
    The menu model is a stack, so by closing a menu, the application
    pops it from the stack and shows the one before it.
    """
    
    def set_screen(self):
        self.screen = pygame.display.set_mode((640, 480), self.display_flags)

    def __init__(self, display_flags=0):
        self.game_display_flags = display_flags | pygame.HWSURFACE | pygame.DOUBLEBUF
        self.display_flags = display_flags
        self.set_screen()
        self.scene_rep = SceneRep()
        self.entity_factory = EntityFactory()
        self.server_initiator = None
        self.client_initiator = None
        print "loading resources"
        self.__load_resources__()
        print "initializing gui"
        self.__init_gui__()
        self.net = FlameingoIPC()
        self.app.permanent_callbacks.append(self.net.Poll)
    
    def __load_resources__(self):
        file_names = [
             os.path.join('data', 'entity_resources', 'white_dot', 'dot.cfg'), 
             os.path.join('data', 'entity_resources', 'yellow_dot', 'dot.cfg'), 
             os.path.join('data', 'entity_resources', 'ammo_box', 'box.cfg'), 
             os.path.join('data', 'entity_resources', 'medic_box', 'box.cfg'), 
             os.path.join('data', 'entity_resources', 'cannon', 'cannon.cfg'), 
             os.path.join('data', 'entity_resources', 'cannon', 'parts.cfg'), 
             os.path.join('data', 'entity_resources', 'bullets', 'bullets.cfg'), 
#             os.path.join('data', 'entity_resources', 'gaus_gun', 'gaus.cfg'), 
             os.path.join('data', 'entity_resources', 'flamingo', 'flamingo.cfg'), 
             os.path.join('data', 'entity_resources', 'flamingo', 'bodyparts.cfg'), 
#             os.path.join('data', 'entity_resources', 'bazooka', 'parts.cfg'), 
             os.path.join('data', 'entity_resources', 'flame1', 'flame.cfg'), 
             os.path.join('data', 'entity_resources', 'flame2', 'flame.cfg'), 
             os.path.join('data', 'entity_resources', 'flame3', 'flame.cfg'), 
             os.path.join('data', 'entity_resources', 'flame4', 'flame.cfg'), 
             #os.path.join('data', 'entity_resources', 'zimm', 'zimm.cfg'), 
             os.path.join('data', 'entity_resources', 'bazooka', 'bazooka.cfg') ]
        for fname in file_names:
            res = EntityResource()
            res.Load(fname)
            classObj = globals().get(res.className)
            if classObj == None:
                msg = 'error while loading resource ' + fname + \
                      ': cannot find class ' + res.className + \
                      ' within the global namespace'
                raise Exception(msg)
            self.entity_factory.AddPrototype(classObj(res))
            self.scene_rep.CreateEntityRep(res)
        
    def __init_gui__(self):
        bg = pygame.Surface((20, 20))
        bg.fill((0, 0, 0))
        self.main = gui.Container(background=bg)
        self.menu_stack = []
        self.app = gui.AppWithCallbacks()
        self.app.connect(gui.QUIT, self.app.quit, None)
        self.OpenMenu(MainMenu(self))

    def OpenMenu(self, menu):
        if len(self.menu_stack) > 0:
            last_menu = self.menu_stack.pop()
            self.menu_stack.append(last_menu)
            self.main.remove(last_menu)
            last_menu.blur()
        self.menu_stack.append(menu)
        self.main.add(menu, 0, 0)
        self.Repaint()

    def CloseMenu(self):
        last_menu = self.menu_stack.pop()
        self.main.remove(last_menu)
        last_menu.blur()
        menu = self.menu_stack.pop()
        self.menu_stack.append(menu)
        self.main.add(menu, 0, 0)
        self.Repaint()
        
    def Repaint(self):
        self.main.resize()
        self.main.repaint()
        self.main.focus()
    
    def Run(self):
        if 'SW_DRIVER' in sys.argv:
            self.game_display_flags = self.display_flags
        self.app.run(widget=self.main, screen=self.screen)

    def Quit(self):
        self.app.quit()

