"""
Charactors of the game are defined here.

A Charactor is a controller of an entity.
"""


from defines import *
import os, sys, random, math
from scene import *
from graphics import *
from m_Dashboard import LoadableDashboardItem
from m_EntityTypes import *


#===============================================================================
#  FlamingoChar
#===============================================================================


class FlamingoChar(IEntityObserver):
    """
    Flamingo Charactor.
    Controls a man riding on a flamingo.
    """
    
    class BodyPartsObserver(IEntityObserver):

        def __init__(self, sceneController):
            self.sceneController = sceneController
        
        def NotifyCollisionWithEntity(self, entity1, entity2):
            pass
            
        def NotifyCollisionWithLand(self, entity_id, x, y, side):
            self.sceneController.DeleteEntity(entity_id)

    
    STATE_STANDING  = 0
    STATE_FLYING    = 1

    def __init__(self, player_id, gameController, sceneController, entity_id=-1):
        self.player_id = player_id
        self.sceneController = sceneController
        self.gameController = gameController
        if entity_id == -1:
            self.entity_id = self.sceneController.CreateNewEntity(TYPE_FLAMINGO, 0, 0, 0, 0, 0)
        else:
            self.entity_id = entity_id
        self.type = TYPE_FLAMINGO
        self.flight = LoadableDashboardItem()
        self.flight.loading_rate = 22.5
        self.flight.value = 100
        self.sceneController.AttachObserverToEntity(self.entity_id, self)
        self.body_parts_observer = FlamingoChar.BodyPartsObserver(self.sceneController)
        self.dir = DIR_RIGHT
        self.state = self.STATE_FLYING
        self.aim_angle = 0
        self.UpdateEntityState()
        
    def UpdateEntityState(self):
        self.GetEntityState()
        i = self.aim_angle
        if i > 90: i = 180 - i
        i += 80
        man_anim_index = int(i / 25)
        self.sceneController.SetEntityState(self.entity_id, man_anim_index + self.dir * 14 + self.state * 7)
        
    def NotifyCollisionWithEntity(self, entity1, entity2):
        pass

    def NotifyCollisionWithLand(self, entity_id, x, y, side):
        if side == SIDE_BOTTOM:
            if x == -1 and y == -1: # reached the bottom of the level
                x0, y0 = self.sceneController.GetEntityPos(entity_id)
                self.sceneController.SetEntityVelocity(entity_id, 0, -30)
                self.CreateBodyparts(x0, y0 -2, 10)
                self.gameController.DecreasePlayerHealth(self.player_id, 101)
                return
            self.state = self.STATE_STANDING

    def GetEntityState(self):
        state = self.sceneController.GetEntityState(self.entity_id)
        # self.dir = state / 14
        self.state = (state / 7) % 2
        
    def Advance(self, time):
        return
    
    def MoveLeft(self):
        vx, vy = self.sceneController.GetEntityVelocity(self.entity_id)
        self.GetEntityState()
        if self.state is self.STATE_FLYING:
            vx -= 30
            if vx < -160: vx = -160
        else:
            if self.dir == DIR_LEFT:
                vy = -50 # -20 * random.random() - 50
                vx = -50 # -20 * random.random() - 50
        self.dir = DIR_LEFT
        self.UpdateEntityState()
        self.sceneController.SetEntityVelocity(self.entity_id, vx, vy)
        
    def MoveRight(self):
        self.GetEntityState()
        vx, vy = self.sceneController.GetEntityVelocity(self.entity_id)
        if self.state is self.STATE_FLYING:
            vx += 30
            if vx > 160: vx = 160
        else:
            if self.dir == DIR_RIGHT:
                vy = -50 # -20 * random.random() - 50
                vx = 50 #  20 * random.random() + 50
        self.dir = DIR_RIGHT
        self.UpdateEntityState()
        self.sceneController.SetEntityVelocity(self.entity_id, vx, vy)
            
    def MoveJump(self):
        self.GetEntityState()
        if self.flight.value < 20:
            return
        vx, vy = self.sceneController.GetEntityVelocity(self.entity_id)
        self.flight.value -= 20
        self.flight.loading = True
        vy -= 120
        if vy < -130:
            vy = -130
        self.UpdateEntityState()
        self.sceneController.SetEntityVelocity(self.entity_id, vx, vy)
            
    def GetDir(self):
        return self.dir

    def DecreaseHealth(self, damage):
        x, y = self.sceneController.GetEntityPosCenter(self.entity_id)
        if self.gameController.GetPlayerHealth(self.player_id) <= damage:
            damage = 45
        self.CreateBodyparts(x, y - 3, int(damage/3) + 1)
        self.gameController.DecreasePlayerHealth(self.player_id, damage)

    def NotifyExplosion(self, entity_id, vx, vy, power):
        self.DecreaseHealth(power)

    def CreateBodyparts(self, x, y, n):
        for i in xrange(0, n):
            state = int(random.uniform(0, 3))
            vx, vy = self.sceneController.GetEntityVelocity(self.entity_id)
            vx = 6 * (random.uniform(0, 40) - 20) + vx
            vy = -20 - random.uniform(0, 40) + vy
            bp = self.sceneController.CreateNewEntity(TYPE_FLAMINGO_BODY_PARTS, x, y, vx, vy, state)
            self.sceneController.AttachObserverToEntity(bp, self.body_parts_observer)
        








