from m_Players import *
from pygame.locals import *


KEYBOARD_LEFT     = 0
KEYBOARD_RIGHT    = 1
KEYBOARD_UP       = 2
KEYBOARD_DOWN     = 3
KEYBOARD_JUMP     = 4
KEYBOARD_FIRE     = 5


##
## keyboard keys for left player
##
KEY_MAPPING_LEFT =  { KEYBOARD_LEFT    : K_a,
                      KEYBOARD_RIGHT   : K_d,
                      KEYBOARD_UP      : K_w,
                      KEYBOARD_DOWN    : K_x,
                      KEYBOARD_JUMP    : K_q,
                      KEYBOARD_FIRE    : K_TAB }

##
## keyboard keys for right player
##
KEY_MAPPING_RIGHT = { KEYBOARD_LEFT    : K_LEFT,
                      KEYBOARD_RIGHT   : K_RIGHT,
                      KEYBOARD_UP      : K_UP,
                      KEYBOARD_DOWN    : K_DOWN,
                      KEYBOARD_JUMP    : K_PERIOD,
                      KEYBOARD_FIRE    : K_SLASH }



class LocalGameController:

    def __init__(self, game):
        self.game = game

    def CreateNewPlayer(self, player_name, player_id=-1, entity_id=-1):
        return self.game.CreateNewPlayer(player_name, player_id, entity_id)

    def DeletePlayer(self, id):
        self.game.DeletePlayer(id)

    def DecreasePlayerHealth(self, id, value):
        player = self.game.playerIdMap[id]
        return player.DecreaseHealth(value)

    def IncreasePlayerHealth(self, id, value):
        if not self.IsPlayerAlive(id): return
        health = self.game.playerIdMap[id].dashboard.health
        health.value = min(100, health.value + value)
    
    def PlayerMoveLeft(self, id):
        if not self.IsPlayerAlive(id): return
        player = self.game.playerIdMap[id]
        player.MoveLeft()

    def PlayerMoveRight(self, id):
        if not self.IsPlayerAlive(id): return
        player = self.game.playerIdMap[id]
        player.MoveRight()
    
    def PlayerMoveJump(self, id):
        if not self.IsPlayerAlive(id): return
        player = self.game.playerIdMap[id]
        player.MoveJump()

    def PlayerShoot(self, id):
        if not self.IsPlayerAlive(id): return
        player = self.game.playerIdMap[id]
        player.Shoot()

    def PlayerAimUp(self, id, milliseconds):
        if not self.IsPlayerAlive(id): return
        player = self.game.playerIdMap[id]
        player.AimUp(milliseconds / 7.0)
        
    def PlayerAimDown(self, id, milliseconds):
        if not self.IsPlayerAlive(id): return
        player = self.game.playerIdMap[id]
        player.AimDown(milliseconds / 7.0)

    def GetAllPlayerIds(self):
        return self.game.playerIdMap.keys()

    def GetPlayerEntityId(self, player_id):
        if self.game.playerIdMap.has_key(player_id):
            return self.game.playerIdMap[player_id].charactor.entity_id
        return -1
    
    def IsPlayerAlive(self, player_id):
        if (self.game.playerIdMap.has_key(player_id) and 
            self.game.sceneController.EntityExists(self.game.playerIdMap[player_id].charactor.entity_id) ):
            return True
        return False
    
    def GetPlayerAimXY(self, player_id):
        aim_marker = self.game.playerIdMap[player_id].aim_marker
        return (aim_marker.x, aim_marker.y)
            
    def GetPlayerName(self, player_id):
        return self.game.playerIdMap[player_id].name
        
    def GetPlayerLives(self, player_id):
        return self.game.playerIdMap[player_id].dashboard.lives.value

    def GetPlayerHealth(self, player_id):
        return self.game.playerIdMap[player_id].dashboard.health.value

    def GetPlayerFlight(self, player_id):
        return self.game.playerIdMap[player_id].dashboard.flight.value
    
    def SetPlayerFlight(self, player_id, value):
        self.game.playerIdMap[player_id].dashboard.flight.value = value
    
    def PlayerExists(self, player_id):
        return self.game.playerIdMap.has_key(player_id)
    
    def GetPlayerByEntityId(self, id):
        return self.game.GetPlayerByEntityId(id)

    def SetTimer(self, timeout, func, args):
        self.game.schedualer.Post(timeout, func, args)




class PlayerKeyboard:

    def __init__(self, pKeyMapping, pGameController, pPlayerId):
        self.gameController = pGameController
        self.playerId = pPlayerId
        self.keyToFunc = { pKeyMapping[KEYBOARD_LEFT]  : self._handle_left_,
                           pKeyMapping[KEYBOARD_RIGHT] : self._handle_right_,
                           # pKeyMapping[KEYBOARD_UP]    : self._handle_up_,
                           # pKeyMapping[KEYBOARD_DOWN]  : self._handle_down_,
                           pKeyMapping[KEYBOARD_JUMP]  : self._handle_jump_,
                           pKeyMapping[KEYBOARD_FIRE]  : self._handle_fire_ }
        self.keyUp = pKeyMapping[KEYBOARD_UP]
        self.keyDown = pKeyMapping[KEYBOARD_DOWN]

    def handleClick(self, key):
        if key in self.keyToFunc:
            self.keyToFunc[key]()

    def handlePressedKeys(self, pressedKeys, milliseconds):
        """
        for keys being held for amount of milliseconds
        pressedKeys is a map from key to boolean value (presed/not-pressed)
        as returned by pygame.key.get_pressed()
        """
        if pressedKeys[self.keyUp]  : self._handle_up_(milliseconds)
        if pressedKeys[self.keyDown]: self._handle_down_(milliseconds)

    def _handle_left_(self):
        self.gameController.PlayerMoveLeft(self.playerId)

    def _handle_right_(self):
        self.gameController.PlayerMoveRight(self.playerId)

    def _handle_up_(self, milliseconds):
        self.gameController.PlayerAimUp(self.playerId, milliseconds)

    def _handle_down_(self, milliseconds):
        self.gameController.PlayerAimDown(self.playerId, milliseconds)

    def _handle_jump_(self):
        self.gameController.PlayerMoveJump(self.playerId)

    def _handle_fire_(self):
        self.gameController.PlayerShoot(self.playerId)


