import os
from graphics import TextBar, ProgBar
import pygame


class DashboardItem:
    
    def __init__(self, value=None):
        self.value = value
        

        
class LoadableDashboardItem(DashboardItem):
    
    def __init__(self, value=None):
        DashboardItem.__init__(self, value)
        self.loading = False
        self.loading_rate = 0
        self.min_value = 0
        self.max_value = 100
        
    def Advance(self, time):
        if self.loading:
            self.value += float(time * self.loading_rate) / 1000
            if self.value > self.max_value:
                self.value = self.max_value
                self.loading = False



class Dashboard:
    
    def __init__(self):
        self.player_name = ""
        self.health = DashboardItem(value=100)
        self.lives = DashboardItem(value=7)
        self.weapon_name = None  # DashboardItem()
        self.ammo = None         # LoadableDashboardItem()
        self.flight = None       # LoadableDashboardItem()
        
    def Advance(self, time):
        self.ammo.Advance(time)
        self.flight.Advance(time)



class DashboardRep:
    
    def __init__(self, dashboard, output_surface):
        self.dashboard = dashboard
        self.output_surface = output_surface
        
        _rect = [0, 0, 120, 5]
        _path = os.path.join('data', 'game_widgets', 'prog_bar.png')
        self.flight_bar = ProgBar(output_surface.subsurface(_rect), _path)
        
        _rect = [0, 15, 150, 5]
        self.health_bar = ProgBar(output_surface.subsurface(_rect), _path)

        _rect = [0, 30, 90, 5]
        self.ammo_bar = ProgBar(output_surface.subsurface(_rect), _path)
        
        _rect = [155, 12, 50, 20]
        self.lives_bar = TextBar(output_surface.subsurface(_rect))

        _rect = [95, 27, 80, 20]
        self.weapon_name_bar = TextBar(output_surface.subsurface(_rect))
        
    def Repaint(self):
        self.flight_bar.value = self.dashboard.flight.value
        self.health_bar.value = self.dashboard.health.value
        self.ammo_bar.value = self.dashboard.ammo.value
        self.weapon_name_bar.value = self.dashboard.weapon_name.value
        self.lives_bar.value = "Lives: " + str(self.dashboard.lives.value)
        
        self.flight_bar.Repaint()
        self.health_bar.Repaint()
        self.ammo_bar.Repaint()
        self.weapon_name_bar.Repaint()
        self.lives_bar.Repaint()
        
    def Paint(self):
        self.flight_bar.value = self.dashboard.flight.value
        self.health_bar.value = self.dashboard.health.value
        self.ammo_bar.value = self.dashboard.ammo.value
        self.weapon_name_bar.value = self.dashboard.weapon_name.value
        self.lives_bar.value = "Lives: " + str(self.dashboard.lives.value)
        
        self.flight_bar.Paint()
        self.health_bar.Paint()
        self.ammo_bar.Paint()
        self.weapon_name_bar.Paint()
        self.lives_bar.Paint()



class RemotePlayerDashboardRep:
    
    def __init__(self, output_surface):
        self.output_surface = output_surface
        
        _rect = [0, 0, 150, 15]
        self.name_bar = TextBar(output_surface.subsurface(_rect))

        _rect = [0, 20, 150, 5]
        _path = os.path.join('data', 'game_widgets', 'prog_bar.png')
        self.health_bar = ProgBar(output_surface.subsurface(_rect), _path)
        
        _rect = [0, 32, 150, 15]
        self.lives_bar = TextBar(output_surface.subsurface(_rect))

    def Repaint(self, dashboard):
        self.name_bar.value = dashboard.player_name
        self.health_bar.value = dashboard.health.value
        self.lives_bar.value = "Lives: " + str(dashboard.lives.value)
        
        self.health_bar.Repaint()
        self.lives_bar.Repaint()
        self.name_bar.Repaint()
        
    def Paint(self, dashboard):
        self.name_bar.value = dashboard.player_name
        self.health_bar.value = dashboard.health.value
        self.lives_bar.value = "Lives: " + str(dashboard.lives.value)
        
        self.health_bar.Paint()
        self.lives_bar.Paint()
        self.name_bar.Paint()



class RemotePlayerDashboardListRep:

    MAX_DASHBOARDS = 5
    
    def __init__(self, dashboard_list, output_surface):
        self.dashboard_list = dashboard_list
        self.output_surface = output_surface
        self.dash_reps = []
        self.list_size = 0
        i = 0
        while i < self.MAX_DASHBOARDS:
            d = RemotePlayerDashboardRep(output_surface.subsurface((0, i * 70, 160, 65)))
            self.dash_reps.append(d)
            i += 1
        self.black_rect = pygame.Surface((160, 70))
        self.black_rect.fill((0, 0, 0))

    def _check_if_list_became_smaller_(self):
        if len(self.dashboard_list) < self.list_size:
            i = self.list_size - 1
            while i >= len(self.dashboard_list):
                self.output_surface.blit(self.black_rect, (0, i * 70))
                i -= 1
        self.list_size = len(self.dashboard_list)

    def Repaint(self):
        i = 0
        while i < min(len(self.dashboard_list), self.MAX_DASHBOARDS):
            rep = self.dash_reps[i]
            dash = self.dashboard_list[i]
            i += 1
            rep.Repaint(dash)
        self._check_if_list_became_smaller_()

    def Paint(self):
        i = 0
        while i < min(len(self.dashboard_list), self.MAX_DASHBOARDS):
            rep = self.dash_reps[i]
            dash = self.dashboard_list[i]
            i += 1
            rep.Paint(dash)
        self._check_if_list_became_smaller_()



