import sys, math
import os, time, random
from scene import *
from weapons import WEAPON_CLASSES
from m_Charactors import *
from m_Sprites import *
from m_Players import *
from m_EntityTypes import *
from m_MedicBox import *
from m_EventSchedualer import *
from m_Dashboard import DashboardRep
from graphics import Sprite, SpriteRes, ReallyStopGameDialog
from m_Controllers import *
import pygame
from pygame.locals import *


FRAME_EVT = pygame.locals.USEREVENT + 7


    
class BaseGame:
    
    def __init__(self):
        self.playerIdMap = {} # from player id to player object
        self.idCounter = 0
        self.sceneController = None
        self.gameController = None
        self.keyboards = []

    def CreateNewPlayer(self, player_name, player_id=-1, entity_id=-1):
        if player_id == -1:
            player_id = self.idCounter
            self.idCounter += 1
        player = Player(player_id, self, player_name)
        self.playerIdMap[player_id] = player
        charactor = FlamingoChar(player_id, self.gameController, self.sceneController, entity_id)
        player.SetCharactor(charactor)
        return player_id

    def DeletePlayer(self, id):
        player = self.playerIdMap.pop(id)
        self.sceneController.DeleteEntity(player.charactor.entity_id)
        for keyboard in self.keyboards:
            if keyboard.playerId == id: self.keyboards.remove(keyboard)
        
    def GetPlayerByEntityId(self, entity_id):
        for player in self.playerIdMap.values():
            if player.charactor.entity_id == entity_id:
                return player.id
        return None

    def GetAllPlayerIDs(self):
        return self.playerIdMap.keys()




class GameConfig:
    def __init__(self):
        self.screen = None
        self.scene = None
        self.scene_rep = None
        self.player1_name = ''
        self.player2_name = ''


        
class Game(BaseGame):
    
    FRAME_DELAY = 33.0 # milliseconds, ofcourse

        
    def NotifyPlayerKilled(self, player):
        output_surface = None
        if player == self.player1:
            output_surface = self.camera1.output_surface
        elif player == self.player2:
            output_surface = self.camera2.output_surface
        res = self.sprite_resources[SPRITE_ZAPPA]
        sprite = SpriteZappa(output_surface, res)
        self.sprites.append(sprite)

    def NotifyPlayerLoss(self, player):
        self.DeletePlayer(player.id)
        self.schedualer.Post(5.5, self.Quit)
        # create SpriteLoser
        output_surface = None
        if player == self.player1:
            output_surface = self.camera1.output_surface
        elif player == self.player2:
            output_surface = self.camera2.output_surface
        res = self.sprite_resources[SPRITE_LOSER]
        self.sprites.append(SpriteLoser(output_surface, res))        
        
    def __init__(self, game_config):
        """
        @param game_config: a GameConfig object
        @param scene: a Scene object
        @param scene_rep: a SceneRep (representation) object
        """
        BaseGame.__init__(self)
        self.config = game_config
        self.screen = game_config.screen
        self.scene = game_config.scene
        self.scene_rep = game_config.scene_rep
        self.sceneController = LocalSceneController(self.scene)
        self.gameController = LocalGameController(self)
        self.sprites = []
        self.quit = False
        self.schedualer = Schedualer()
        self.schedualer.StartTimer()
        ##
        ## background images
        ##
        self.img_background = pygame.image.load(
            os.path.join('data', 'app', 'local_game_bg.png')).convert()
            
        ##
        ## create players
        ##
        self.gameController.CreateNewPlayer(game_config.player1_name)
        self.gameController.CreateNewPlayer(game_config.player2_name)
        self.player1, self.player2 = self.playerIdMap.values()
        
        self.player1.MoveRight()
        self.player2.MoveLeft()
        self.players = [self.player1, self.player2]

        ##
        ## create keyboards
        ##
        self.keyboard1 = PlayerKeyboard(KEY_MAPPING_LEFT,
                                        self.gameController,
                                        self.player1.id)

        self.keyboard2 = PlayerKeyboard(KEY_MAPPING_RIGHT,
                                        self.gameController,
                                        self.player2.id)

        self.keyboards = [self.keyboard1, self.keyboard2]
        
        ##
        ## create DashboardRep objects
        ##
        self.dashboard_rep1 = DashboardRep(
                 self.player1.dashboard, 
                 self.screen.subsurface([20, 415, 219, 64]) )
        self.dashboard_rep2 = DashboardRep(
                 self.player2.dashboard, 
                 self.screen.subsurface([420, 415, 219, 64]) )
        ##
        ## create camera's
        ##
        self.camera1 = PlayerCamera(
                self.player1.id,
                self,
                self.screen.subsurface([0, 41, 319, 349]) )
        self.camera2 = PlayerCamera(
                self.player2.id,
                self,
                self.screen.subsurface([321, 41, 319, 349]) )
        ##
        ## define collisions to be detected
        ##
        detector = self.scene.level.collidetector
        detector.DefineCollisionCheck(TYPE_BAZOOKA, TYPE_FLAMINGO)
        detector.DefineCollisionCheck(TYPE_MEDIC_BOX, TYPE_FLAMINGO)


        ##
        ## load sprite resources
        ##
        self.sprite_resources = {}
        paths = [ os.path.join('data', 'sprites', 'zappa', 'zappa.cfg'),
                  os.path.join('data', 'sprites', 'cleese', 'cleese.cfg'),
                  os.path.join('data', 'sprites', 'foot', 'foot.cfg'),
                  os.path.join('data', 'sprites', 'loser', 'loser.cfg') ]
        for path in paths:
            res = SpriteRes()
            res.Load(path)
            self.sprite_resources[res.type] = res
        ##
        ## declare the frame event
        ##
        pygame.event.Event(FRAME_EVT)

        ##
        ## initialize dialogs
        ##
        self.reallyQuitDialog = ReallyStopGameDialog()
        self.reallyQuitDialog.value_returned_callbacks.append(self.HandleReallyQuitDialogValue)
        self.dialogs = [self.reallyQuitDialog]

        ##
        ## almost finished...
        ##
        self._post_random_sprite()
        self._post_create_medic_box()


    def HandleReallyQuitDialogValue(self, dlg, val):
        dlg.Close()
        self.DrawBackground()
        if val == True: self.Quit()

    def _post_random_sprite(self):
        next_time = random.randint(8, 90)
        self.schedualer.Post(next_time, self._create_random_sprite)
        
    def _create_random_sprite(self):
        output_surface = random.choice( [self.camera1.output_surface,
                                         self.camera2.output_surface ])
        res = self.sprite_resources[SPRITE_CLEESE]
        self.sprites.append( SpriteCleese(output_surface, res) )
        self._post_random_sprite()
       
    def _create_medic_box(self):
        self._post_create_medic_box()
        if len(self.scene.level.GetEntities(TYPE_MEDIC_BOX)) >= 6: return
        x = random.randint(0, self.scene.level.GetWidth() - 10)
        y = 1
        box = MedicBox(self.gameController, self.sceneController, x, y)
        
    def _post_create_medic_box(self):
        next_time = random.randint(10, 90)
        self.schedualer.Post(next_time, self._create_medic_box)

    def Quit(self, arg=None):
        self.quit = True

    def DrawBackground(self):
        self.screen.blit(self.img_background, (0, 0))
        self.dashboard_rep1.Repaint()
        self.dashboard_rep2.Repaint()

    def Run(self):
        pygame.mouse.set_visible(False)
        self.DrawBackground()
        pygame.time.set_timer(FRAME_EVT, int(self.FRAME_DELAY))
        self.quit = False
        self._last_frame_time = time.time()

        while self.quit == False:
            event = pygame.event.wait()
            
            if event.type is KEYDOWN:
                for dlg in self.dialogs: dlg.HandleKey(event.key)
                if event.key == K_ESCAPE: self.reallyQuitDialog.Open()
                for keyboard in self.keyboards:
                    keyboard.handleClick(event.key)
                    
            elif event.type == FRAME_EVT:
                _t = time.time()
                frame_delay = min( (_t - self._last_frame_time) * 1000, self.FRAME_DELAY * 2)
                self._last_frame_time = _t
                
                self.schedualer.Poll()
                
                p_keys = pygame.key.get_pressed()
                for keyboard in self.keyboards:
                    keyboard.handlePressedKeys(p_keys, frame_delay)
        
                self.scene.physics_engine.Advance(frame_delay)
                self.player1.Advance(frame_delay)
                self.player2.Advance(frame_delay)
                self.camera1.Paint()
                self.camera2.Paint()
                self.dashboard_rep1.Paint()
                self.dashboard_rep2.Paint()
                
                # handling sprites (which are just for beauty)
                _to_delete = []
                for s in self.sprites:
                    s.Advance(frame_delay)
                    if s.Finished():
                        _to_delete.append(s)
                    s.Paint()
                for s in _to_delete: self.sprites.remove(s)

                for dlg in self.dialogs: dlg.Paint()
                
                #self.screen.blit(self.img_logo_bottom, (245, 375))
                pygame.display.flip()
                
        pygame.mouse.set_visible(True)



