"""
players and player representation.
"""


import os, math
import pygame
from m_Charactors import *
from weapons import *
from scene import Camera
from m_Dashboard import Dashboard



class AimMarker:

    RADIUS = 25
    
    def __init__(self):
        self.angle = 0
        self.x = self.RADIUS
        self.y = 0

    def GetAngle(self):
        return self.angle
        
    def SetAngle(self, angle):
        self.angle = angle
        self.x = int ( self.RADIUS * math.cos((angle * math.pi) / 180.0) )
        self.y = int (-self.RADIUS * math.sin((angle * math.pi) / 180.0) )
        
    def FlipHorizontal(self):
        self.SetAngle(180 - self.GetAngle())



class Player:
    
    def __init__(self, id, game, name, charactor=None):
        self.id = id
        self.game = game
        self.name = name
        self.charactor = None
        self.dashboard = Dashboard()
        self.aim_marker = AimMarker()
        self.weapon = Bazooka(self.id, game.gameController, game.sceneController)
        self.dashboard.ammo = self.weapon.ammo
        self.dashboard.weapon_name = self.weapon.name
        self.dashboard.player_name = self.name
        if charactor != None:
            self.SetCharactor(charactor)
        
    def Advance(self, time):
        self.weapon.Advance(time)
        self.dashboard.Advance(time)
        self.charactor.Advance(time)
        
##    def AddWeapon(self, weapon_class):
##        weapon = weapon_class(self)
##        self.weapons.append(weapon)
##        self.SetWeapon(0)
##        
##    def SetWeapon(self, index):
##        self.weapon_index = index
##        weapon = self.weapons[self.weapon_index]
##        self.dashboard.weapon_name = weapon.name
##        self.dashboard.ammo = weapon.ammo
##    
##    def ChangeWeapon(self, dir):
##        self.SetWeapon( (self.weapon_index + dir) % len(self.weapons) )
##
    def SetCharactor(self, charactor):
        self.charactor = charactor
        self.dashboard.flight = charactor.flight
        self.SetRandomEntityPosition()

    def SetDir(self, dir):
        if self.charactor.GetDir() != dir:
            self.aim_marker.FlipHorizontal()
        self.charactor.dir = dir
        self.charactor.UpdateEntityState()
        
    def GetType(self):
        return self.charactor.type
        
    def MoveLeft(self):
        if self.charactor.GetDir() == DIR_RIGHT:
            self.aim_marker.FlipHorizontal()
        self.charactor.MoveLeft()

    def MoveRight(self):
        if self.charactor.GetDir() == DIR_LEFT:
            self.aim_marker.FlipHorizontal()
        self.charactor.MoveRight()

    def SetAimAngle(self, angle):
        self.aim_marker.SetAngle(angle)
        self.charactor.aim_angle = angle
        self.charactor.UpdateEntityState()
        
    def AimUp(self, delta_angle):
        ang = self.aim_marker.GetAngle()
        if self.charactor.GetDir() == DIR_LEFT:
            ang -= delta_angle
            if ang < 100: ang = 100
        else:
            ang += delta_angle
            if ang > 80: ang = 80
        self.SetAimAngle(ang)
        
    def AimDown(self, delta_angle):
        ang = self.aim_marker.GetAngle()
        if self.charactor.GetDir() == DIR_LEFT:
            ang += delta_angle
            if ang > 260: ang = 260
        else:
            ang -= delta_angle
            if ang < -80: ang = -80
        self.SetAimAngle(ang)

    def MoveJump(self):
        self.charactor.MoveJump()
        
    def Shoot(self):
        if self.dashboard.lives.value <= 0: return
        x, y = self.game.sceneController.GetEntityPosCenter(self.charactor.entity_id)
        x += self.aim_marker.x / 2.5
        y += self.aim_marker.y / 2.5
        self.weapon.Fire(x, y, self.aim_marker.GetAngle())
        
    def DecreaseHealth(self, damage):
        """
        returns True if one life has lost.
        """
        self.dashboard.health.value -= damage
        if self.dashboard.health.value <= 0:
            self.dashboard.health.value = 100
            self.dashboard.flight.value = 100
            self.dashboard.lives.value -= 1
            if self.dashboard.lives.value <= 0:
                self.game.NotifyPlayerLoss(self)
                return True
            self.game.NotifyPlayerKilled(self)
            self.SetRandomEntityPosition()
            return True
        return False
            
    def SetRandomEntityPosition(self):
        entity_id = self.charactor.entity_id
        x = random.uniform(0, self.game.scene.level.GetWidth())
        y = 30
        vx = 0
        vy = -80
        self.game.sceneController.SetEntityPos(entity_id, x, y)
        self.game.sceneController.SetEntityVelocity(entity_id, vx, vy)



class PlayerCamera(Camera):
    
    def __init__(self, player_id, game, output_surface):
        Camera.__init__(self, game.scene, game.scene_rep, output_surface)
        self.sceneController = game.sceneController
        self.gameController = game.gameController
        self.player_id = player_id
        self.entity_id = self.gameController.GetPlayerEntityId(player_id)
        # the target aim
        path = os.path.join('data', 'app', 'aim_target.png')
        self.img_aim_marker = pygame.image.load(path).convert()
        self.img_aim_marker.set_colorkey(self.img_aim_marker.get_at((0,0)),
                                         pygame.locals.RLEACCEL)
        self.player_names = {} # from player_id to surface
        for player_id in self.gameController.GetAllPlayerIds():
            self.__create_player_name__(
                player_id,
                self.gameController.GetPlayerName(player_id))
            
    def __create_player_name__(self, player_id, player_name):
        """
        we cache player name as a surface so we won't need to render
        the font more than onces.
        """
        path = os.path.join('data', 'fonts', 'vera.ttf')
        font = pygame.font.Font(path, 10)
        color = (250, 250, 250)
        self.player_names[player_id] = font.render(player_name, 0, color)
    
    def Paint(self):
        is_player_alive = self.sceneController.EntityExists(self.entity_id)
        if is_player_alive:
            cx, cy = self.sceneController.GetEntityPosCenter(self.entity_id)
            self.SetPos(cx, cy)
        Camera.Paint(self)
        if is_player_alive:
            x, y = self.gameController.GetPlayerAimXY(self.player_id)
            x += cx - self.img_aim_marker.get_width() / 2
            y += cy - self.img_aim_marker.get_height() / 2
            self.PaintObj(x, y, self.img_aim_marker)
        # draw player names
        for player_id in self.gameController.GetAllPlayerIds():
            entity_id = self.gameController.GetPlayerEntityId(player_id)
            if self.sceneController.EntityExists(entity_id):
                ex, ey = self.sceneController.GetEntityPos(entity_id)
                if not self.player_names.has_key(player_id):
                    self.__create_player_name__(
                        player_id,
                        self.gameController.GetPlayerName(player_id))
                self.PaintObj(ex, ey - 17, self.player_names[player_id] )






        
        
