import os
import random
import pygame
from graphics import Sprite


SPRITE_ZAPPA   = 0
SPRITE_CLEESE  = 1
SPRITE_FOOT    = 2
SPRITE_LOSER   = 3


class SpriteZappa(Sprite):
    """
    Frank Zappa!
    """
    
    STATE_MOVING_RIGHT    = 0
    STATE_TALKING         = 1
    STATE_MOVING_LEFT     = 2
    
    quotes = ["You're an asshole",
              "Nothing but the best for my dog",
              "Jazz is not dead",
              "Number one ain't you",
              "Who are the brain police?",
              "Here's one for mother",
              "she knows how to NASTY",
              "Eddie, are you kidding?",
              "Kill Ugly Radio",
              "Freak Out",
              "Don't cry",
              "Get your ass out there!",
              "For me, cigarettes are food",
              "Don't mind your make-up",
              "this is the exciting part",
              "There is no hell. There is only France",
              "Nobody looks good bent over",
              "Rain is bad for electrical equipment",
              "Beware of the fish people",
              "And now... you are going to dance",
              "Vegetables are your friends"]

    def __init__(self, output_surface, res=None, prototype=None):
        Sprite.__init__(self, output_surface, res, prototype)
        self.width = self.GetImage().get_width()
        self.height = self.GetImage().get_height()
        self.x = -self.width
        self.y = output_surface.get_height() - self.height
        self.state = self.STATE_MOVING_RIGHT
        path = os.path.join('data', 'fonts', 'courier.ttf')
        font = pygame.font.Font(path, 12)
        text = random.choice(self.quotes)
        self.text_surface = font.render(text, 0, (200, 200, 200), (255, 0, 0))
        self.text_surface.set_colorkey((255, 0, 0))
        self._time = 0
        self.finished = False
        
    def Clone(self):
        return SpriteZappa(self.x, self.y, self.output_surface, \
                               res=None, prototype=self)

    def Finished(self):
        return self.finished

    def Advance(self, time):
        Sprite.Advance(self, time)
        if self.state == self.STATE_MOVING_RIGHT:
            self.x += float(time) / 6
            if self.x > 0:
                self.x = 0
                self.state = self.STATE_TALKING
        elif self.state == self.STATE_TALKING:
            self._time += time
            if self._time > 1800:
                self.state = self.STATE_MOVING_LEFT
        elif self.state == self.STATE_MOVING_LEFT:
            self.x -= float(time) / 5
            if self.x < -self.width:
                self.finished = True
                
    def Paint(self):
        Sprite.Paint(self)
        if self.state == self.STATE_TALKING:
            pos = [5, self.y - self.text_surface.get_height() - 3]
            self.output_surface.blit(self.text_surface, pos)



class SpriteFoot(Sprite):
    """
    Rotating foot.
    """

    def __init__(self, output_surface, res=None, prototype=None):
        Sprite.__init__(self, output_surface, res, prototype)
        self.width = self.GetImage().get_width()
        self.height = self.GetImage().get_height()
        x1 = output_surface.get_width()
        x2 = -self.width
        y1 = int(random.uniform(0, output_surface.get_height()))
        y2 = int(random.uniform(0, output_surface.get_height()))
        if random.randint(0, 1) == 0:
            self.x1 = x1
            self.x2 = x2
        else:
            self.x1 = x2
            self.x2 = x1
        if random.randint(0, 1) == 0:
            self.y1 = y1
            self.y2 = y2
        else:
            self.y1 = y2
            self.y2 = y1
        self.x = self.x1
        self.y = self.y1
        self.steps = 50
        self.dx = float(self.x2 - self.x1) / self.steps
        self.dy = float(self.y2 - self.y1) / self.steps
        self.finished = False
        
    def Clone(self):
        return SpriteFoot(self.x, self.y, self.output_surface, \
                          res=None, prototype=self)

    def Finished(self):
        return self.finished

    def Advance(self, time):
        Sprite.Advance(self, time)
        self.steps -= float(time)/30
        self.x += self.dx
        self.y += self.dy
        if self.steps < 0:
            self.finished = True


class SpriteCleese(Sprite):
    """
    John Cleese!
    """
    
    STATE_MOVING_LEFT     = 0
    STATE_TALKING         = 1
    STATE_MOVING_RIGHT    = 2

    
    def __init__(self, output_surface, res=None, prototype=None):
        Sprite.__init__(self, output_surface, res, prototype)
        self.width = self.GetImage().get_width()
        self.height = self.GetImage().get_height()
        self.x1 = output_surface.get_width()
        self.x2 = self.x1 - self.width
        self.x = self.x1
        self.y = random.uniform(20, output_surface.get_height() - self.height)
        self.state = self.STATE_MOVING_LEFT
        self._time = 0
        self.finished = False
        
    def Clone(self):
        return SpriteCleese(self.x, self.y, self.output_surface, \
                            res=None, prototype=self)

    def Finished(self):
        return self.finished

    def Advance(self, time):
        Sprite.Advance(self, time)
        if self.state == self.STATE_MOVING_LEFT:
            self.x -= float(time) / 6
            if self.x <= self.x2:
                self.x = self.x2
                self.state = self.STATE_TALKING
        elif self.state == self.STATE_TALKING:
            self._time += time
            if self._time > 1300:
                self.state = self.STATE_MOVING_RIGHT
        elif self.state == self.STATE_MOVING_RIGHT:
            self.x += float(time) / 5
            if self.x > self.x1:
                self.finished = True


class SpriteLoser(Sprite):
    """
    American holds "Loser!".
    """
    
    STATE_MOVING_DOWN     = 0
    STATE_TALKING         = 1
    STATE_MOVING_UP       = 2

    
    def __init__(self, output_surface, res=None, prototype=None):
        Sprite.__init__(self, output_surface, res, prototype)
        self.width = self.GetImage().get_width()
        self.height = self.GetImage().get_height()
        self.x = (output_surface.get_width() - self.width) / 2
        self.y1 = -self.height
        self.y2 = 0
        self.y = self.y1
        self.state = self.STATE_MOVING_DOWN
        self._time = 0
        self.finished = False
        
    def Clone(self):
        return SpriteLoser(self.x, self.y, self.output_surface, \
                           res=None, prototype=self)

    def Finished(self):
        return self.finished

    def Advance(self, time):
        Sprite.Advance(self, time)
        if self.state == self.STATE_MOVING_DOWN:
            self.y += float(time) / 6
            if self.y >= self.y2:
                self.y = self.y2
                self.state = self.STATE_TALKING
        elif self.state == self.STATE_TALKING:
            self._time += time
            if self._time > 2500:
                self.state = self.STATE_MOVING_UP
        elif self.state == self.STATE_MOVING_UP:
            self.y -= float(time) / 5
            if self.y < self.y1:
                self.finished = True

