
"""
Defines the Animation class.
"""

class Animation:
    """
    Animation is a collection of Image objects.
    @ivar cycles: number of cycles the animation
    has finished.
    """
    
    def __init__(self, images, delay):
        """
        Constructs a new Animation object.
        @param images: list of Image objects.
        @param delay: delay in milliseconds between
        the frames.
        """
        self._images = images
        self._delay = delay
        self._time = 0
        self._image_index = 0 # current index
        self.cycles = 0
        
    def Advance(self, time):
        """
        Advances the animation.
        @param time: time in milliseconds to advance.
        """
        self._time += time
        while self._time > self._delay:
            self._time -= self._delay
            self._image_index += 1
            if self._image_index >= len(self._images):
                self._image_index = 0
                self.cycles += 1
    
    def GetFrame(self):
        """
        @return: the current Image.
        """
        return self._images[self._image_index]
    
    def GetFrameIndex(self):
        """
        @return: the current Image's index.
        """
        return self._image_index
   
    def SetFrameIndex(self, index):
        """
        Skip to frame with the given index.
        @param index: the frame index to skip to.
        """
        self._image_index = index
    