import os
import pygame
from m_Animation import *


class SpriteRes:
    
    class _AnimPrototype:
        def __init__(self):
            self.images = []
            self.delay = 0
            self.type = None
            
    def __init__(self):
        self.animations = []

    def Load(self, filename):
        """
        load this object with data from the given file name.
        """
        f = file(filename, 'r')
        _anim = None
        while 1:
            line = f.readline()
            if len(line) == 0: break
            line = line.strip()
            if len(line) < 3 or line.find('#') == 0: continue
            kv = line.split(':')
            k, v = kv[0].strip(), kv[1]
            if k == 'type':
                self.type = int( v.strip() )
            if k == 'animation':
                v = v.strip()
                _anim = SpriteRes._AnimPrototype()
                self.animations.append(_anim)
            elif k == 'animation.delay':
                _anim.delay = int(v.strip())
            elif k == 'frame':
                path = os.path.join(os.path.dirname(filename), v.strip() )
                img = pygame.image.load(path).convert()
                img.set_colorkey(img.get_at((0, 0)))
                _anim.images.append(img)


class Sprite:
    
    def __init__(self, output_surface, res=None, prototype=None):
        self._anims = []
        if res != None:
            for a in res.animations:
                self._anims.append(Animation(a.images, a.delay))
            self.type = res.type
        elif prototype != None:
            for a in prototype._anims:
                self._anims.append(Animation(a._images, a._delay))
            self.type = prototype.type
        else:
            raise Exception('Sprite must be initialized with either a resource or a prototype')
        self.x = 0
        self.y = 0
        self.output_surface = output_surface
        self.anim_index = 0
        
    def Clone(self):
        return Sprite(self.x, self.y, self.output_surface, \
                      res=None, prototype=self)
                      
    def Finished(self):
        return False
    
    def Advance(self, time):
        self._anims[self.anim_index].Advance(time)
    
    def GetAnimation(self):
        return self._anims[self.anim_index]

    def GetImage(self):
        return self._anims[self.anim_index].GetFrame()
    
    def Paint(self):
        self.output_surface.blit(self.GetImage(), [self.x, self.y] )

