"""
primitive widgets for the game itself (not the app)
"""

import os
import pygame
import pygame.locals



#class FloatProgBar:
#    """
#    Like ProgBar, but with no surface nor value attached.
#    """
#    
#    BACKGROUND_COLOR  = (0, 0, 0)
#    BORDER_COLOR      = (250, 250, 250)
#    
#    def __init__(self, w, h, image_file_name):
#        self.w = w
#        self.h = h
#        self.img_fill = pygame.image.load(image_file_name).convert()
#        self.img_fill = pygame.transform.scale(self.img_fill, (w, h) )
#
#    def Paint(self, surface, x, y, value):
#        s = surface.subsurface([x, y, self.w, self.h])
#        s.fill( self.BACKGROUND_COLOR )
#        rect = [0, 0, self.w, self.h]
#        pygame.draw.rect(s, self.BORDER_COLOR, rect, 1)
#        rect = [1, 1, ( value * (self.w - 2) ) / 100, self.h-2]
#        s.set_clip(rect)
#        s.blit(self.img_fill, (1, 1) )
#
#
#
#
#class ProgBar(FloatProgBar):
#    
#    def __init__(self, output_surface, value=0, image_file_name=None):
#        self.surface = output_surface
#        self.value = value
#        w = output_surface.get_width()
#        h = output_surface.get_height()
#        if image_file_name == None:
#            image_file_name = os.path.join('data', 'game_widgets', \
#                                           'prog_bar_blue.png')
#        FloatProgBar.__init__(self, w, h, image_file_name)
#
#    def Paint(self):
#        FloatProgBar.Paint(self, self.surface, 0, 0, self.value)



class ProgBar:
    """
    Like ProgBar, but with no surface nor value attached.
    """
    
    BACKGROUND_COLOR  = (0, 0, 0)
    BORDER_COLOR      = (250, 250, 250)
    
    def __init__(self, output_surface, image_file_name):
        self.output_surface = output_surface
        self.w, self.h = output_surface.get_size()
        self.img_fill = pygame.image.load(image_file_name).convert()
        self.img_fill = pygame.transform.scale(self.img_fill, (self.w, self.h))
        self.bgcolor = (0, 0, 0)
        self.border_color = (80, 160, 80)
        self.value = None
        self.surface = pygame.Surface(self.output_surface.get_size())
        
    def draw_box(self):
        s = self.surface
        s.set_clip()
        s.fill(self.bgcolor)
        rect = [0, 0, self.w, self.h]
        pygame.draw.rect(s, self.border_color, rect, 1)

    def Repaint(self):
        self.draw_box()
        self.Paint()
        
    def Paint(self):
        s = self.output_surface
        s.set_clip()
        s.blit(self.surface, (0, 0))
        rect = [1, 1, ( self.value * (self.w - 2) ) / 100, self.h-2]
        s.set_clip(rect)
        s.blit(self.img_fill, (1, 1) )



class TextBar:
    
    def __init__(self, output_surface, value=""):
        self.output_surface = output_surface
        self.value = value
        self.old_value = None
        _tmp = os.path.join('data', 'fonts', 'courier.ttf')
        self.font = pygame.font.Font(_tmp, 10)
        self.bgcolor = (0, 0, 0)
        self.fgcolor = (180, 180, 180)
        self.surface = pygame.Surface(self.output_surface.get_size())
        
    def Repaint(self):
        self.old_value = self.value
        self.surface.fill(self.bgcolor)
        img = self.font.render(self.value, 1, self.fgcolor, self.bgcolor)
        self.surface.blit(img, (0, 0) )
        self.output_surface.blit(self.surface, (0, 0))
            
    def Paint(self):
        if self.value != self.old_value:
            self.Repaint()
        self.output_surface.blit(self.surface, (0, 0))




class TextDialog:
    def __init__(self, text, size=20):
        self.output_surface = pygame.display.get_surface()
        _tmp = os.path.join('data', 'fonts', 'courier.ttf')
        self.font = pygame.font.Font(_tmp, size)
        self.fgcolor = (80, 160, 80)
        self.bgcolor = (0, 0, 0)
        self.surface = self.font.render(text, 0, self.fgcolor, self.bgcolor)
        self.surface.set_colorkey(self.bgcolor)
        w1, h1 = self.output_surface.get_size()
        w2, h2 = self.surface.get_size()
        self.pos = ( (w1 - w2) / 2, (h1 - h2) / 2 )
        self.is_open = False
        self.value_returned_callbacks = []

    def Open(self):
        self.is_open = True
        
    def Close(self):
        self.is_open = False

    def HandleKey(self, key): return

    def NotifyValue(self, val):
        for cb in self.value_returned_callbacks:
            cb(self, val)
    
    def Paint(self):
        if self.is_open:
            self.output_surface.blit(self.surface, self.pos)

    

class ReallyStopGameDialog(TextDialog):
    
    def __init__(self):
        TextDialog.__init__(self, "Really Quit? (y/n)", 26)
        
    def HandleKey(self, key):
        if self.is_open == False: return
        elif key == pygame.locals.K_y:
            self.NotifyValue(True)
        elif key == pygame.locals.K_n:
            self.NotifyValue(False)
            

class CameraBottomTextDialog:
            
    def __init__(self, schedualer, output_surface):
        self.output_surface = output_surface
        self.schedualer = schedualer
        _tmp = os.path.join('data', 'fonts', 'courier.ttf')
        self.font = pygame.font.Font(_tmp, 14)
        self.fgcolor = (240, 240, 200)
        self.bgcolor = (70, 70, 0)
        self.open_count = 0
        self.value_returned_callbacks = []
        self.bg_pos = (0, self.output_surface.get_height() - 20)
        self.bg_surface = pygame.Surface( (self.output_surface.get_width(), 20) )
        self.bg_surface.fill(self.bgcolor)
        self.bg_surface.set_alpha(90, pygame.locals.RLEACCEL)
        
    def Open(self, text):
        self.open_count += 1
        self.font_surface = self.font.render(text, 0, self.fgcolor, self.bgcolor)
        self.font_surface.set_colorkey(self.bgcolor)
        self.font_pos = (self.bg_pos[0] + 5, 
                         self.bg_pos[1] + (20 - self.font_surface.get_height()) / 2)
        self.schedualer.Post(2, self.Close)
        
    def Close(self):
        self.open_count = max(self.open_count - 1, 0)

    def HandleKey(self, key): return

    def NotifyValue(self, val):
        for cb in self.value_returned_callbacks:
            cb(self, val)
    
    def Paint(self):
        if self.open_count > 0:
            self.output_surface.blit(self.bg_surface, self.bg_pos)
            self.output_surface.blit(self.font_surface, self.font_pos)

    

