import pygame, os
from pgu import gui
from tools.m_Multifile import Compiler, Multifile
from StringIO import StringIO



ROCKS_VERSION_PREFIX = "rv_0_1"



class NamedFile:
    def __init__(self, name, file):
        self.name = name
        self.file = file



wellcome_msg =\
"""
Wellcome to FLAMEingo experimental level editor.

This is a very immature version with many bugs
and un-implemented features.

Enjoy!
"""


help_msg =\
"""
How to create a new level

This editor is quiet stupid. It's only job is to take
existing land and sky images and compile a ".lev" file.
(and adding rocks, using the left tool bar)

If you wanna start right away, click "New" and load the
default (and empty) land and default sky. Then add some
rocks and save the whole thing.

If you want land in your level, you should close this
editor and open an image editor to draw it.
Note that in a game's world, not all of the world is
land! there are also "land-less" areas where you see
the sky behind.
The editor takes the color at the upper-left point (at 0, 0)
of the land image as the "land-less color".
All the points in the land image with the "land-less color"
shall be land-less, ofcourse.

The sky could be any image. it will be stretched to fit
the land automatically.

"""



class MsgDialogWithScrollers(gui.Dialog):
    def __init__(self, msg_header_text, msg_body_text):
        title = gui.Label(msg_header_text, cls="text_white_16")
        doc = gui.Document(background=(30, 30, 30))
        for line in msg_body_text.split("\n"):
            for word in line.split(" "):
                doc.add(gui.Label(word + " ", cls="text_white_12"))
            doc.br(22)
        gui.Dialog.__init__(self,title,gui.ScrollArea(doc, 400, 240, no_hslider=True))



class MsgDialog(gui.Dialog):
    def __init__(self, msg_header_text, msg_body_text):
        title = gui.Label(msg_header_text, cls="text_white_16")
        style = {'padding_left': 10,
                 'padding_right': 10,
                 'padding_top': 10,
                 'padding_bottom': 10}
        doc = gui.Document(style=style)
        for line in msg_body_text.split("\n"):
            for word in line.split(" "):
                doc.add(gui.Label(word + " ", cls="text_white_12"))
            doc.br(22)
        gui.Dialog.__init__(self,title,doc)
    



#===============================================================================
# Wizard classes
#===============================================================================


class Step1(gui.Document):
    def __init__(self, wizard):
        gui.Document.__init__(self)
        self.wizard = wizard
        self.add(gui.Label("Step 1", cls="text_white_16"))
        self.br(20); self.br(1)
        self.add(gui.Label("Let's load a PNG image to be our land.", cls="text_white_12"))
        self.br(20); self.br(1)
        t = gui.Table()
        t.tr()
        td_style = {'padding_right': 10}
        t.td(gui.Label("File Name:", cls="text_white_12"), style=td_style)
        self.input_file = gui.Input()
        t.td(self.input_file, style=td_style)
        self.button_load = gui.Button("Load")
        self.button_load.connect(gui.CLICK, self.handle_click_load, None)
        t.td(self.button_load, style=td_style)
        self.button_browse = gui.Button("Browse...")
        self.button_browse.connect(gui.CLICK, self.handle_click_browse, None)
        t.td(self.button_browse, style=td_style)
        self.add(t)
        self.br(5); self.br(1)
        
    def handle_click_load(self, arg):
        fname = self.input_file.value
        try:
            f = file(fname, 'rb')
            img = pygame.image.load(f).convert()
        except Exception, Error:
            dlg = MsgDialog("Ooops!", "file not found or not an image")
            dlg.open()
            return
        self.wizard.handle_new_land_image(NamedFile(os.path.split(fname)[1], f), img)
        
    def handle_click_browse(self, arg):
        path = os.path.join(os.getcwd(), 'data', 'level_editor', 'default_lands')
        if not os.path.exists(path): path=None
        dlg = gui.FileDialog(button_txt="Open", path=path)
        dlg.connect(gui.CLOSE, self.handle_browse_closed, dlg)
        dlg.open()
        
    def handle_browse_closed(self, dlg):
        if dlg.value != None: self.input_file.value = dlg.value
        
    
class Step2(gui.Document):
    def __init__(self, wizard):
        gui.Document.__init__(self)
        self.wizard = wizard
        self.add(gui.Label("Step 2", cls="text_white_16"))
        self.br(20); self.br(1)
        self.add(gui.Label("Alright! Now let's load an image to be our sky.", cls="text_white_12"))
        self.br(10); self.br(1)
        t = gui.Table()
        t.tr()
        td_style = {'padding_right': 10}
        t.td(gui.Label("File Name:", cls="text_white_12"), style=td_style)
        self.input_file = gui.Input()
        t.td(self.input_file, style=td_style)
        self.button_load = gui.Button("Load")
        self.button_load.connect(gui.CLICK, self.handle_click_load, None)
        t.td(self.button_load, style=td_style)
        self.button_browse = gui.Button("Browse...")
        self.button_browse.connect(gui.CLICK, self.handle_click_browse, None)
        t.td(self.button_browse, style=td_style)
        self.add(t)
        
    def handle_click_load(self, arg):
        fname = self.input_file.value
        try:
            f = file(fname, 'rb')
            img = pygame.image.load(f).convert()
        except Exception:
            dlg = MsgDialog("Ooops!", "file not found or not an image")
            dlg.open()
            return
        self.wizard.handle_new_sky_image(NamedFile(os.path.split(fname)[1], f), img)
        
    def handle_click_browse(self, arg):
        path = os.path.join(os.getcwd(), 'data', 'level_editor', 'default_skies')
        if not os.path.exists(path): path=None
        dlg = gui.FileDialog(button_txt="Open", path=path)
        dlg.connect(gui.CLOSE, self.handle_browse_closed, dlg)
        dlg.open()
        
    def handle_browse_closed(self, dlg):
        if dlg.value != None: self.input_file.value = dlg.value
        


class Wizard(gui.Dialog):
    
    def __init__(self, editor):
        self.title = gui.Label("Level Wizard", cls="text_white_16")
        self.body = gui.Container()
        self.editor = editor
        self.step1 = Step1(self)
        self.step2 = Step2(self)
        self.body.add(self.step1, 0, 0)
        gui.Dialog.__init__(self, self.title, self.body)
        
    def handle_new_land_image(self, file, image):
        self.body.remove(self.step1)
        self.step1.blur()
        self.editor.handle_new_land_image(file, image)
        self.body.add(self.step2, 0, 0)
        self.body.resize()
        self.editor.app.app.repaintall()

    def handle_new_sky_image(self, file, image):
        self.editor.handle_new_sky_image(file, image)
        self.close()
        self.editor.app.app.repaintall()


class Rock:
    def __init__(self, file, surface, pos):
        self.file = file
        self.surface = surface
        self.pos = pos


class EditArea(gui.ScrollArea):
    
    def __init__(self, app, width, height):
        self.app = app
        self.main = gui.Container(width=width, height=height)
        self.main.add(gui.Spacer(10, 10), 0, 0)
        bg = pygame.image.load(os.path.join('data', 'themes', 'default', 'list.png'))
        style = {'padding_left': 1,
                 'padding_right': 1,
                 'padding_top': 1,
                 'padding_bottom': 1}
        gui.ScrollArea.__init__(self, self.main, width, height, background=bg, style=style)
        self.land_img = None
        self.sky_img = None
        self.land_file = None
        self.sky_file = None
        self.edit_image = None
        self.rocks = []

    def handle_new_land_image(self, file, image):
        self.rocks = []
        self.sky_img = None
        self.sky_file = None
        image.set_colorkey(image.get_at((0, 0)))
        self.land_img = image
        self.land_file = file
        self.edit_image = image.convert()
        self.main.widgets = []
        self.main.add(gui.Image(self.edit_image), 0, 0)
        self.resize()

    def handle_new_sky_image(self, file, image):
        self.sky_file = file
        image = pygame.transform.scale(image, self.edit_image.get_size())
        self.sky_img = image.convert()
        image.blit(self.edit_image, (0, 0))
        self.edit_image = image
        self.main.widgets = []
        self.main.add(gui.Image(self.edit_image), 0, 0)
        self.resize()

    def add_rock(self, rock):
        if not self.land_img or not self.sky_img: return
        self.rocks.append(rock)
        self.edit_image.blit(rock.surface, rock.pos)
        self.main.repaint()
        
    def get_offset(self): return (-self.sbox.offset[0], -self.sbox.offset[1])

    def load(self, fname):
        try:
            mf = Multifile(fname)
            config_file = mf.GetFile('config.txt')
            visited_rock_files = {}
            while True:
                line = config_file.readline()
                if len(line) == 0: break
                if len(line) < 3: continue
                line = line.rstrip('\n\r')
                k, v = line.split(':')
                if k == 'land-image':
                    fname = v.strip()
                    f = mf.GetFile(fname)
                    img = pygame.image.load(f).convert()
                    
                    self.handle_new_land_image(NamedFile(fname, f), img)
                elif k == 'sky-image':
                    fname = v.strip()
                    f = mf.GetFile(fname)
                    img = pygame.image.load(f).convert()
                    self.handle_new_sky_image(NamedFile(fname, f), img)
                elif k == 'rock':
                    p, n = v.split(';')
                    x, y = p.split(',')
                    x = int( x.strip(' (') )
                    y = int( y.strip(' )') )
                    n = n.strip(' ')
                    if visited_rock_files.has_key(n):
                        f, s = visited_rock_files[n]
                        rock = Rock(f, s, (x, y))
                        self.add_rock(rock)
                    else:
                        f = mf.GetFile(n)
                        s = pygame.image.load(f).convert()
                        s.set_colorkey(s.get_at((0, 0)))
                        nf = NamedFile(n, f)
                        visited_rock_files[n] = (nf, s)
                        rock = Rock(nf, s, (x, y))
                        self.add_rock(rock)
        except Exception, Error:
            MsgDialog("Oh boy!", "Something horrible happened.").open()
            return

    def save(self, fname):
        try:
            c = Compiler()
            c.Start(fname)
            rock_files = []
            visited_rock_files = []
            ##
            ## create the "config.txt" file
            ##
            config_str =  "land-image: " + self.land_file.name + "\n"
            config_str += "sky-image: " + self.sky_file.name + "\n"
            for rock in self.rocks:
                config_str += "rock: (" + str(rock.pos[0]) + ", " + str(rock.pos[1]) + "); " + rock.file.name + "\n"
                if rock.file not in visited_rock_files:
                    rock_files.append(rock.file)
                    visited_rock_files.append(rock.file)
            c.WriteFile("config.txt", StringIO(config_str))
            ##
            ## write the images
            ##
            files = [self.land_file, self.sky_file]
            files.extend(rock_files)
            for named_file in files:
                c.WriteFile(named_file.name, named_file.file)
            c.Close()
        except Exception, Error:
            MsgDialog("Oh boy!", "Something horrible happened.").open()
            return
        MsgDialog("Okay", "level saved").open()



class TopMenuBar(gui.Table):
    def __init__(self, app):
        gui.Table.__init__(self)
        self.app = app

        self.button_new = gui.Button("New...", width=90)
        self.button_load = gui.Button("Load...", width=90)
        self.button_save = gui.Button("Save...", width=90)
        self.button_help = gui.Button("Help!", width=90)
        self.button_quit = gui.Button("Quit", width=90)
        
        self.button_new.connect(gui.CLICK, self.handle_click_new, None)
        self.button_load.connect(gui.CLICK, self.handle_click_load, None)
        self.button_save.connect(gui.CLICK, self.handle_click_save, None)
        self.button_help.connect(gui.CLICK, self.handle_click_help, None)
        self.button_quit.connect(gui.CLICK, self.handle_click_quit, None)
        
        td_style = {'padding_right': 10, 'padding_bottom': 4}
        
        self.tr()
        self.td(self.button_new, style=td_style)
        self.td(self.button_load, style=td_style)
        self.td(self.button_save, style=td_style)
        self.td(self.button_help, style=td_style)
        self.td(self.button_quit, style=td_style)
        
    def handle_click_new(self, arg):
        Wizard(self.app.edit_area).open()

    def handle_click_load(self, arg):
        path = os.path.join(os.getcwd(), 'data', 'levels')
        if not os.path.exists(path): path=None
        dlg = gui.FileDialog(title_txt="Load Level..", button_txt="Open", path=path)
        dlg.connect(gui.CLOSE, self.handle_load_dlg_closed, dlg)
        dlg.open()

    def handle_click_save(self, arg):
        if (self.app.edit_area.land_file == None or
            self.app.edit_area.sky_file == None):
            MsgDialog("Not so fast", "Saving what? \nThe edit area is empty!").open()
            return
        path = os.path.join(os.getcwd(), 'data', 'levels')
        if not os.path.exists(path): path=None
        save_dlg = gui.FileDialog(title_txt="Save Level..", button_txt="Save", path=path)
        save_dlg.connect(gui.CLOSE, self.handle_save_dlg_closed, save_dlg)
        save_dlg.open()
        
    def handle_load_dlg_closed(self, dlg):
        if dlg.value != None:
            self.app.edit_area.load(dlg.value)

    def handle_save_dlg_closed(self, dlg):
        if dlg.value != None:
            if os.path.exists(dlg.value):
                MsgDialog("So sorry!", "This version does not allow overriding\nexisting files...").open()
                return
            self.app.edit_area.save(dlg.value)

    def handle_click_help(self, arg):
        MsgDialogWithScrollers("Help!", help_msg).open()

    def handle_click_quit(self, arg):
        self.app.Quit()



class ToolRockEraser:
    def handle_click(self, edit_area, x, y):
        x1, y1 = edit_area.get_offset()
        p = (x + x1, y + y1)
        to_remove = []
        for rock in edit_area.rocks:
            rect = rock.surface.get_rect().move(rock.pos)
            if rect.collidepoint(p):
                to_remove.append(rock)
                edit_area.edit_image.blit(edit_area.sky_img, rock.pos, rect)
                edit_area.edit_image.blit(edit_area.land_img, rock.pos, rect)
                # redraw other rocks
                for rock2 in edit_area.rocks:
                    if rock2 not in to_remove:
                        edit_area.edit_image.blit(rock2.surface, rock2.pos)
        for r in to_remove: edit_area.rocks.remove(r)
        edit_area.repaint()



class ToolRock:
    def __init__(self, img_dir):
        self.rock_images = []
        for fname in os.listdir(img_dir):
            fname = os.path.join(img_dir, fname)
            if os.path.isfile(fname):
                try: 
                    f = file(fname, 'rb')
                    img = pygame.image.load(f).convert()
                    img.set_colorkey(img.get_at((0, 0)))
                    name = ROCKS_VERSION_PREFIX + os.path.split(fname)[1]
                    self.rock_images.append((NamedFile(name, f), img))
                except: pass
        if len(self.rock_images) == 0:
            raise Exception("no valid images found at " + img_dir)
        self.cur_rock_idx = 0
        self.cur_rock_img = self.rock_images[self.cur_rock_idx]
                
    def get_rock_image(self): return self.cur_rock_img[1]

    def handle_click(self, edit_area, x, y):
        file, img = self.cur_rock_img
        x -= img.get_width() / 2
        y -= img.get_height() / 2
        x1, y1 = edit_area.get_offset()
        rock = Rock(file, img, (x + x1, y + y1))
        edit_area.add_rock(rock)

    def open_rock_select(self):
        self.cur_rock_idx = (self.cur_rock_idx + 1) % len(self.rock_images)
        self.cur_rock_img = self.rock_images[self.cur_rock_idx]
        
        

class ToolBar(gui.Container):
    def __init__(self, app, width, height):

        self.tool_rock = ToolRock(os.path.join('data', 'level_editor', 'rocks'))
        self.tool_rock_eraser = ToolRockEraser()
        bg = pygame.image.load(os.path.join('data', 'themes', 'default', 'list.png'))
        style = {'padding_left': 1,
                 'padding_right': 1,
                 'padding_top': 1,
                 'padding_bottom': 1}
        gui.Container.__init__(self, width=width, height=height, background=bg, style=style)
        self.my_app = app
        _items = (("add rock", self.tool_rock), ("erase rock", self.tool_rock_eraser))
        self.toolbox = gui.Toolbox(_items, cols=1, rows=2, width=width-20)
        self.toolbox.value = None
        self.add(self.toolbox, 10, 10)
        self.rock_select = gui.Container(width=width-40, height=width-40, background=gui.theme.get("console", "", "background"))
        self.add(self.rock_select, 20, 80)
        self.set_selected_rock_img(self.tool_rock.get_rock_image())

    def set_selected_rock_img(self, img):
        self.rock_select.widgets = []
        w1 = self.rock_select.style.width
        h1 = self.rock_select.style.height
        w2 = img.get_width()
        h2 = img.get_height()
        self.rock_select.add(gui.Image(img), (w1 - w2)/2, (h1 - h2)/2)
        self.rock_select.resize()
        self.rock_select.repaint()
            
    def handle_click(self, edit_area, x, y):
        tool = self.toolbox.value
        if tool: tool.handle_click(edit_area, x, y)
        
    def open_rock_select(self):
        self.tool_rock.open_rock_select()
        self.set_selected_rock_img(self.tool_rock.get_rock_image())



class App:
    def __init__(self):
        self.screen = pygame.display.set_mode((640, 480), pygame.locals.FULLSCREEN)
        gui.theme.load(os.path.join('data', 'themes', 'default'))
        self.app = gui.App()
        self.main = gui.Container(background=(0, 0, 0))
        self.top_menu = TopMenuBar(self)
        self.toolbar = ToolBar(self, 120, 250)
        self.main.add(self.top_menu, 20, 20)
        self.edit_area = EditArea(self, 465, 350)
        self.main.add(self.edit_area, 145, 80)
        self.app.connect(gui.QUIT, self.Quit, None)
        self.main.add(self.toolbar, 10, 80)
        
        wellcome_dlg = MsgDialog("Wellcome!", wellcome_msg)
        self.app.connect(gui.INIT, wellcome_dlg.open, None)

        self.main.resize()
        self.app.connect(pygame.locals.MOUSEBUTTONDOWN, self.handle_button_down, None)
        self.app.run(widget=self.main, screen=self.screen)

    def Quit(self):
        self.app.quit()

    def handle_button_down(self, arg):
        if len(self.app.windows) > 0: return
        x, y = pygame.mouse.get_pos()
        r = self.edit_area.sbox.get_abs_rect()
        if r.collidepoint(x, y):
            x -= r.x
            y -= r.y
            self.toolbar.handle_click(self.edit_area, x, y)
            return
        if self.toolbar.rock_select.get_abs_rect().collidepoint(x, y):
            self.toolbar.open_rock_select()
            return


app = App()

