"""
Example of a console-based chat using the flameingo IPC mechanism.
Don't panic :)
Most of the code deals with parsing user commands.

"""


from m_FlameingoIPC import FlameingoIPC, FLAG_ACK_REQUIRED
from m_Messages import ChatMessage
from time import sleep
from threading import Thread
from sys import stdout


CHAT_FLAGS = FLAG_ACK_REQUIRED


class IPCPollerThread(Thread):
    
    def __init__(self, ipc):
        Thread.__init__(self)
        self.ipc = ipc
        self.setDaemon(True)
    
    def run(self):
        while True:
            self.ipc.Poll()
            sleep(0.1)
        


class ChatExample:

    def __init__(self):
        self.ipc = FlameingoIPC()
        self.ipc.RegisterErrorHandler(self.HandleIPCError)
        self.ipc.RegisterNewConnectionHandler(self.HandleNewConnection)
        self.host_count = 0
        self.remote_hosts = {}
        poller = IPCPollerThread(self.ipc)
        poller.start()

    def Prompt(self):
        stdout.write(">> ")
        
    def HandleIPCError(self, sender, text):
        print "\nerror:", text
        
    def HandleConnectionClosed(self, host):
        host_id = None
        for host_id in self.remote_hosts:
            if self.remote_hosts[host_id] == host: break
        if host_id != None:
            print "\nconnection lost to", host.addr, ", host-id:", host_id
            self.remote_hosts.pop(host_id)
        
    def HandleNewConnection(self, host):
        print "\nnew host connected. addr:", host.addr, ", host-id:", self.host_count
        self.remote_hosts[self.host_count] = host
        self.host_count += 1
        host.RegisterObjectReceivedHandler(ChatMessage.type, self.HandleTextMessage)
        host.RegisterConnectionClosedHandler(self.HandleConnectionClosed)
    
    def HandleTextMessage(self, sender, msg):
        print "\nnew msg from", sender.addr, ": ", msg.text
    
    def Start(self):
        print ""
        print "Chat Example"
        print ""
        print "COMMANDS "
        print "     q                           quit "
        print "     init <port>                 start listening to port for connections"
        print "     connect <host> <port>       connect to remote destination"
        print "     send <host-id> <text>       send chat message"
        print ""
        
        while (True):
            self.Prompt()
            input = raw_input()
            argv = input.split(' ', 2)
            
            if len(argv[0]) == 0: continue
            
            elif argv[0] == 'q':
                break
            
            elif argv[0] == 'init':
                if len(argv) < 2:
                    print "no port specified"
                    continue
                elif not argv[1].isdigit():
                    print "port must be a number"
                    continue
                elif self.ipc.Init(int(argv[1])) == True:
                    print "Ok"
                else:
                    print "Failed"
                    
            elif argv[0] == 'connect':
                if len(argv) < 3:
                    print "not enough arguments"
                    continue
                elif not argv[2].isdigit():
                    print "port must be a number"
                    continue
                host = self.ipc.CreateNewConnection(argv[1], int(argv[2]))
                if host == None:
                    print "connection failed"
                else:
                    self.HandleNewConnection(host)
                    print "connected successfully"
        
            elif argv[0] == 'send':
                if len(argv) < 3:
                    print "not enough arguments"
                    continue
                elif not argv[1].isdigit():
                    print "host-id must be a number"
                    continue
                elif int(argv[1]) not in self.remote_hosts:
                    print "no host with this id is connected"
                    continue
                msg = ChatMessage()
                msg.text = argv[2]
                self.remote_hosts[int(argv[1])].SendObject(msg, CHAT_FLAGS)
                print "Ok"
                
            else:
                print "unknown command", argv[0]
        
        

example = ChatExample()
example.Start()

   