from m_Serializer import *
from defines import FLAMEINGO_VERSION


class ChatMessage:
    
    type = 10
    serializer = ClassSerializer()
    serializer.DefineAttribute("player_name", Type_str)
    serializer.DefineAttribute("text", Type_str)
    
    def __init__(self, player_name="", text=""):
        self.player_name = player_name
        self.text = text
        

#============================================================================#
# Game Initialization Messages
#============================================================================#


class JoinRequest:
        
    type = 20
    serializer = ClassSerializer()
    serializer.DefineAttribute("player_name", Type_str)
    serializer.DefineAttribute("client_version", Type_str)
    
    def __init__(self, player_name=""):
        self.player_name = player_name
        self.client_version = FLAMEINGO_VERSION



class JoinResponse:
    
    type = 21
    serializer = ClassSerializer()
    serializer.DefineAttribute("value", Type_u8)
    serializer.DefineAttribute("reason", Type_str)
    serializer.DefineAttribute("player_id", Type_u16)
    serializer.DefineAttribute("entity_id", Type_u32)
    
    def __init__(self, value=0, reason="", player_id=-1, entity_id=-1):
        self.value = value
        self.reason = reason
        self.player_id = player_id
        self.entity_id = entity_id



class GameInfoRequest:
    type = 22
    serializer = ClassSerializer()
    


class GameInfoResponse:
    
    type = 23
    serializer = ClassSerializer()
    serializer.DefineAttribute("level_file_name", Type_str)
    serializer.DefineAttribute("level_oid", Type_str)
    serializer.DefineAttribute("player_name", Type_str)
    serializer.DefineAttribute("version", Type_str)
    
    def __init__(self):
        self.level_file_name = ""
        self.level_oid = ""
        self.player_name = ""
        self.version = FLAMEINGO_VERSION

        

class MessageStartGame:
    type = 24
    serializer = ClassSerializer()



class CreatedObjectsRequest:
    type = 25
    serializer = ClassSerializer()


    
class CreatedObjectsResponse:
    type = 26
    serializer = ClassSerializer()

        

#============================================================================#
# Scene Controller Messages
#============================================================================#


class SceneEntityData:
    """
    This is not a message, just a data object.
    """
    serializer = ClassSerializer()
    serializer.DefineAttribute("entityType", Type_u8)
    serializer.DefineAttribute("x", Type_s16)
    serializer.DefineAttribute("y", Type_s16)
    serializer.DefineAttribute("vx", Type_f64)
    serializer.DefineAttribute("vy", Type_f64)
    serializer.DefineAttribute("state", Type_u8)
    serializer.DefineAttribute("id", Type_u32)
    
    def __init__(self, entityType=0, x=0, y=0, vx=0, vy=0, state=0, id=-1):
        self.entityType = entityType
        self.x = x
        self.y = y
        self.vx = vx
        self.vy = vy
        self.state = state
        self.id = id

        
        
class FullSceneState:
        
    serializer = ClassSerializer()
    serializer.DefineAttribute("entity_list", Type_list(Type_class(SceneEntityData)))
    
    def __init__(self):
        self.entity_list = []



# class SceneMessageCreateNewEntity:
#     
#     type = 40
#     serializer = ClassSerializer()
#     serializer.DefineAttribute("time", Type_f64)
#     serializer.DefineAttribute("entity", Type_class(SceneEntityData))
#     
#     def __init__(self, time=0, entityType=0, x=0, y=0, vx=0, vy=0, state=0, id=-1):
#         self.time = time
#         self.entity = SceneEntityData(entityType, x, y, vx, vy, state, id)
        


class SceneMessageDeleteEntity:
    
    type = 41
    serializer = ClassSerializer()
    serializer.DefineAttribute("time", Type_f64)
    serializer.DefineAttribute("id", Type_u32)
    
    def __init__(self, time=0, id=0):
        self.time = time
        self.id = id



class SceneMessageUpdateEntity:
        
    type = 42
    serializer = ClassSerializer()
    serializer.DefineAttribute("time", Type_f64)
    serializer.DefineAttribute("entity", Type_class(SceneEntityData))
    
    def __init__(self, time=0, entityType=0, x=0, y=0, vx=0, vy=0, state=0, id=-1):
        self.time = time
        self.entity = SceneEntityData(entityType, x, y, vx, vy, state, id)



        
class SceneMessageRemoveLand:
        
    type = 43
    serializer = ClassSerializer()
    serializer.DefineAttribute("time", Type_f64)
    serializer.DefineAttribute("shapeIndex", Type_u8)
    serializer.DefineAttribute("x", Type_u16)
    serializer.DefineAttribute("y", Type_u16)
    
    def __init__(self, time=0, x=0, y=0, shapeIndex=0):
        self.time = time
        self.x = x
        self.y = y
        self.shapeIndex = shapeIndex



class SceneMessageFullState:
        
    type = 44
    serializer = ClassSerializer()
    serializer.DefineAttribute("scene_state", Type_class(FullSceneState))
    serializer.DefineAttribute("time", Type_f64)
    
    def __init__(self, time=0, scene_state=None):
        self.time = time
        self.scene_state = scene_state



#============================================================================#
# Game Controller Messages
#============================================================================#


class NewPlayerData:
        
    serializer = ClassSerializer()
    serializer.DefineAttribute("player_id", Type_u16)
    serializer.DefineAttribute("player_name", Type_str)
    serializer.DefineAttribute("entity_data", Type_class(SceneEntityData))
    
    def __init__(self, player_id=0, player_name="", entity_data=None):
        self.player_id = player_id
        self.player_name = player_name
        self.entity_data = entity_data
        
        

class GameMessageCreateNewPlayer:
        
    type = 60
    serializer = ClassSerializer()
    serializer.DefineAttribute("player_data", Type_class(NewPlayerData))
    
    def __init__(self, player_data=None):
        self.player_data = player_data



class GameMessageDeletePlayer:
        
    type = 61
    serializer = ClassSerializer()
    serializer.DefineAttribute("player_id", Type_u16)
    
    def __init__(self, player_id=0):
        self.player_id = player_id



class GameMessageUpdatePlayer:
    
    type = 62
    serializer = ClassSerializer()
    serializer.DefineAttribute("time", Type_f64)
    serializer.DefineAttribute("player_id", Type_u16)
    serializer.DefineAttribute("lives", Type_u8)
    serializer.DefineAttribute("health", Type_u8)
    serializer.DefineAttribute("flight", Type_u8)
    serializer.DefineAttribute("weapon_name", Type_str)
    serializer.DefineAttribute("ammo_value", Type_u8)
    serializer.DefineAttribute("ammo_is_loading", Type_u8)
    serializer.DefineAttribute("aim_angle", Type_f32)
    
    def __init__(self, timeVal=0, player_id=0, lives=0, health=0, flight=0,
                 weapon_name="", ammo_value=0, ammo_is_loading=0,
                 aim_angle=0):
        self.time = timeVal
        self.player_id = player_id
        self.lives = lives
        self.health = health
        self.flight = flight
        self.weapon_name = weapon_name
        self.ammo_value = ammo_value
        self.ammo_is_loading = ammo_is_loading
        self.aim_angle = aim_angle



PLAYER_INPUT_UP          = 1
PLAYER_INPUT_DOWN        = 2
PLAYER_INPUT_LEFT        = 4
PLAYER_INPUT_RIGHT       = 8
PLAYER_INPUT_JUMP        = 16
PLAYER_INPUT_SHOOT       = 32


class PlayerInput:
    """
    the member 'input_byte' is a bit-wise OR of key pressed.
    for example, input_byte = PLAYER_INPUT_UP | PLAYER_INPUT_JUMP
    """
    serializer = ClassSerializer()
    serializer.DefineAttribute("time", Type_f64)
    serializer.DefineAttribute("input_byte", Type_u8)
        
    def __init__(self, time=0, input_byte=0):
        self.time = time
        self.input_byte = input_byte


class GameMessagePlayerInput:
        
    type = 63
    serializer = ClassSerializer()
    serializer.DefineAttribute("player_id", Type_u16)
    serializer.DefineAttribute("input_list", Type_list(Type_class(PlayerInput)))
    serializer.DefineAttribute("aim_angle", Type_f32)
        
    def __init__(self, player_id=0, input_list=[], aim_angle=0):
        self.player_id = player_id
        self.input_list = input_list
        self.aim_angle = aim_angle



class GameMessagePlayerCorrection:
        
    type = 64
    serializer = ClassSerializer()
    serializer.DefineAttribute("time", Type_f64)
    serializer.DefineAttribute("x", Type_f32)
    serializer.DefineAttribute("y", Type_f32)
    serializer.DefineAttribute("vx", Type_f32)
    serializer.DefineAttribute("vy", Type_f32)
    serializer.DefineAttribute("flight", Type_u8)
    
    def __init__(self, time=0, x=0, y=0, vx=0, vy=0, flight=0):
        self.time = time
        self.x = x
        self.y = y
        self.vx = vx
        self.vy = vy
        self.flight = flight
        
        
        
class GameMessageInitialStateRequest:
    type = 70
    serializer = ClassSerializer()



class GameMessageInitialStateResponse:
    
    type = 71
    serializer = ClassSerializer()
    serializer.DefineAttribute("player_list", Type_list(Type_class(NewPlayerData)))
    serializer.DefineAttribute("scene_state", Type_class(FullSceneState))
    
    def __init__(self):
        self.player_list = []
        self.scene_state = None
        
        
        

all_message_types = { ChatMessage.type: ChatMessage,
                      JoinRequest.type: JoinRequest,
                      JoinResponse.type: JoinResponse,
                      GameInfoRequest.type: GameInfoRequest,
                      GameInfoResponse.type: GameInfoResponse,
                      MessageStartGame.type: MessageStartGame,
                      CreatedObjectsRequest.type: CreatedObjectsRequest,
                      CreatedObjectsResponse.type: CreatedObjectsResponse,
                      # SceneMessageCreateNewEntity.type: SceneMessageCreateNewEntity,
                      SceneMessageDeleteEntity.type: SceneMessageDeleteEntity,
                      SceneMessageUpdateEntity.type: SceneMessageUpdateEntity,
                      SceneMessageRemoveLand.type: SceneMessageRemoveLand,
                      SceneMessageFullState.type: SceneMessageFullState,
                      GameMessageCreateNewPlayer.type: GameMessageCreateNewPlayer,
                      GameMessageDeletePlayer.type: GameMessageDeletePlayer,
                      GameMessageUpdatePlayer.type: GameMessageUpdatePlayer,
                      GameMessagePlayerInput.type: GameMessagePlayerInput,
                      GameMessagePlayerCorrection.type: GameMessagePlayerCorrection,
                      GameMessageInitialStateRequest.type: GameMessageInitialStateRequest,
                      GameMessageInitialStateResponse.type: GameMessageInitialStateResponse }

