import cPickle
from defines import *


class INetObject:
    """ An interface for objects that can be sent over the network.
    The user is responsible to provide the type member to identify the
    class of the object.
    Serialization and deserialization happens automatically by this class.
    """
    
    type = -1

    def ToString(self):
        """ serialization method. """
        return cPickle.dumps(self, cPickle.HIGHEST_PROTOCOL)
        
    def staticmethod_FromString(data_string):
        """ FromString is a static method that returns """
        return cPickle.loads(data_string)

    def Print(self): return
    
    FromString = staticmethod(staticmethod_FromString)



#============================================================================#
# User defined messeges
#============================================================================#


class TextMessege(INetObject):
    """ used by test_client/server demo """
    type = 1
    def __init__(self, text):
        self.text = text


# ------------------| Game Initialization Messages |------------------

class JoinRequest(INetObject):
    type = 2
    def __init__(self, player_name):
        self.player_name = player_name
        self.client_version = FLAMEINGO_PROTOCOL


class JoinResponse(INetObject):
    type = 3
    def __init__(self, value, reason, player_id=-1, entity_id=-1):
        self.value = value
        self.reason = reason
        self.player_id = player_id
        self.entity_id = entity_id


class ChatMessage(INetObject):
    type = 4
    def __init__(self, player_name, text):
        self.player_name = player_name
        self.text = text


class GameInfoRequest(INetObject):
    type = 5


class GameInfoResponse(INetObject):
    type = 6
    def __init__(self):
        self.level_file_name = ""
        self.level_oid = ""
        self.player_name = ""
        self.version = FLAMEINGO_PROTOCOL


class MessageStartGame(INetObject):
    type = 7


class CreatedObjectsRequest(INetObject):
    type = 8


class CreatedObjectsResponse(INetObject):
    type = 9


class EchoRequest(INetObject):
    type = 30
    def __init__(self):
        self.time = 0
        self.data = "this is a ping message"


class EchoReply(INetObject):
    type = 31
    def __init__(self):
        self.time = 0
        self.data = "this is a ping message"


# -----------------------| Scene Controller Messages |-----------------------

class SceneMessageCreateNewEntity(INetObject):
    type = 100
    def __init__(self, time, entityType, x=0, y=0, vx=0, vy=0, state=0, id=-1):
        self.time = time
        self.entityType = entityType
        self.x = x
        self.y = y
        self.vx = vx
        self.vy = vy
        self.state = state
        self.id = id
    def Print(self):
        print ""
        print "id:", self.id
        print "time:", self.time
        print "entityType:", self.entityType
        print ""


class SceneMessageDeleteEntity(INetObject):
    type = 101
    def __init__(self, time, id):
        self.time = time
        self.id = id


class SceneMessageUpdateEntity(INetObject):
    type = 102
    def __init__(self, time, id, x, y, vx, vy, state):
        self.time = time
        self.id = id
        self.x = x
        self.y = y
        self.vx = vx
        self.vy = vy
        self.state = state


class SceneMessageRemoveLand(INetObject):
    type = 120
    def __init__(self, time, x, y, shapeIndex):
        self.time = time
        self.id = id
        self.x = x
        self.y = y
        self.shapeIndex = shapeIndex


# -----------------------| Game Controller Messages |-----------------------

class GameMessageCreateNewPlayer(INetObject):
    type = 200
    def __init__(self, player_id, player_name, entity_id):
        self.player_id = player_id
        self.player_name = player_name
        self.entity_id = entity_id


class GameMessageDeletePlayer(INetObject):
    type = 201
    def __init__(self, player_id):
        self.player_id = player_id


class GameMessageUpdatePlayer(INetObject):
    type = 202
    def __init__(self, timeVal, player_id, lives, health, flight,
                 weapon_name, ammo_value, ammo_is_loading,
                 aim_angle):
        self.time = timeVal
        self.player_id = player_id
        self.lives = lives
        self.health = health
        self.flight = flight
        self.weapon_name = weapon_name
        self.ammo_value = ammo_value
        self.ammo_is_loading = ammo_is_loading
        self.aim_angle = aim_angle



PLAYER_MOVE_LEFT   = 0
PLAYER_MOVE_RIGHT  = 1
PLAYER_AIM_UP      = 2
PLAYER_AIM_DOWN    = 3
PLAYER_MOVE_JUMP   = 4
PLAYER_SHOOT       = 5


class GameMessagePlayerMove(INetObject):
    type = 220
    def __init__(self, time, player_id, action, arg1=0):
        self.time = time
        self.player_id = player_id
        self.action = action
        self.arg1 = arg1



class GameMessageSetPlayerAimAngle(INetObject):
    type = 221
    def __init__(self, player_id, angle):
        self.player_id = player_id
        self.angle = angle



class PlayerMoveClientPredictionFlaw(INetObject):
    type = 230
    def __init__(self, flaw):
        self.flaw = flaw








