"""
High level network module.

This module implements a non blocking messeging queue over TCP.
This module does not use threads.
Rather, the user of this module must create a Net object, and call its 
Poll() method periodically.
(It may easlly be wrapped with a thread which does this polling, if you really
really want)

The main feature of this module is sending objects between remote hosts.
A remote host is represented by the RemoteHost class.
Any object to be sent must inherit from INetObject.
"""


import struct
from socket import *
import select
from m_Messeges import *



# for debug

##from game.m_EventSchedualer import Schedualer
##
##from sys import stdout
##
##def print_binary_data(data):
##    for ch in data:
##        stdout.write(hex(ord(ch)) + "  ")
##    print ""


##import random, time
##
##DEBUG_SENDER_SLOW_NETWORK     = 0
##DEBUG_SENDER_MEDIUM_NETWORK   = 1
##
##class DebugSender:
##
##    def __init__(self, network_type):
##        self.schedualer = Schedualer()
##        self.schedualer.StartTimer()
##        self.last_msg_arive_time = 0
##        if network_type == DEBUG_SENDER_SLOW_NETWORK:
##            self.BYTES_PER_SECOND = 10000
##            self.NET_DELAY_MIN = 75     # milliseconds
##            self.NET_DELAY_MAX = 90     # milliseconds
##        elif network_type == DEBUG_SENDER_MEDIUM_NETWORK:
##            self.BYTES_PER_SECOND = 10000
##            self.NET_DELAY_MIN = 20       # milliseconds
##            self.NET_DELAY_MAX = 32.5     # milliseconds
##        else:
##            raise Exception("debugger network type not specified")
##
##    def Poll(self):
##        self.schedualer.Poll()
##        
##    def send(self, socket, data):
##        l = len(data)
##        host_delay = float(l) / self.BYTES_PER_SECOND
##        time.sleep(host_delay)
##        self.schedualer.Poll()
##        t = self.schedualer.time
##        net_delay = 0.001 * random.uniform(self.NET_DELAY_MIN, self.NET_DELAY_MAX)
##        if net_delay + t < self.last_msg_arive_time:
##            net_delay = self.last_msg_arive_time - t + 0.001 * random.uniform(0, self.NET_DELAY_MIN)
##        self.last_msg_arive_time = t + net_delay
##        # print "Net delay:", net_delay, "\t Post: ", + t + net_delay
##        self.schedualer.Post(net_delay, socket.sendall, data)
##            
##
##debugSender = DebugSender(DEBUG_SENDER_MEDIUM_NETWORK)
        


class _netobject:

    def __init__(self):
        self.error_handlers = []
        
    def RegisterErrorHandler(self, handlerMethod):
        self.error_handlers.append(handlerMethod)
        
    def NotifyError(self, text):
        for h in self.error_handlers:
            h(self, text)
            
    def Poll(self):
        raise "_netobject.Poll -- un implemented method!"

    def PollRead(self): return 0
    def PollWrite(self): return 0
        
 
       
class _messege_queue_protocol:
    """ a protocol for encapsulating messeges in a stream.
    the protocol uses 4 bytes header specifing the messege length.
    one side of the protocol is sending messeges by calling EncodeMessege
    and sending the resulting string over a stream.
    The other side reads chunks from the stream and writes it to the protocol
    buffer. then call FetchMesseges to fetch messeges from the buffer.
    """

    def __init__(self):
        self.buffer = ""
        self.in_header = True
        self.next_len = 4
        
    def EncodeMessege(self, data_string):
        return struct.pack("L", len(data_string)) + data_string
        
    def FetchMesseges(self):
        result = []
        while self.next_len <= len(self.buffer):
           if self.in_header:
               self.in_header = False
               self.next_len = int(struct.unpack("L", self.buffer[:4])[0])
               self.buffer = self.buffer[4:]
           else:
               result.append(self.buffer[:self.next_len])
               self.buffer = self.buffer[self.next_len:]
               self.next_len = 4
               self.in_header = True
        return result   
        
    

class Net:
    """ A framework object.
    You should always have one Net object, and you must call Poll()
    periodically.
    """
    
    def __init__(self):
        self.my_objects = []

    def Poll(self):
        for o in self.my_objects:
            o.Poll()
        
    def PollRead(self):
        for o in self.my_objects:
            o.PollRead()
        
    def PollWrite(self):
        for o in self.my_objects:
            o.PollWrite()
    
    def AddChild(self, obj):
        """ for internal use """
        self.my_objects.append(obj)

    def RemoveChild(self, obj):
        """ for internal use """
        if obj in self.my_objects:
            self.my_objects.remove(obj)




class RemoteHost(_netobject):

    def __init__(self, net, ip_addr, port):
        _netobject.__init__(self)
        self.object_received_handlers = {}
        self.socket = None
        self.net = net
        self.port = port
        self.ip_addr = ip_addr
        self.msg_protocol = _messege_queue_protocol()
        self.send_buffer = ""
        
    def Connect(self):
        try:
            self.socket = socket()
            self.socket.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
            self.socket.connect((self.ip_addr, self.port))
        except:
            self.NotifyError('error connecting to ' + self.ip_addr + ' using port ' + str(self.port))
            return False
        self.net.AddChild(self)
        return True
        
    def Disconnect(self):
        self.net.RemoveChild(self)
        self.socket.close()
    
    def SendObject(self, obj):
        self.send_buffer += self.msg_protocol.EncodeMessege(obj.ToString())
        self.PollWrite()
    
    def RegisterObjectReceivedHandler(self, objType, handleMethod):
        if self.object_received_handlers.has_key(objType):
            self.object_received_handlers[objType].append(handleMethod)
        else:
            self.object_received_handlers[objType] = [handleMethod]


    def PollWrite(self):
        if len(self.send_buffer) == 0:
            return 0
        try:
            ret = select.select([], [self.socket], [], 0)[1]
            if len(ret) > 0:
                l = self.socket.send(self.send_buffer)
                self.send_buffer = self.send_buffer[l:]
                return l
        except:
            self.NotifyError('connection error')
            self.Disconnect()
            return 0
        

    def PollRead(self):
        ret = None
        data = None
        try:
            ret = select.select([self.socket], [], [], 0)[0]
        except:
            self.NotifyError('connection error')
            self.Disconnect()
            return 0
        
        if len(ret) > 0:
            try:
                data = self.socket.recv(32000)
            except:
                self.NotifyError('connection error')
                self.Disconnect()
                return 0
                
            if len(data) == 0:
                self.NotifyError('connection closed by remote peer')
                self.Disconnect()
                return 0
            self.msg_protocol.buffer += data
            for msg in self.msg_protocol.FetchMesseges():
                obj = None
                try:
                    obj = INetObject.FromString(msg)
                except:
                    self.NotifyError('unknown message received from remote peer. closing connection.')
                    self.Disconnect()
                    return 0
                # obj.Print()
                if self.object_received_handlers.has_key(obj.type):
                    for h in self.object_received_handlers[obj.type]:
                        h(self, obj)
            return len(data)

        
    def Poll(self):
        while self.PollRead()  > 0: pass
        while self.PollWrite() > 0: pass


class NetListener(_netobject):
    """ A listener for RemoteHost objects. """
    def __init__(self, net, port):
        _netobject.__init__(self)
        self.new_connection_handlers = []
        self.port = port
        self.socket = None
        self.net = net
    
    def RegisterNewConnectionHandler(self, handlerMethod):
        self.new_connection_handlers.append(handlerMethod)
    
    def Start(self):
        try:
            self.socket = socket()
            self.socket.bind(('', self.port))
            self.socket.listen(10)
        except:
            self.NotifyError('error binding to port ' + str(self.port))
            return False
        self.net.AddChild(self)
        return True
        
    def Stop(self):        
        self.net.RemoveChild(self)
        self.socket.close()

    def Poll(self):
        new_sock = None
        addr = None
        while True:
            try:
                if len(select.select([self.socket], [], [], 0)[0]) == 0:
                    break
                new_sock, addr = self.socket.accept()
                new_sock.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
            except:
                self.NotifyError('error while listening for connections.')
                self.Stop()
                break
            host = RemoteHost(self.net, addr[0], addr[1])
            host.socket = new_sock
            self.net.AddChild(host)
            for h in self.new_connection_handlers:
                h(host)




