import sys, math
import os, time, random
from defines import *
from scene import *
from util import CircularList
from weapons import WEAPON_CLASSES
from game.m_Controllers import *
from game.m_Game import BaseGame, FRAME_EVT
from game.m_Charactors import *
from game.m_EntityTypes import *
from game.m_EventSchedualer import *
from game.m_Dashboard import DashboardRep, RemotePlayerDashboardListRep
from game.m_Players import Player, PlayerCamera
from graphics import Sprite, SpriteRes, ReallyStopGameDialog
from m_ChatBox import *
from net import *
import pygame
from pygame.locals import *



SEND_PLAYER_INPUT_INTERVAL_SECONDS       = 0.03
POSITION_CORRECTION_THRESHOLD_PIXELS     = 5
MAX_NUM_OF_INPUT_RETRANSMISSIONS         = 3
INPUT_RETRANSMISSION_EXPIRED_SECONDS     = 0.12


class ClientGameConfig:
    def __init__(self):
        self.net = None
        self.screen = None
        self.scene = None
        self.scene_rep = None
        self.player_name = ""
        self.player_id = ""
        self.level_file_name = ""
        remote_host = None


class ClientGameInitiator:

    def __init__(self, net, player_name):
        self.net = net
        self.client_game_config = ClientGameConfig()
        self.client_game_config.net = self.net
        self.client_game_config.player_name = player_name
        self.connection_progress_callbacks = []
        self.error_callbacks = []
        self.player_chat_msg_callbacks = []
        self.game = None
        self.isGameRunning = False

    def InitConnection(self, ip_addr, port):
        source_port = 52002
        while (self.net.Init(source_port) == False) and source_port <= 52010:
            source_port += 1
        if source_port == 52011:
            return False
        self.remote_host = self.net.CreateNewConnection(ip_addr, port)
        if self.remote_host == None:
            return False
        self.client_game_config.remote_host = self.remote_host
        self.remote_host.RegisterConnectionClosedHandler(self.HandleConnectionClosed)
        self.remote_host.RegisterObjectReceivedHandler(JoinResponse.type, self.HandleJoinResponse)
        self.remote_host.RegisterObjectReceivedHandler(GameInfoResponse.type, self.HandleGameInfoResponse)
        self.remote_host.RegisterObjectReceivedHandler(ChatMessage.type, self.HandleChatMessage)
        self.remote_host.RegisterObjectReceivedHandler(MessageStartGame.type, self.HandleStartGameMessage)
        self.remote_host.RegisterObjectReceivedHandler(GameMessageCreateNewPlayer.type, self.HandleGameMessageCreateNewPlayer)
        self.remote_host.RegisterObjectReceivedHandler(GameMessageDeletePlayer.type, self.HandleGameMessageDeletePlayer)
        self.remote_host.RegisterObjectReceivedHandler(GameMessageInitialStateResponse.type, self.HandleGameMessageInitialStateResponse)
        self.remote_host.SendObject(GameInfoRequest(), FLAG_ACK_REQUIRED)
        self.connection_progress = 1
        for cb in self.connection_progress_callbacks:
            cb(self.connection_progress, "waiting for game info")
        return True
    
    def HandleGameMessageInitialStateResponse(self, sender, msg):
        if self.connection_progress != 4: return
        self.connection_progress = 5
        text = "currently connected players: "
        self.game.sceneController.HandleFullSceneStateObject(msg.scene_state)
        for player_info in msg.player_list:
            self.game.gameController.InternalCreateNewPlayer(player_info)
            text += player_info.player_name + ", "
        text = text[:len(text) - 2] + ". waiting for game to begin"
        self.game.InitPlayer(self.client_game_config.player_id,
                             self.client_game_config.entity_id)
        for cb in self.connection_progress_callbacks:
            cb(self.connection_progress, text)
        
    def HandleGameMessageCreateNewPlayer(self, sender, msg):
        if self.connection_progress < 4: return
        for player_id in self.game.gameController.GetAllPlayerIds():
            if self.game.gameController.GetPlayerName(player_id) == msg.player_data.player_name:
                return
        for cb in self.connection_progress_callbacks:
            cb(self.connection_progress, msg.player_data.player_name + " has joined the game")

    def HandleGameMessageDeletePlayer(self, sender, msg):
        if self.connection_progress < 4 or not self.game.gameController.PlayerExists(msg.player_id): 
            return
        player_name = self.game.gameController.GetPlayerName(msg.player_id)
        for cb in self.connection_progress_callbacks:
            cb(999, player_name + " has left the game")

    def HandleConnectionClosed(self, connection):
        self.connection_progress = -1
        if self.game != None: self.game.Quit()
        self.game = None
        self.isGameRunning = False
        for cb in self.error_callbacks: cb("connection lost")

    def IsGameRunning(self):
        return self.isGameRunning

    def HandleGameInfoResponse(self, sender, msg):
        retVal = True
        text = ""
        if self.connection_progress != 1: return

        if msg.version != FLAMEINGO_VERSION:
            retVal = False
            text = 'incompatible versions. server protocol is ' + msg.version

        if retVal:            
            levelRes = LevelResource()
            try:
                self.client_game_config.level_file_name = os.path.join('data', 'levels', msg.level_file_name)
                levelRes.Load(self.client_game_config.level_file_name)
                if levelRes.id != msg.level_oid:
                    retVal = False
                    text = 'you do have the level "' + msg.level_file_name + '", but its content is different'
            except:
                    retVal = False
                    text = 'you dont have the level "' + msg.level_file_name + '"'
                
        if retVal:
            request = JoinRequest(self.client_game_config.player_name)
            self.remote_host.SendObject(request, FLAG_ACK_REQUIRED)
            self.connection_progress = 2
            for cb in self.connection_progress_callbacks:
                cb(self.connection_progress, "Level: " + msg.level_file_name)
                cb(self.connection_progress, "Server player name: " + msg.player_name)
                cb(self.connection_progress, "waiting to be granted by the remote guy")
        else:
            self.connection_progress = -1
            for cb in self.connection_progress_callbacks:
                cb(-1, text)

    def HandleStartGameMessage(self, sender, msg):
        if self.connection_progress != 5: return
        self.isGameRunning = True
        self.game.schedualer.StartTimer()
        self.connection_progress = 200
        for cb in self.connection_progress_callbacks:
            cb(200, "Game has started")

    def HandleJoinResponse(self, sender, msg):
        if self.connection_progress != 2: return
        text = ""
        if msg.value:
            text = "Granted. waiting for more game info"
            self.client_game_config.player_id = msg.player_id
            self.client_game_config.entity_id = msg.entity_id
            self.connection_progress = 3
        else:
            text = "Bad luck. you were rejected for the following reason: " + msg.reason
            self.connection_progress = -1
        for cb in self.connection_progress_callbacks:
            cb(self.connection_progress, text)
            
    def SendChatMessage(self, text):
        msg = ChatMessage(self.client_game_config.player_name, text)
        self.remote_host.SendObject(msg)
        
    def HandleChatMessage(self, sender, msg):
        for cb in self.player_chat_msg_callbacks:
            cb(msg.player_name, msg.text)

    def Close(self):
        self.remote_host.Close()
        self.net.Close()

    def CreateNewGame(self):
        self.connection_progress = 4
        self.game = ClientGame(self.client_game_config)
        self.game.chatbox.inputCallbacks.append(self.SendChatMessage)
        self.remote_host.RegisterObjectReceivedHandler(ChatMessage.type, self.game.chatbox.HandleChatMessage)
        msg = GameMessageInitialStateRequest()
        self.remote_host.SendObject(msg)
        
    def StartGame(self):
        if self.connection_progress != 200: return
        self.game.Run()



        
#===============================================================================
# clientSceneController
#===============================================================================

class ClientSceneController:
    
    def __init__(self, scene, game):
        self.localSceneController = LocalSceneController(scene)
        self.game = game
        self.scene = scene

    def CreateNewEntity(self, entityType, x=0, y=0, vx=0, vy=0, state=0, id=-1):
        return -1

    def DeleteEntity(self, id):
        return

    def SetEntityState(self, id, state):
        self.localSceneController.SetEntityState(id, state)

    def SetEntityVelocity(self, id, vx, vy):
        self.localSceneController.SetEntityVelocity(id, vx, vy)
        
    def SetEntityPos(self, id, x, y):
        self.localSceneController.SetEntityPos(id, x, y)

    def GetEntityPos(self, id):
        if self.localSceneController.EntityExists(id) == False:
            return (0, 0)
        return self.localSceneController.GetEntityPos(id)

    def GetEntityPosCenter(self, id):
        if self.localSceneController.EntityExists(id) == False:
            return (0, 0)
        return self.localSceneController.GetEntityPosCenter(id)
    
    def GetEntityVelocity(self, id):
        if self.localSceneController.EntityExists(id) == False:
            return (0, 0)
        return self.localSceneController.GetEntityVelocity(id)
    
    def GetEntityState(self, id):
        if self.localSceneController.EntityExists(id) == False:
            return 0
        return self.localSceneController.GetEntityState(id)

    def EntityExists(self, id):
        return self.localSceneController.EntityExists(id)

    def AttachObserverToEntity(self, id, observer):
        if self.localSceneController.EntityExists(id) == False:
            return
        self.localSceneController.AttachObserverToEntity(id, observer)
    
    def RemoveLand(self, x, y, shapeIndex):
        return
        
    def CreateExplosion(self, x, y, vx, vy, radius, power):
        return
    
    def UpdateEntityFromServer(self, server_entity_data):
        if self.localSceneController.EntityExists(server_entity_data.id):
            if self.game.player != None and server_entity_data.id == self.game.player.charactor.entity_id: return
            self.localSceneController.SetEntityPos(server_entity_data.id, server_entity_data.x, server_entity_data.y)
            self.localSceneController.SetEntityVelocity(server_entity_data.id, server_entity_data.vx, server_entity_data.vy)
            self.localSceneController.SetEntityState(server_entity_data.id, server_entity_data.state)
        else:
            self.localSceneController.CreateNewEntity(server_entity_data.entityType, 
                                                        server_entity_data.x, 
                                                        server_entity_data.y, 
                                                        server_entity_data.vx, 
                                                        server_entity_data.vy, 
                                                        server_entity_data.state,
                                                        server_entity_data.id)
    
    def HandleFullSceneStateObject(self, obj):
        local_id_list = self.localSceneController.GetEntityIdList()
        for entity in obj.entity_list:
            if entity.id in local_id_list:
                local_id_list.remove(entity.id)
            self.UpdateEntityFromServer(entity)
        for entity_id in local_id_list:
            self.localSceneController.DeleteEntity(entity_id)

    def AdvanceEntity(self, entity_id, time):
        self.localSceneController.AdvanceEntity(entity_id, time)

        
#===============================================================================
# ClientGameController
#===============================================================================


class PlayerMove:
    def __init__(self, time, x, y, vx, vy, input_byte):
        self.time = time
        self.x = x
        self.y = y
        self.vx = vx
        self.vy = vy
        self.input_byte = input_byte
        

class ClientGameController:

    def __init__(self, game):
        self.game = game
        self.host = self.game.host
        self.game.schedualer.PostAt(0, self.SendPlayerInputAndPostAgain)
        self.localGameController = LocalGameController(self.game)
        self.current_input_byte = 0
        self.player_move_history = CircularList(1024)
        self.last_input_sent_time = 0
        self.inputs_to_send_again = CircularList(MAX_NUM_OF_INPUT_RETRANSMISSIONS)
        
    def RegisterMove(self):
        entity_id = self.GetPlayerEntityId(self.game.player.id)
        x, y = self.game.sceneController.GetEntityPos(entity_id)
        vx, vy = self.game.sceneController.GetEntityVelocity(entity_id)
        move = PlayerMove(self.game.time, x, y, vx, vy, self.current_input_byte)
        self.player_move_history.append(move)
        self.current_input_byte = 0
        
    def SendPlayerInputAndPostAgain(self):
        if self.localGameController.IsPlayerAlive(self.game.player.id) == False:
            return
        self.game.schedualer.Post(SEND_PLAYER_INPUT_INTERVAL_SECONDS, self.SendPlayerInputAndPostAgain)
        if self.last_input_sent_time != self.game.time:
            # if a key pressed, input already sent asynchronously
            self.SendPlayerInput()
        
    def SendPlayerInput(self, send_again_next_time=False):
        first_input_to_resend = self.inputs_to_send_again.get_head()
        while ( first_input_to_resend != None and 
                self.game.time - first_input_to_resend.time > INPUT_RETRANSMISSION_EXPIRED_SECONDS):
            first_input_to_resend = self.inputs_to_send_again.pop_head()
        inputs = self.inputs_to_send_again.get_all_items()
        this_input = PlayerInput(self.game.time, self.current_input_byte)
        inputs.append(this_input)
        msg = GameMessagePlayerInput(self.game.player.id, inputs, self.game.player.aim_marker.angle)
        self.host.SendObject(msg)
        self.RegisterMove()
        self.last_input_sent_time = self.game.time
        if send_again_next_time:
            self.inputs_to_send_again.append(this_input)
        
    def CorrectPlayer(self, msg):
        player = self.game.playerIdMap[self.game.player.id]
        entity_id = self.GetPlayerEntityId(player.id)
        self.game.sceneController.SetEntityPos(entity_id, msg.x, msg.y)
        self.game.sceneController.SetEntityVelocity(entity_id, msg.vx, msg.vy)
        player.dashboard.flight.value = msg.flight
        prev_move = self.player_move_history.pop_head()
        if prev_move == None: return
        moves = self.player_move_history.get_all_items()
        for move in moves:
            dt = (move.time - prev_move.time) * 1000
            self.game.sceneController.AdvanceEntity(entity_id, dt)
            player.dashboard.flight.Advance(dt)
            if move.input_byte & PLAYER_INPUT_LEFT: 
                self.localGameController.PlayerMoveLeft(self.game.player.id)
            if move.input_byte & PLAYER_INPUT_RIGHT: 
                self.localGameController.PlayerMoveRight(self.game.player.id)
            if move.input_byte & PLAYER_INPUT_JUMP: 
                self.localGameController.PlayerMoveJump(self.game.player.id)
            move.x, move.y = self.game.sceneController.GetEntityPos(entity_id)
            move.vx, move.vy = self.game.sceneController.GetEntityVelocity(entity_id)
            prev_move = move
        dt = (self.game.time - prev_move.time) * 1000
        self.game.sceneController.AdvanceEntity(entity_id, dt)
        player.dashboard.flight.Advance(dt)
        
    def CreateNewPlayer(self, player_name, id=-1):
        print "[WARNING] ClientGameController.CreateNewPlayer: This method is depricated in the client game"

    def DeletePlayer(self, id):
        print "[WARNING] ClientGameController.DeletePlayer: This method is depricated in the client game"

    def DecreasePlayerHealth(self, id, value):
        return False

    def IncreasePlayerHealth(self, id, value):
        return
    
    def PlayerMoveLeft(self, id):
        self.current_input_byte = self.current_input_byte | PLAYER_INPUT_LEFT
        self.localGameController.PlayerMoveLeft(id)
        self.SendPlayerInput(True)

    def PlayerMoveRight(self, id):
        self.current_input_byte = self.current_input_byte | PLAYER_INPUT_RIGHT
        self.localGameController.PlayerMoveRight(id)
        self.SendPlayerInput(True)
    
    def PlayerMoveJump(self, id):
        self.current_input_byte = self.current_input_byte | PLAYER_INPUT_JUMP
        self.localGameController.PlayerMoveJump(id)
        self.SendPlayerInput(True)

    def PlayerShoot(self, id):
        self.current_input_byte = self.current_input_byte | PLAYER_INPUT_SHOOT
        self.SendPlayerInput(True)

    def PlayerAimUp(self, id, time):
        self.localGameController.PlayerAimUp(id, time)
        self.current_input_byte = self.current_input_byte | PLAYER_INPUT_UP
        
    def PlayerAimDown(self, id, time):
        self.localGameController.PlayerAimDown(id, time)
        self.current_input_byte = self.current_input_byte | PLAYER_INPUT_DOWN

    def GetAllPlayerIds(self):
        return self.localGameController.GetAllPlayerIds()

    def GetPlayerEntityId(self, player_id):
        return self.localGameController.GetPlayerEntityId(player_id)
    
    def GetPlayerAimXY(self, player_id):
        return self.localGameController.GetPlayerAimXY(player_id)
            
    def GetPlayerName(self, player_id):
        return self.localGameController.GetPlayerName(player_id)
        
    def GetPlayerLives(self, player_id):
        return self.localGameController.GetPlayerLives(player_id)

    def GetPlayerHealth(self, player_id):
        return self.localGameController.GetPlayerHealth(player_id)

    def PlayerExists(self, player_id):
        return self.localGameController.PlayerExists(player_id)
    
    def GetPlayerByEntityId(self, entity_id):
        return self.localGameController.GetPlayerByEntityId(entity_id)

    def SetTimer(self, timeout, func, args):
        return
    
    def InternalCreateNewPlayer(self, new_player_data):
        entity_data = new_player_data.entity_data
        if self.game.playerIdMap.has_key(new_player_data.player_id): return
        self.game.sceneController.UpdateEntityFromServer(entity_data)
        id = self.localGameController.CreateNewPlayer(new_player_data.player_name, 
                                                      new_player_data.player_id, 
                                                      entity_data.id)
        player = self.game.playerIdMap[id]
        self.game.players.append(player)
        if new_player_data.player_name != self.game.player_name:
            self.game.remotePlayerDashboardList.append(player.dashboard)
    
    def InternalDeletePlayer(self, player_id):
        if not self.game.playerIdMap.has_key(player_id): return
        player = self.game.playerIdMap[player_id]
        self.game.players.remove(player)
        self.localGameController.DeletePlayer(player_id)
        if player.dashboard in self.game.remotePlayerDashboardList:
            self.game.remotePlayerDashboardList.remove(player.dashboard)
            if self.game.started:
                self.game.camera_bottom_dlg.Open(player.name + " has quit")


#===============================================================================
# ClientGameMessageHandler
#===============================================================================


class ClientGameMessageHandler:

    def __init__(self, game):
        self.last_update_time = -1
        self.game = game
        self.localSceneController = LocalSceneController(self.game.scene)
        self.localGameController = LocalGameController(self.game)
        
    def _validate_msg_time_(self, time):
        if time > self.last_update_time:
            self.last_update_time = time
            return True
        else:
            return False

#     def HandleSceneMessageCreateNewEntity(self, sender, msg):
#         if not self._validate_msg_time_(msg.time): return
#         if msg.entity.id in self.localSceneController.scene.objectMapById: return
#         id = self.localSceneController.CreateNewEntity(msg.entity.entityType, 
#                                                        msg.entity.x, 
#                                                        msg.entity.y, 
#                                                        msg.entity.vx, 
#                                                        msg.entity.vy, 
#                                                        msg.entity.state, 
#                                                        msg.entity.id)
#         entity = self.localSceneController.scene.objectMapById[id]

    def HandleSceneMessageDeleteEntity(self, sender, msg):
        self.localSceneController.DeleteEntity(msg.id)

    def HandleSceneMessageRemoveLand(self, sender, msg):
        self.localSceneController.RemoveLand(msg.x, msg.y, msg.shapeIndex)

    def HandleSceneMessageUpdateEntity(self, sender, msg):
        if not self._validate_msg_time_(msg.time): return
        self.game.sceneController.UpdateEntityFromServer(msg.entity)

    def HandleGameMessageCreateNewPlayer(self, sender, msg):
        self.game.gameController.InternalCreateNewPlayer(msg.player_data)
    
    def HandleGameMessageDeletePlayer(self, sender, msg):
        self.game.gameController.InternalDeletePlayer(msg.player_id)

    def HandleGameMessageUpdatePlayer(self, sender, msg):
        if not self.game.playerIdMap.has_key(msg.player_id): return
        player = self.game.playerIdMap[msg.player_id]
        if msg.lives < player.dashboard.lives.value:
            if msg.lives == 0:
                self.game.NotifyPlayerLoss(player)
            else:
                self.game.NotifyPlayerKilled(player)
        player.dashboard.lives.value = msg.lives
        player.dashboard.health.value = msg.health
        player.dashboard.flight.value = msg.flight
        player.dashboard.weapon_name.value = msg.weapon_name
        player.dashboard.ammo.value = msg.ammo_value
        player.dashboard.ammo.loading = msg.ammo_is_loading
        player.dashboard.flight.loading = True
        if player != self.game.player:
            player.SetAimAngle(msg.aim_angle)

    def HandleSceneMessageFullState(self, sender, msg):
        if not self._validate_msg_time_(msg.time): return
        self.game.sceneController.HandleFullSceneStateObject(msg.scene_state)
    
    def HandleGameMessagePlayerCorrection(self, sender, msg):
        if self.localGameController.IsPlayerAlive(self.game.player.id) == False:
            return
        while True:
            move = self.game.gameController.player_move_history.get_head()
            if move == None: break
            if move.time < msg.time: 
                self.game.gameController.player_move_history.pop_head()
            elif move.time == msg.time:
                if ( (abs(move.x - msg.x) > POSITION_CORRECTION_THRESHOLD_PIXELS) or
                     (abs(move.y - msg.y) > POSITION_CORRECTION_THRESHOLD_PIXELS) ):
#                     print ""
#                     print "move.x:", move.x, "\t msg.x:", msg.x
#                     print "move.y:", move.y, "\t msg.y:", msg.y
#                     print "move.vx:", move.vx, "\t msg.vx:", msg.vx
#                     print "move.vy:", move.vy, "\t msg.vy:", msg.vy
#                     print "now:", self.game.time, "\t msg.time:", msg.time
                    self.game.gameController.CorrectPlayer(msg)
                break
            else:
                break
    

#===============================================================================
# ClientGame
#===============================================================================

class ClientGame(BaseGame):
    
    FRAME_DELAY = 10.0   # milliseconds
    PAINT_FRAME_SKIP = 4
    
    def NotifyPlayerKilled(self, player):
        if player == self.player:
            self.camera_bottom_dlg.Open("you were killed once")
        else:
            self.camera_bottom_dlg.Open(player.name + " was killed once")

    def NotifyPlayerLoss(self, player):
        if player == self.player and (self.keyboard in self.keyboards):
            self.keyboards.remove(self.keyboard)
        if player == self.player:
            self.camera_bottom_dlg.Open("you are gone for good!")
        else:
            self.camera_bottom_dlg.Open(player.name + " is gone for good!")

    def InitPlayer(self, player_id, entity_id):
        """
        make sure to call this method only after the game's scene really have
        the specified entity_id, and the specified player_id has been created
        as well.
        """
        self.player = self.playerIdMap[player_id]
        if self.player == None:
            raise Exception("the specified player id does not yet exist in the client game")
        if self.sceneController.EntityExists(entity_id) == False:
            raise Exception("the specified entity id does not yet exist in the client game")

        # self.players.append(self.player)
        # no need to. controller has put it there earlier
        
        ##
        ## create keyboard
        ##
        self.keyboard = PlayerKeyboard(KEY_MAPPING_RIGHT,
                                       self.gameController,
                                       self.player.id)
        
        self.keyboards.append(self.keyboard)
        ##
        ## create DashboardRep object
        ##
        self.dashboard_rep = DashboardRep(self.player.dashboard,
                                          self.screen.subsurface([205, 410, 219, 69]) )
        ##
        ## create camera
        ##
        self.camera = PlayerCamera(self.player.id,
                                   self,
                                   self.camera_surface)
        

    def __init__(self, game_config):
        BaseGame.__init__(self)
        self.started = False
        self.schedualer = Schedualer()
        self.player_name = game_config.player_name
        self.net = game_config.net
        self.host = game_config.remote_host
        self.screen = game_config.screen
        self.scene = game_config.scene
        self.scene_rep = game_config.scene_rep
        self.sceneController = ClientSceneController(self.scene, self)
        self.gameController = ClientGameController(self)
        self.quit = False
        self.players = []
        self.time = 0
        self.camera_surface = self.screen.subsurface([200, 23, 430, 366])

        self.remotePlayerDashboardList = []
        self.remotePlayerDashboardListRep = RemotePlayerDashboardListRep(
            self.remotePlayerDashboardList,
            self.screen.subsurface((20, 40, 160, 420)))
        
        # player related objects. shall be created later by InitPlayer()
        self.player = None
        self.keyboard = None
        self.dashboard_rep = None
        self.camera = None
        
        self.messageHandler = ClientGameMessageHandler(self)
        #self.host.RegisterObjectReceivedHandler(SceneMessageCreateNewEntity.type, self.messageHandler.HandleSceneMessageCreateNewEntity)
        self.host.RegisterObjectReceivedHandler(SceneMessageDeleteEntity.type, self.messageHandler.HandleSceneMessageDeleteEntity)
        self.host.RegisterObjectReceivedHandler(SceneMessageRemoveLand.type, self.messageHandler.HandleSceneMessageRemoveLand)
        self.host.RegisterObjectReceivedHandler(SceneMessageUpdateEntity.type, self.messageHandler.HandleSceneMessageUpdateEntity)
        self.host.RegisterObjectReceivedHandler(GameMessageCreateNewPlayer.type, self.messageHandler.HandleGameMessageCreateNewPlayer)
        self.host.RegisterObjectReceivedHandler(GameMessageDeletePlayer.type, self.messageHandler.HandleGameMessageDeletePlayer)
        self.host.RegisterObjectReceivedHandler(GameMessageUpdatePlayer.type, self.messageHandler.HandleGameMessageUpdatePlayer)
        self.host.RegisterObjectReceivedHandler(SceneMessageFullState.type, self.messageHandler.HandleSceneMessageFullState)
        self.host.RegisterObjectReceivedHandler(GameMessagePlayerCorrection.type, self.messageHandler.HandleGameMessagePlayerCorrection)
        
        ##
        ## background images
        ##
        self.img_background  = pygame.image.load(
            os.path.join('data', 'app', 'server_game_bg.png')).convert()
        
        ##
        ## define collisions to be detected
        ##

        ## ahh...! client is not interested with collision between entities

        ##
        ## create chat box
        ##
        self.chatbox = ChatBox(self, game_config.player_name)
        
        ##
        ## initialize dialogs
        ##
        self.reallyQuitDialog = ReallyStopGameDialog()
        self.reallyQuitDialog.value_returned_callbacks.append(self.HandleReallyQuitDialogValue)
        
        self.camera_bottom_dlg = CameraBottomTextDialog(self.schedualer,
                                                        self.camera_surface)
        
        self.dialogs = [self.camera_bottom_dlg, self.reallyQuitDialog]
        
        ##
        ## declare the frame event
        ##
        pygame.event.Event(FRAME_EVT)


    def HandleReallyQuitDialogValue(self, dlg, val):
        dlg.Close()
        self.DrawBackground()
        if val == True: self.Quit()
        
    def Quit(self, arg=None):
        self.quit = True

    def DrawBackground(self):
        self.screen.blit(self.img_background, (0, 0))
        self.chatbox.Paint()
        self.dashboard_rep.Repaint()
        self.remotePlayerDashboardListRep.Repaint()

    def Run(self):
        self.DrawBackground()
        self.schedualer.Poll()
        pygame.mouse.set_visible(False)
        pygame.time.set_timer(FRAME_EVT, int(self.FRAME_DELAY))
        self.quit = False
        self._last_frame_time = time.time()
        self.paint_frame_skip_count = 0
        self.started = True

        while self.quit == False:
            event = pygame.event.wait()

            if event.type != FRAME_EVT and self.chatbox.IsFocused():
                self.chatbox.event(event)

            if event.type is KEYDOWN:
                for dlg in self.dialogs: dlg.HandleKey(event.key)
                if event.key == K_ESCAPE: self.reallyQuitDialog.Open()
                elif event.key == K_RETURN:
                    self.chatbox.ToggleFocus()
                for keyboard in self.keyboards:
                    keyboard.handleClick(event.key)

            elif event.type == FRAME_EVT:
                _t = time.time()
                frame_delay_seconds = _t - self._last_frame_time
                frame_delay = frame_delay_seconds * 1000
                self._last_frame_time = _t

                self.schedualer.Poll()
                self.net.Poll()
                
                if not self.chatbox.IsFocused():
                    for keyboard in self.keyboards:
                        keyboard.handlePressedKeys(pygame.key.get_pressed(),
                                                   frame_delay)
                
                self.scene.physics_engine.Advance(frame_delay)
                
                for player in self.players:
                    player.Advance(frame_delay)
                    
                self.time += frame_delay_seconds

                self.paint_frame_skip_count = (1 + self.paint_frame_skip_count) % self.PAINT_FRAME_SKIP
                if self.paint_frame_skip_count == 0:
                    self.camera.Paint()
                    self.dashboard_rep.Paint()
                    self.remotePlayerDashboardListRep.Paint()
                    self.chatbox.Update()
                    for dlg in self.dialogs: dlg.Paint()
                    pygame.display.flip()
            
        pygame.mouse.set_visible(True)



