import sys, math
import os, time, random
from defines import *
from scene import *
from weapons import WEAPON_CLASSES
from game.m_MedicBox import *
from game.m_Controllers import *
from game.m_Game import BaseGame, FRAME_EVT
from game.m_Charactors import *
from game.m_EntityTypes import *
from game.m_EventSchedualer import *
from game.m_Dashboard import DashboardRep, RemotePlayerDashboardListRep
from graphics import Sprite, SpriteRes, ReallyStopGameDialog, CameraBottomTextDialog
from game.m_Players import Player, PlayerCamera
from m_ChatBox import *
from net import *
import pygame
from pygame.locals import *


SEND_FULL_SCENE_STATE_INTERVAL_SECONDS = 1


#===============================================================================
# RemotePlayer
#===============================================================================

class RemotePlayer:
    def __init__(self, player, host):
        self.player = player
        self.host = host


#===============================================================================
# ServerGameConfig
#===============================================================================

class ServerGameConfig:
    def __init__(self):
        server_player_name = ""
        self.screen = None
        self.scene = None
        self.scene_rep = None
        self.level_file_name = ""
        self.level_oid = ""
        self.net = None


#===============================================================================
# ServerGameInitiator
#===============================================================================

class ServerGameInitiator:
    
    def __init__(self, net):
        self.net = net
        self.initialized = False

    def Init(self, port):
        self.initialized = False
        if self.net.Init(port) == False:
            return False
        self.net.RegisterNewConnectionHandler(self.HandleNewConnection)
        self.net.RegisterErrorHandler(self.HandleNetError)
        self.server_game_config = ServerGameConfig()
        self.server_game_config.net = self.net
        self.error_callbacks = []
        self.new_player_callbacks = []
        self.player_closed_callbacks = []
        self.player_chat_msg_callbacks = []
        self.waiting_to_be_granted = {}
        self.all_connections = []
        self.game = None
        self.isGameRunning = False
        self.initialized = True
        self.BreakNAT()
        return True
    
    def BreakNAT(self):
        host_file = HostListConfigLoader(os.path.join('config_files', 'break_nat.txt'))
        self.net.BreakNAT(host_file.GetList(), 52002, 52010)

    def CreateNewGame(self):
        self.game = ServerGame(self.server_game_config)
        self.game.chatbox.inputCallbacks.append(self.SendChatMessage)

    def Close(self):
        if self.initialized:
            for host in self.all_connections:
                host.Close()
            self.net.Close()
            self.isGameRunning = False
            if self.game != None:
                self.game.Quit()
                self.game = None
            self.initialized = False

    def HandleNetError(self, sender, text):
        # self.Close()
        for cb in self.error_callbacks:
            cb(text)
        
    def HandleConnectionClosed(self, connection):
        if connection not in self.all_connections: return
        self.all_connections.remove(connection)
        for key in self.waiting_to_be_granted:
            if self.waiting_to_be_granted[key] == connection:
                self.waiting_to_be_granted.pop(key)
                break
        rp = self.getRemotePlayerByHost(connection)
        if (rp != None):
            self.game.gameController.DeletePlayer(rp.player.id)
            for cb in self.player_closed_callbacks: cb(rp.player.name)
        
    def HandleNewConnection(self, newHost):
        newHost.RegisterObjectReceivedHandler(JoinRequest.type, self.HandleJoinRequest)
        newHost.RegisterObjectReceivedHandler(GameInfoRequest.type, self.HandleGameInfoRequest)
        newHost.RegisterObjectReceivedHandler(GameMessageInitialStateRequest.type, self.HandleGameMessageInitialStateRequest)
        newHost.RegisterConnectionClosedHandler(self.HandleConnectionClosed)
        self.all_connections.append(newHost)
    
    def HandleChatMessage(self, sender, msg):
        rp = self.getRemotePlayerByHost(sender)
        if rp == None: return
        for cb in self.player_chat_msg_callbacks:
            cb(msg.player_name, msg.text)
        # forward the message to other players but the sender
        for other_player in self.game.remote_players:
            if other_player.player.name != rp.player.name:
                other_player.host.SendObject(msg)

    def SendChatMessage(self, text):
        msg = ChatMessage(self.server_game_config.server_player_name, text)
        for rp in self.game.remote_players:
            rp.host.SendObject(msg)
        
    def HandleJoinRequest(self, sender, msg):
        if ( (self.getRemotePlayerByHost(sender) != None) or
             (msg.player_name in self.waiting_to_be_granted)): return
        elif FLAMEINGO_VERSION != msg.client_version:
            response = JoinResponse(False, "server version is not compatible with this client. server version: " + FLAMEINGO_VERSION)
            sender.SendObject(response, FLAG_ACK_REQUIRED)
            return
        elif self.isGameRunning:
            response = JoinResponse(False, "game already started")
            sender.SendObject(response, FLAG_ACK_REQUIRED)
            return
        elif ( (self.getRemotePlayerByName(msg.player_name) != None) or
             (self.server_game_config.server_player_name == msg.player_name)):
            response = JoinResponse(False, "name already taken")
            sender.SendObject(response, FLAG_ACK_REQUIRED)
            return
        self.waiting_to_be_granted[msg.player_name] = sender
        for cb in self.new_player_callbacks: cb(msg.player_name)

    def HandleGameInfoRequest(self, sender, msg):
        response = GameInfoResponse()
        response.level_file_name = os.path.basename(self.server_game_config.level_file_name)
        response.level_oid = self.server_game_config.level_oid
        response.player_name = self.server_game_config.server_player_name
        sender.SendObject(response, FLAG_ACK_REQUIRED)

    def GrantPlayer(self, player_name, is_granted, reason_text):
        if player_name not in self.waiting_to_be_granted: return
        newPlayerId = -1
        newPlayerEntityId = -1
        if is_granted:
            host = self.waiting_to_be_granted.pop(player_name)
            newPlayerId = self.game.gameController.CreateNewRemotePlayer(player_name, host)
            newPlayerEntityId = self.game.gameController.GetPlayerEntityId(newPlayerId)
            host.RegisterObjectReceivedHandler(ChatMessage.type, self.HandleChatMessage)
            host.RegisterObjectReceivedHandler(ChatMessage.type, self.game.chatbox.HandleChatMessage)
        msg = JoinResponse(is_granted, reason_text, newPlayerId, newPlayerEntityId)
        host.SendObject(msg, FLAG_ACK_REQUIRED)
        
    def HandleGameMessageInitialStateRequest(self, sender, msg) :
        msg = GameMessageInitialStateResponse()
        msg.scene_state = self.game.sceneController.CreateFullSceneStateObject()
        for player_id in self.game.gameController.GetAllPlayerIds():
            player_data = self.game.gameController.GetNewPlayerData(player_id)
            msg.player_list.append(player_data)
        sender.SendObject(msg, FLAG_ACK_REQUIRED)
        
    def getRemotePlayerByHost(self, host):
        for rp in self.game.remote_players:
            if rp.host == host: return rp
        return None

    def getRemotePlayerByName(self, player_name):
        for rp in self.game.remote_players:
            if rp.player.name == player_name: return rp
        return None

    def StartGame(self):
        msg = MessageStartGame()
        for rp in self.game.remote_players:
            rp.host.SendObject(msg, FLAG_ACK_REQUIRED)
        self.isGameRunning = True
        self.game.schedualer.StartTimer()
        self.game.Run()

    def IsGameRunning(self):
        return self.isGameRunning

    def GetNumberOfClients(self):
        if self.game == None: return 0
        return len(self.game.remote_players)


#===============================================================================
# ServerSceneController
#===============================================================================

class ServerSceneController:
    
    def __init__(self, scene, remote_hosts, schedualer):
        self.localSceneController = LocalSceneController(scene)
        self.remote_hosts = remote_hosts
        self.schedualer = schedualer
        self.dirty_entities = {}
        self.scene = scene

    def SendToAllPlayers(self, msg, flags=0):
        for host in self.remote_hosts:
            host.SendObject(msg, flags)

    def CreateNewEntity(self, entityType, x=0, y=0, vx=0, vy=0, state=0, id=-1):
        timeVal = self.schedualer.time
        entity_id = self.localSceneController.CreateNewEntity(entityType, x, y, vx, vy, state, id)
        self.dirty_entities[entity_id] = 1
        return entity_id

    def DeleteEntity(self, id):
        timeVal = self.schedualer.time # + ACTION_DELAY_SECONDS
        self.localSceneController.DeleteEntity(id)
        if id in self.dirty_entities:
            self.dirty_entities.pop(id)
        msg = SceneMessageDeleteEntity(timeVal, id)
        self.SendToAllPlayers(msg)

    def SetEntityState(self, id, state):
        if self.localSceneController.GetEntityState(id) ==  state:
            return
        self.localSceneController.SetEntityState(id, state)
        self.dirty_entities[id] = 1

    def SetEntityVelocity(self, id, vx, vy):
        self.localSceneController.SetEntityVelocity(id, vx, vy)
        self.dirty_entities[id] = 1
        
    def SetEntityPos(self, id, x, y):
        self.localSceneController.SetEntityPos(id, x, y)
        self.dirty_entities[id] = 1

    def SendChanges(self):
        timeVal = self.schedualer.time
        for id in self.dirty_entities:
            if self.localSceneController.EntityExists(id):
                entity = self.localSceneController.scene.objectMapById[id]
                msg = SceneMessageUpdateEntity(timeVal, entity.type, entity.x, entity.y, entity.vx, entity.vy, entity.state, entity.id)
                self.SendToAllPlayers(msg)
        self.dirty_entities.clear()
        
    def GetEntityPos(self, id):
        return self.localSceneController.GetEntityPos(id)

    def GetEntityPosCenter(self, id):
        return self.localSceneController.GetEntityPosCenter(id)
    
    def GetEntityVelocity(self, id):
        return self.localSceneController.GetEntityVelocity(id)
    
    def GetEntityState(self, id):
        return self.localSceneController.GetEntityState(id)

    def EntityExists(self, id):
        return self.localSceneController.EntityExists(id)
    
    def GetEntityData(self, id):
        return self.localSceneController.GetEntityData(id)

    def AttachObserverToEntity(self, id, observer):
        self.localSceneController.AttachObserverToEntity(id, observer)
    
    def RemoveLand(self, x, y, shapeIndex):
        timeVal = self.schedualer.time
        self.localSceneController.RemoveLand(x, y, shapeIndex)
        msg = SceneMessageRemoveLand(timeVal, x, y, shapeIndex)
        self.SendToAllPlayers(msg, FLAG_ACK_REQUIRED)
        
    def CreateExplosion(self, x, y, vx, vy, radius, power):
        infected_entity_id_list = self.localSceneController.CreateExplosion(
            x, y,
            vx, vy,
            radius, power)
        for id in infected_entity_id_list:
            self.dirty_entities[id] = 1

    def CreateFullSceneStateObject(self):
        obj = FullSceneState()
        for key in self.scene.objectMapById:
            entityData = SceneEntityData()
            realEntity = self.scene.objectMapById[key]
            entityData.id = key
            entityData.x = realEntity.x
            entityData.y = realEntity.y
            entityData.vx = realEntity.vx
            entityData.vy = realEntity.vy
            entityData.state = realEntity.state
            entityData.entityType = realEntity.type
            obj.entity_list.append(entityData)
        return obj
        
    def AdvanceEntity(self, entity_id, time):
        self.localSceneController.AdvanceEntity(entity_id, time)

        
#===============================================================================
# ServerGameController
#===============================================================================

class ServerGameController:

    def __init__(self, game):
        self.game = game
        self.localGameController = LocalGameController(self.game)

    def SendToAllPlayers(self, msg, flags=0):
        for host in self.game.remote_hosts:
            host.SendObject(msg, flags)

    def SendPlayerUpdate(self, player):
        msg = GameMessageUpdatePlayer(
            self.game.schedualer.time,
            player.id,
            player.dashboard.lives.value,
            player.dashboard.health.value,
            player.dashboard.flight.value,
            player.dashboard.weapon_name.value,
            player.dashboard.ammo.value,
            player.dashboard.ammo.loading,
            player.aim_marker.angle)
        self.SendToAllPlayers(msg)
        
    def GetNewPlayerData(self, player_id):
        entity_id = self.localGameController.GetPlayerEntityId(player_id)
        obj = self.game.sceneController.GetEntityData(entity_id)
        entity_data = SceneEntityData(obj.type, obj.x, obj.y, obj.vx, obj.vy, obj.state, obj.id)
        new_player_data = NewPlayerData(player_id,
                                        self.GetPlayerName(player_id),
                                        entity_data)
        return new_player_data

    def CreateNewPlayer(self, player_name, player_id=-1, entity_id=-1):
        player_id = self.localGameController.CreateNewPlayer(player_name)
        player_data = self.GetNewPlayerData(player_id)
        msg = GameMessageCreateNewPlayer(player_data)
        self.SendToAllPlayers(msg, FLAG_ACK_REQUIRED)
        return player_id
                    
    def CreateNewRemotePlayer(self, player_name, host):
        player_id = self.CreateNewPlayer(player_name)
        entity_id = self.localGameController.GetPlayerEntityId(player_id)
        self.game.scene.physics_engine.entity_ignore_list.append(entity_id)
        rp = RemotePlayer(self.game.playerIdMap[player_id], host)
        self.game.remote_players.append(rp)
        self.game.remote_hosts.append(host)
        self.game.remotePlayerDashboardList.append(rp.player.dashboard)
        host.RegisterObjectReceivedHandler(GameMessagePlayerInput.type, self.game.messageHandler.HandleGameMessagePlayerInput)
        return player_id

    def CreateNewLocalPlayer(self, player_name):
        return self.CreateNewPlayer(player_name)

    def DeletePlayer(self, id):
        msg = GameMessageDeletePlayer(id)
        self.SendToAllPlayers(msg, FLAG_ACK_REQUIRED)
        self.localGameController.DeletePlayer(id)
        for rp in self.game.remote_players:
            if rp.player.id == id:
                self.game.remote_players.remove(rp)
                self.game.remote_hosts.remove(rp.host)
                self.game.remotePlayerDashboardList.remove(rp.player.dashboard)
                if self.game.started:
                    self.game.camera_bottom_dlg.Open(rp.player.name + " has quit the game")

    def DecreasePlayerHealth(self, id, value):
        player = self.game.playerIdMap[id]
        retVal = self.localGameController.DecreasePlayerHealth(id, value)
        if self.PlayerExists(id): # decreasing health might kill a player and delete it
            self.SendPlayerUpdate(player)
        return retVal

    def IncreasePlayerHealth(self, id, value):
        self.localGameController.IncreasePlayerHealth(id, value)
        self.SendPlayerUpdate(self.game.playerIdMap[id])
    
    def PlayerMoveLeft(self, id):
        self.localGameController.PlayerMoveLeft(id)
        self.SendPlayerUpdate(self.game.playerIdMap[id])

    def PlayerMoveRight(self, id):
        self.localGameController.PlayerMoveRight(id)
        self.SendPlayerUpdate(self.game.playerIdMap[id])
    
    def PlayerMoveJump(self, id):
        self.localGameController.PlayerMoveJump(id)
        self.SendPlayerUpdate(self.game.playerIdMap[id])

    def PlayerShoot(self, id):
        self.localGameController.PlayerShoot(id)
        self.SendPlayerUpdate(self.game.playerIdMap[id])

    def PlayerAimUp(self, id, time):
        self.localGameController.PlayerAimUp(id, time)
        self.SendPlayerUpdate(self.game.playerIdMap[id])
        
    def PlayerAimDown(self, id, time):
        self.localGameController.PlayerAimDown(id, time)
        self.SendPlayerUpdate(self.game.playerIdMap[id])

    def GetAllPlayerIds(self):
        return self.localGameController.GetAllPlayerIds()

    def GetPlayerEntityId(self, player_id):
        return self.localGameController.GetPlayerEntityId(player_id)
    
    def GetPlayerAimXY(self, player_id):
        return self.localGameController.GetPlayerAimXY(player_id)
            
    def GetPlayerName(self, player_id):
        return self.localGameController.GetPlayerName(player_id)
        
    def GetPlayerLives(self, player_id):
        return self.localGameController.GetPlayerLives(player_id)

    def GetPlayerHealth(self, player_id):
        return self.localGameController.GetPlayerHealth(player_id)

    def GetPlayerFlight(self, player_id):
        return self.localGameController.GetPlayerFlight(player_id)
    
    def SetPlayerFlight(self, player_id, value):
        return self.localGameController.SetPlayerFlight(player_id, value)
    
    def PlayerExists(self, player_id):
        return self.localGameController.PlayerExists(player_id)
    
    def GetPlayerByEntityId(self, entity_id):
        return self.localGameController.GetPlayerByEntityId(entity_id)

    def SetTimer(self, timeout, func, args):
        self.localGameController.SetTimer(timeout, func, args)



#===============================================================================
# ServerGameMessageHandler
#===============================================================================


class ServerGameMessageHandler:

    def __init__(self, game):
        self.game = game
        self.game.schedualer.PostAt(0, self.SendFullSceneState)
        self.player_input_map = {} # from player id to PlayerInput
        
    def SendFullSceneState(self):
        self.game.schedualer.Post(SEND_FULL_SCENE_STATE_INTERVAL_SECONDS, self.SendFullSceneState)
        msg = SceneMessageFullState(self.game.schedualer.time,
                                    self.game.sceneController.CreateFullSceneStateObject())
        for host in self.game.remote_hosts:
            host.SendObject(msg)

    def HandleGameMessagePlayerInput(self, sender, msg):
        rp = self.game.getRemotePlayerByHost(sender)
        if (rp == None) or (rp.player.id != msg.player_id) or (rp.player.dashboard.lives.value <= 0):
            return
        
        if msg.player_id not in self.player_input_map:
            self.player_input_map[msg.player_id] = PlayerInput()
        last_input = self.player_input_map[msg.player_id]
        
        entity_id = self.game.gameController.GetPlayerEntityId(msg.player_id)
        if not self.game.sceneController.EntityExists(entity_id): return

        for next_input in msg.input_list:
            if next_input.time <= last_input.time: 
                continue
            dt = (next_input.time - last_input.time) * 1000
            
            self.game.sceneController.AdvanceEntity(entity_id, dt)
            if not self.game.sceneController.EntityExists(entity_id):
                # advancing the physics might cause the player to loose
                return
            rp.player.Advance(dt)
                    
            if  next_input.input_byte & PLAYER_INPUT_LEFT != 0 :
                self.game.gameController.PlayerMoveLeft(msg.player_id)
            if next_input.input_byte & PLAYER_INPUT_RIGHT != 0 :
                self.game.gameController.PlayerMoveRight(msg.player_id)
            if next_input.input_byte & PLAYER_INPUT_JUMP != 0 :
                self.game.gameController.PlayerMoveJump(msg.player_id)
            if next_input.input_byte & PLAYER_INPUT_SHOOT != 0 :
                self.game.gameController.PlayerShoot(msg.player_id)
                
            if last_input.input_byte & PLAYER_INPUT_UP != 0 :
                self.game.gameController.PlayerAimUp(msg.player_id, dt)
            if last_input.input_byte & PLAYER_INPUT_DOWN != 0 :
                self.game.gameController.PlayerAimDown(msg.player_id, dt)
            
            last_input = next_input
            
            x, y = self.game.sceneController.GetEntityPos(entity_id)
            vx, vy = self.game.sceneController.GetEntityVelocity(entity_id)
            cmsg = GameMessagePlayerCorrection(next_input.time,
                                               x, y, vx, vy,
                                               self.game.gameController.GetPlayerFlight(msg.player_id))
            sender.SendObject(cmsg)
            
        rp.player.SetAimAngle(msg.aim_angle)
        self.player_input_map[msg.player_id] = last_input
        
        

#===============================================================================
# ServerGame
#===============================================================================

class ServerGame(BaseGame):
    
    FRAME_DELAY = 10.0   # milliseconds
    PAINT_FRAME_SKIP = 3

    def NotifyPlayerKilled(self, player):
        if player == self.player:
            self.camera_bottom_dlg.Open("you were killed once")
        else:
            self.camera_bottom_dlg.Open(player.name + " was killed once")

    def NotifyPlayerLoss(self, player):
        # self.gameController.DeletePlayer(player.id)
        self.sceneController.DeleteEntity(player.charactor.entity_id)
        player.dashboard.lives.value = 0
        player.dashboard.health.value = 0
        for k in self.keyboards:
            if k.playerId == player.id:
                self.keyboards.remove(k)
                break
        if player == self.player:
            self.camera_bottom_dlg.Open("you are gone for good!")
        else:
            self.camera_bottom_dlg.Open(player.name + " is gone for good!")
        
    def __init__(self, game_config):
        BaseGame.__init__(self)
        self.started = False
        self.net = game_config.net
        self.screen = game_config.screen
        self.scene = game_config.scene
        self.scene_rep = game_config.scene_rep
        self.remote_hosts = []
        self.remote_players = []
        self.schedualer = Schedualer()
        self.sceneController = ServerSceneController(self.scene, self.remote_hosts, self.schedualer)
        self.gameController = ServerGameController(self)        
        self.quit = False
        self.messageHandler = ServerGameMessageHandler(self)
        self.remotePlayerDashboardList = []
        self.remotePlayerDashboardListRep = RemotePlayerDashboardListRep(
            self.remotePlayerDashboardList,
            self.screen.subsurface((20, 40, 160, 420)))
        
        ##
        ## background images
        ##
        self.img_background  = pygame.image.load(
            os.path.join('data', 'app', 'server_game_bg.png')).convert()
        
        ##
        ## create my player
        ##
        self.player_id = self.gameController.CreateNewLocalPlayer(game_config.server_player_name)
        self.player = self.playerIdMap[self.player_id]

        ##
        ## create keyboard
        ##
        self.keyboard = PlayerKeyboard(KEY_MAPPING_RIGHT,
                                       self.gameController,
                                       self.player_id)
        
        self.keyboards.append(self.keyboard)
        ##
        ## create DashboardRep object
        ##
        self.dashboard_rep = DashboardRep(self.player.dashboard,
                                          self.screen.subsurface([205, 410, 219, 69]) )

        ##
        ## create camera
        ##
        self.camera = PlayerCamera(self.player_id,
                                   self,
                                   self.screen.subsurface([200, 23, 430, 366]) )

        ##
        ## define collisions to be detected
        ##
        detector = self.scene.level.collidetector
        detector.DefineCollisionCheck(TYPE_BAZOOKA, TYPE_FLAMINGO)
        detector.DefineCollisionCheck(TYPE_MEDIC_BOX, TYPE_FLAMINGO)

        ##
        ## create chat box
        ##
        self.chatbox = ChatBox(self, game_config.server_player_name)
        
        ##
        ## declare the frame event
        ##
        pygame.event.Event(FRAME_EVT)

        ##
        ## initialize dialogs
        ##
        self.reallyQuitDialog = ReallyStopGameDialog()
        self.reallyQuitDialog.value_returned_callbacks.append(self.HandleReallyQuitDialogValue)
        
        self.camera_bottom_dlg = CameraBottomTextDialog(self.schedualer,
                                                        self.camera.output_surface)
        
        self.dialogs = [self.camera_bottom_dlg, self.reallyQuitDialog]
        
        ## create medic box in the future
        
        next_time = random.randint(10, 90)
        self.schedualer.PostAt(next_time, self._create_medic_box)


    def HandleReallyQuitDialogValue(self, dlg, val):
        dlg.Close()
        self.DrawBackground()
        if val == True: self.Quit()
        
    def _create_medic_box(self):
        self._post_create_medic_box()
        if len(self.scene.level.GetEntities(TYPE_MEDIC_BOX)) >= 2: return
        x = random.randint(0, self.scene.level.GetWidth() - 10)
        y = 1
        box = MedicBox(self.gameController, self.sceneController, x, y)

        
    def _post_create_medic_box(self):
        next_time = random.randint(10, 90)
        self.schedualer.Post(next_time, self._create_medic_box)

        
    def Quit(self, arg=None):
        self.quit = True

    def DrawBackground(self):
        self.screen.blit(self.img_background, (0, 0))
        self.chatbox.Paint()
        self.dashboard_rep.Repaint()
        self.remotePlayerDashboardListRep.Repaint()


    def Run(self):
        self.DrawBackground()
        self.schedualer.Poll()
        pygame.mouse.set_visible(False)
        pygame.time.set_timer(FRAME_EVT, int(self.FRAME_DELAY))
        self.quit = False
        self._last_frame_time = time.time()
        self.paint_frame_skip_count = 0
        self.started = True
        
        while self.quit == False:
            event = pygame.event.wait()

            if event.type != FRAME_EVT and self.chatbox.IsFocused():
                self.chatbox.event(event)

            if event.type is KEYDOWN:
                for dlg in self.dialogs: dlg.HandleKey(event.key)
                if event.key == K_ESCAPE: self.reallyQuitDialog.Open()
                elif event.key == K_RETURN:
                    self.chatbox.ToggleFocus()
                for keyboard in self.keyboards:
                    keyboard.handleClick(event.key)

            elif event.type == FRAME_EVT:
                _t = time.time()
                frame_delay = (_t - self._last_frame_time) * 1000
                self._last_frame_time = _t

                self.schedualer.Poll()
                self.net.Poll()
                self.sceneController.SendChanges()
                self.net.Poll()
                
                if not self.chatbox.IsFocused():
                    p_keys = pygame.key.get_pressed()
                    for keyboard in self.keyboards:
                        keyboard.handlePressedKeys(p_keys, frame_delay)
                
                self.scene.physics_engine.Advance(frame_delay)
                self.player.Advance(frame_delay)

                self.paint_frame_skip_count = (1 + self.paint_frame_skip_count) % self.PAINT_FRAME_SKIP
                if self.paint_frame_skip_count == 0:
                    self.camera.Paint()
                    self.dashboard_rep.Paint()
                    self.remotePlayerDashboardListRep.Paint()
                    self.chatbox.Update()
                    for dlg in self.dialogs: dlg.Paint()
                    pygame.display.flip()
            
        pygame.mouse.set_visible(True)



    def getRemotePlayerByHost(self, host):
        for rp in self.remote_players:
            if rp.host == host: return rp
        return None


