"""
Objects which controls scene.
For now, it's only the physics engine.
"""


import math
from m_Shapes import *


class RetVal: pass

        
class LocalSceneController:
    
    def __init__(self, scene):
        self.scene = scene
        self.localIdCounter = 0

    def CreateNewEntity(self, entityType, x=0, y=0, vx=0, vy=0, state=0, id=-1):
        if id == -1:
            id = self.localIdCounter
            self.localIdCounter += 1
        entity = self.scene.level.factory.Create(entityType)
        entity.x = x
        entity.y = y
        entity.vx = vx
        entity.vy = vy
        entity.state = state
        entity.id = id
        self.scene.level.AddEntity(entity)
        return id

    def DeleteEntity(self, id):
        if not self.scene.objectMapById.has_key(id): return
        entity = self.scene.objectMapById.pop(id)
        self.scene.level.RemoveEntity(entity)

    def SetEntityState(self, id, state):
        if not self.scene.objectMapById.has_key(id): return
        entity = self.scene.objectMapById[id]
        entity.state = state

    def SetEntityVelocity(self, id, vx, vy):
        if not self.scene.objectMapById.has_key(id): return
        entity = self.scene.objectMapById[id]
        entity.vx = vx
        entity.vy = vy
        
    def SetEntityPos(self, id, x, y):
        if not self.scene.objectMapById.has_key(id): return
        entity = self.scene.objectMapById[id]
        entity.x = x
        entity.y = y

    def GetEntityPos(self, id):
        entity = self.scene.objectMapById[id]
        return (entity.x, entity.y)

    def GetEntityPosCenter(self, id):
        entity = self.scene.objectMapById[id]
        xc, yc = entity.GetCenter()
        return (entity.x + xc, entity.y + yc)
    
    def GetEntityVelocity(self, id):
        entity = self.scene.objectMapById[id]
        return (entity.vx, entity.vy)
    
    def GetEntityState(self, id):
        entity = self.scene.objectMapById[id]
        return entity.state

    def EntityExists(self, id):
        return self.scene.objectMapById.has_key(id)

    def AttachObserverToEntity(self, id, observer):
        if not self.scene.objectMapById.has_key(id): return
        entity = self.scene.objectMapById[id]
        entity.AttachObserver(observer)
    
    def RemoveLand(self, x, y, shapeIndex):
        shape = shapes[shapeIndex]
        self.scene.level.RemoveLand(x, y, shape)
        
    def _shortest_point_on_rect(self, rect, dest_point):
        """
        given a rectangle and a destination point, find the point
        on the rectangle with the shortest distance.
        (this way we calculate the distance between a point and a rect)
        """
        x, y = dest_point
        x0, y0, dx, dy = rect
        xr, yr = 0, 0
        if x < x0: xr = x0
        elif x > x0 + dx: xr = x0 + dx
        else: xr = x
        if y < y0: yr = y0
        elif y > y0 + dy: yr = y0 + dy
        else: yr = y
        return (xr, yr)
        
    def CreateExplosion(self, x, y, vx, vy, radius, power):
        infected_entity_id_list = []
        for entity in self.scene.level.GetEntities():
            if entity.mass <= 0: continue
            x0, y0 = self._shortest_point_on_rect(
                        entity.rect.move((entity.x, entity.y)),
                        (x, y))
            xc, yc = entity.GetCenter()
            xc += entity.x
            yc += entity.y
            r = math.sqrt( (x0 - x) * (x0 -x) + (y0 - y) * (y0 - y) )
            if r < radius:
                radius *= 1.6
                if radius < 20: radius = 20
                hit_power = (radius - r) * power / radius
                hit_vx = 2 * (xc - x) * math.sqrt(power) * (radius - r) / radius
                hit_vy = 2 * (yc - y - 2) * math.sqrt(power) * (radius - r) / radius
                entity.vx += hit_vx / entity.mass
                entity.vy += hit_vy / entity.mass
                entity.NotifyExplosion(hit_vx, hit_vy, hit_power)
                infected_entity_id_list.append(entity.id)
        return infected_entity_id_list
        

    def AdvanceEntity(self, entity_id, time):
        if not self.scene.objectMapById.has_key(entity_id): return
        self.scene.physics_engine.AdvanceEntity(self.scene.objectMapById[entity_id], time)

    def GetEntityIdList(self):
        return self.scene.objectMapById.keys()

    def GetEntityData(self, id):
        if not self.scene.objectMapById.has_key(id): return
        entity = self.scene.objectMapById[id]
        retVal = RetVal()
        retVal.id = entity.id
        retVal.type = entity.type
        retVal.x = entity.x
        retVal.y = entity.y
        retVal.vx = entity.vx
        retVal.vy = entity.vy
        retVal.state = entity.state
        return retVal
        

