import struct, random, time



class FileHeader:
    """
    84 bytes len header.
    """
    
    SIZE = 84
    
    def __init__(self):
        self.filename = ''
        self.file_size = 0
        
    def ToString(self):
        return struct.pack("l 80s", self.file_size, self.filename)
        
    def FromString(self, s):
        t = struct.unpack("l 80s", s)
        self.file_size = t[0]
        self.filename = t[1].strip('\x00')
        

        
class Compiler:
    """
    creates a multifile.
    """
    
    def __init__(self):
        self.out_file = None

    def Start(self, out_file_name):
        self.out_file = file(out_file_name, 'wb')
        header = MultifileHeader()
        header.OID = header.GenerateOID()
        self.out_file.write(header.ToString())

    def Close(self):
        if self.out_file: 
            self.out_file.close
            self.out_file = None

    def WriteFile(self, filename, file_obj=None):
        if not file_obj:
            file_obj = file(filename, 'rb')
        ## calculate file size
        file_obj.seek(0, 2) # move to the end
        file_size = file_obj.tell()
        file_obj.seek(0) # rewind to beginning
        ##
        ## create header
        ##
        header = FileHeader()
        header.filename = filename
        header.file_size = file_size
        self.out_file.write(header.ToString())
        ## write the file itself
        while file_size > 0:
            buffer = file_obj.read(1024)
            file_size -= len(buffer)
            self.out_file.write(buffer)



class FileProxy:
    """
    a file-like object returned by Multifile.GetFile.
    oh, and it is read-only!
    """
    def __init__(self, multifile, pos, size):
        self.multifile = multifile
        self.pos = pos
        self.size = size
        self.x = 0 # referal position
        
    def tell(self): return self.x
    
    def seek(self, x, r=0):
        if r == 0: self.x = min(x, self.size)
        elif r == 1: self.x = min(x + self.x, self.size)
        elif r == 2: self.x = max(0, self.size - x)

    def close(self): return

    def read(self, size):
        tmp = self.multifile.file.tell()
        self.multifile.file.seek(self.pos + self.x)
        buffer = self.multifile.file.read(min(size, self.size - self.x))
        self.x += len(buffer)
        self.multifile.file.seek(tmp)
        return buffer
        
    def readline(self, size=-1):
        if size < 0: size = self.size - self.x
        tmp = self.multifile.file.tell()
        self.multifile.file.seek(self.pos + self.x)
        buffer = self.multifile.file.readline(min(size, self.size - self.x))
        self.x += len(buffer)
        self.multifile.file.seek(tmp)
        return buffer


class MultifileHeader:
    
    SIZE = 64
    
    def __init__(self):
        self.OID = ''

    def GenerateOID(self):
        abc = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
        result = time.asctime() + '-'
        while len(result) < 64: result += random.choice(abc)
        return result

    def ToString(self):
        return struct.pack("64s", self.OID)
        
    def FromString(self, s):
        t = struct.unpack("64s", s)
        self.OID = t[0]


    
class Multifile:
    """
    a read-only multifile
    """
    
    def __init__(self, fname):
        self.file = file(fname, 'rb')
        self.header = MultifileHeader()
        self.header.FromString(self.file.read(self.header.SIZE))
        
    def GetFile(self, fname):
        self.file.seek(MultifileHeader.SIZE)
        while True:
            header_str = self.file.read(FileHeader.SIZE)
            if len(header_str) < FileHeader.SIZE: break
            header = FileHeader()
            header.FromString(header_str)
            if header.filename == fname:
                return FileProxy(self, self.file.tell(), header.file_size)
            self.file.seek(header.file_size, 1)
        return None
    
    def GetSize(self):
        if self.file:
            self.file.seek(0, 2) 
            file_size = self.file.tell()
            self.file.seek(0)
            return file_size
        else:
            return None