"""
CuteThread is a thread which you can pass functions to run
in its own context instead in the main thread context.
(to acheive parallelism)

Moreover, it can wrapp instace methods of classes so you wont
have to modify your code! 

Written by Kornelious (koren_3@yahoo.com)


A Simple Usage Example
======================

class Human:
    def talk(self):
        print "im human!"


# initialization starts here

c = CuteThread()
c.start()

Human.talk = c.CreateInstanceMethod( Human.talk )

# that's it! everything else is just the same

h1 = Human()
h1.talk() # this will run within the threads context

c.Quit()


"""


from threading import Thread
from Queue import Queue



class _MyCallable:
    """ callable object which makes CuteThread to run a real function
    wrapped inside. """
    
    def __init__(self, func, cute_thread):
        self.cute_thread = cute_thread
        self.func = func

    def __call__(self, *args):
        self.cute_thread.Call(self.func, *args)



type_instancemethod = type(_MyCallable.__call__)



class CuteThread(Thread):
    
    def __init__(self):
        Thread.__init__(self)
        self._quit = False
        self.func_queue = Queue(500)
        self.setDaemon(True)

    def Call(self, func, *args):
        self.func_queue.put( (func, args) )

    def run(self):
        while not self._quit:
            func, args = self.func_queue.get()
            # print "===> CuteThread invoking a method"
            func(*args)

    def Quit(self):
        self.Call(self._do_quit_)

    def _do_quit_(self):
        self._quit = True

    def CreateInstanceMethod(self, method):
        """ returns a new instancemethod object which shall run
        in this thread context. """
        my_callable = _MyCallable(method.im_func, self)
        return type_instancemethod(my_callable, None, method.im_class)



