"""
EntityResource Viewer

This little program loads an EntityResource file and renders it's data.
I wanted it to be s small example for using EntityResource to generate
Entity and EntityRep, but it turned to be quiet long because of the pgu
package.

The only parts which is interesting for the example is::
    
    # loading a resource file
    res = EntityResource()
    res.Load('data/flamingo.cfg')
    
    # using the resource
    bird = Entity(res)
    bird_rep = EntityRep(res)
    
"""


from scene import *
from scene.m_View import EntityRep
import pygame
import sys
import os, time
from pygame.locals import *
from pgu import gui


usage_txt = \
"""
Usage: python rviewer.py <filename>
"""

if len(sys.argv) < 2:
    print usage_txt
    sys.exit(1)
    
screen = pygame.display.set_mode( (640, 480) ) 

gui.theme.load(os.path.join('data', 'themes', 'default'))

res = EntityResource()
res.Load(sys.argv[1])

#
# some globals
#

bird = Entity(res)
bird_rep = EntityRep(res)

mouse_move_counter = 10 # small optimization
pause = False
quit = False
show_frame_rect = False
show_frame_outlines = False

# callbacks from the gui

def switch_frame_rect_changed(switch):
    global show_frame_rect
    show_frame_rect = switch.value
    myView.Paint()
    pygame.display.flip()

def switch_frame_outlines_changed(switch):
    global show_frame_outlines
    show_frame_outlines = switch.value
    myView.Paint()
    pygame.display.flip()

def switch_fullscreen_changed(switch):
    pygame.display.toggle_fullscreen()

def slider_zoom_changed(slider):
    myView.SetZoom(slider.value)
    myView.Paint()
    pygame.display.flip()

def select_anim_changed(select):
    bird.SetAnimation(select.value)
    myGui.input_delay.value = bird._anims[bird.GetAnimationIndex()]._delay
    myView.Paint()
    pygame.display.flip()

def button_play_changed(obj):
    global pause
    pause = False
    myGui.input_frame.value = ""
    
def button_next_changed(obj):
    global pause
    pause = True
    a = bird._anims[bird.GetAnimationIndex()]
    bird.Advance(a._delay)
    myGui.input_frame.value = str(bird.GetFrameIndex())
    myView.Paint()
    pygame.display.flip()

# the gui

class TestGui(gui.Document):
    def __init__(self):
        gui.Document.__init__(self,x=10, y=20, cls='button', style={'padding_top':10, 'padding_bottom':10, 'padding_left':5, 'padding_right':5})
        t = gui.Table()
        t.tr()
        t.td(gui.Label("Zoom ", cls="text1"))
        slider_zoom = gui.HSlider(6,3,12,size=20,width=100,height=16)
        t.td(slider_zoom)
        
        t.tr(); t.td(gui.Spacer(5, 5))
        
        t.tr()
        t.td(gui.Label("Animations ", cls="text1"))
        select_anim = gui.Select()
        for i in range(0, len(bird._anims)):
            select_anim.add(str(i), i)
        t.td(select_anim)

        t.tr(); t.td(gui.Spacer(5, 5))

        t.tr()
        t.td(gui.Label("Anim Delay", cls="text1"))
        input_delay = gui.Input(size=4)
        input_delay.value = bird._anims[bird.GetAnimationIndex()]._delay
        t.td(input_delay)
        
        self.add(t)
        self.br(20)
        self.br(20)
        
        t = gui.Table()
        t.tr()
        button_play = gui.Button("Play")
        button_next = gui.Button(">>")
        input_frame = gui.Input(size=4)
        t.td(button_play)
        t.td(input_frame)
        t.td(button_next)
        
        self.add(t)
        self.br(20)
        self.br(20)
        
        e = gui.Switch()
        self.add(e)
        self.add(gui.Spacer(10, 10))
        e.connect(gui.CHANGE, switch_frame_rect_changed, (e))
        self.add(gui.Label("Show frame.rect", cls="text1"))

        self.br(20)

        e = gui.Switch()
        self.add(e)
        self.add(gui.Spacer(10, 10))
        e.connect(gui.CHANGE, switch_frame_outlines_changed, (e))
        self.add(gui.Label("Show frame.outlines", cls="text1"))

        self.br(20)
        self.br(20)

        switch_fullscreen = gui.Switch()
        self.add(switch_fullscreen)
        self.add(gui.Spacer(10, 10))
        self.add(gui.Label("Full screen", cls="text1"))

        self.br(50)
        
        # connecting callbacks
        
        slider_zoom.connect(gui.CHANGE, slider_zoom_changed, (slider_zoom) )
        select_anim.connect(gui.CHANGE, select_anim_changed, (select_anim) )
        button_play.connect(gui.CLICK, button_play_changed, None )
        button_next.connect(gui.CLICK, button_next_changed, None )
        switch_fullscreen.connect(gui.CHANGE, switch_fullscreen_changed, (switch_fullscreen) )
        
        self.input_delay = input_delay
        self.input_frame = input_frame


class TestView:
    def __init__(self, rect, screen):
        self.s = screen.subsurface(rect)
        self.zoom = 6
        self.bgcolor = (100, 200, 100)
        
    def SetZoom(self, zoom):
        self.zoom = zoom

    def _paint_grid(self, w, h):
        self.s.fill(self.bgcolor)
        dw = w * (self.zoom + 1)
        dh = h * (self.zoom + 1)
        for i in xrange(0, w+1):
            x = i * (self.zoom + 1)
            pygame.draw.line(self.s, (255,255,255), (x, 0), (x, dh))
            
        for i in range(0, h+1):
            y = i * (self.zoom + 1)
            pygame.draw.line(self.s, (255,255,255), (0, y), (dw, y))
    
    def _put_pixel(self, x, y, color):
        self.s.fill(color, [x*(self.zoom+1)+1, y*(self.zoom+1)+1, self.zoom, self.zoom])
        
    def _paint_image(self, s):
        w, h = s.get_width(), s.get_height()
        for i in range(0, w):
            for j in range(0, h):
                c = s.get_at( (i, j) )
                if c != s.get_colorkey():
                    self._put_pixel(i, j, c)
            
    def Paint(self):
        img = bird_rep.GetImageOf(bird)
        frame = bird.GetFrame()
        w, h = img.get_width(), img.get_height()
        self._paint_grid(w, h)
        self._paint_image(img)
        
        if show_frame_rect:
            for _x in range(frame.rect[0], frame.rect[0] + frame.rect[2]):
                self._put_pixel(_x, frame.rect[1], (60, 255, 255))
                self._put_pixel(_x, frame.rect[1] + frame.rect[3] - 1, (60, 255, 255))
            for _y in range(frame.rect[1], frame.rect[1] + frame.rect[3]):
                self._put_pixel(frame.rect[0], _y, (60, 255, 255))
                self._put_pixel(frame.rect[0] + frame.rect[2] - 1, _y, (60, 255, 255))
                
        if show_frame_outlines:
            lines = [frame.outlines.top, frame.outlines.bottom, frame.outlines.left, frame.outlines.right]
            for line in lines:
                for p in line:
                    self._put_pixel(p[0], p[1], (255, 255, 0))



FRAME_EVT = pygame.locals.USEREVENT + 1
pygame.event.Event(FRAME_EVT)
pygame.time.set_timer(FRAME_EVT, 30)


myView = TestView( [220, 20, 400, 440], screen)


app = gui.App()
myGui = TestGui()
app.init(widget=myGui,screen=screen)


while quit == False:

    if pause == False:
        bird.Advance(50)
        myView.Paint()

    for event in pygame.event.get():
        
        if event.type is KEYDOWN and event.key == K_ESCAPE:
            quit = True
            continue
        app.event(event)

    app.paint(screen)
    pygame.display.flip()
    time.sleep(0.05)

pygame.display.quit()
