"""

       i0   i1   i2
+----+----+----+----+----+----+
|    |head|    |    |tail|    |
+----+----+----+----+----+----+
       ^              ^
  first item      where to append
   (index 0)        next item

   
when head==tail, the list is either full or empty.
the member 'is_full' determins which is true.

"""

class CircularList:
    
    def __init__(self, size=1024):
        self.size = size
        self.head = 0
        self.tail = self.head
        self.items = []
        self.is_full = False
        
    def append(self, item):
        if len(self.items) < self.size:
            self.items.append(item)
            self.tail = (self.tail + 1) % self.size
        else:
            self.items[self.tail] = item
            if self.is_full:
                self.head = self.tail = (self.tail + 1) % self.size
            else:
                self.tail = (self.tail + 1) % self.size
        if self.head == self.tail: self.is_full = True
    
    def get_head(self):
        if self.head == self.tail and (not self.is_full): 
            return None
        else:
            return self.items[self.head]

    def pop_head(self):
        if self.head == self.tail and (not self.is_full): 
            return None
        else:
            retVal = self.items[self.head]
            self.head = (self.head + 1) % self.size
            self.is_full = False
            return retVal
        
    def get_all_items(self):
        retVal = []
        h = self.head
        if h == self.tail:
            if not self.is_full:
                return retVal
            else:
                retVal.append(self.items[h])
                h = (h + 1) % self.size
        while h != self.tail:
            retVal.append(self.items[h])
            h = (h + 1) % self.size
        return retVal

    def clear(self):
        self.tail = self.head
        self.is_full = False        
