import random
from scene import *
from game.m_EntityTypes import *
from scene.m_Shapes import *
from weapons import WEAPON_CLASSES


class AmmoBox(Entity):

    def __init__(self, res=None, level=None, prototype=None):
        Entity.__init__(self, res, level, prototype)
        self.shape = SHAPE_CIRCLE_12
        self.game = None
        
    def Init(self, game):
        self.game = game
        
    def Clone(self):
        return AmmoBox(prototype=self)
        
    def Explode(self):
        if self not in self.level.GetEntities(TYPE_AMMO_BOX): return
        self.game.sceneController.DeleteEntity(self)
        flame = self.game.sceneController.CreateNewEntity(TYPE_FLAME2)
        x1, y1 = flame.GetCenter()
        x0, y0 = self.GetCenter()
        x0 += self.x
        y0 += self.y
        self.sceneController.SetEntityPos(flame, x0 - x1, y0 - y1)
        self.sceneController.RemoveLand(int(x0),  int(y0), self.shape)
        self.game.sceneController.CreateExplosion(x0, y0, 0, 0, 25, 15)

    def CollideWithEntity(self, entity):
        """
        A callback called when the collision detector detects a collision
        with another entity.
        @param entity: the entity collided.
        """
        Entity.CollideWithEntity(self, entity)
        self.game.sceneController.DeleteEntity(self)
        player = self.game.GetPlayerByEntity(entity)
        if player != None:
            classObj = random.choice(WEAPON_CLASSES)
            player.weapons[player.weapon_index] = classObj(player)
            player.SetWeapon(player.weapon_index)

    def CollideWithLand(self, x, y, side):
        Entity.CollideWithLand(self, x, y, side)
        if side == SIDE_BOTTOM:
            if x == -1 and y == -1:
                self.game.sceneController.DeleteEntity(self)
                return

    def NotifyExplosion(self, vx, vy, power):
        self.Explode()
            
