"""
Bazooka is defined here.
"""

import os, sys, math, random
from scene import *
from m_Weapon import *
from game.m_EntityTypes import *
from scene.m_Shapes import *



class Bazooka(Weapon):
    
    radius = 25
    
    def __init__(self, player_id, gameController, sceneController):
        Weapon.__init__(self, player_id, gameController, sceneController)
        self.ammo.loading_rate = 20
        self.single_fire_ammo = 20
        self.DELAY_NEXT_FIRE = 420
        self.delay_next_fire = 0
        self.name.value = "Bazooka"
        self.power = 18
        self._parts_observer = RemovingObserver(self.sceneController)
        self.shape = SHAPE_CIRCLE_25
        
    def _create_small_flame(self, args):
        pos = args[0]
        v = args[1]
        x = pos[0]
        y = pos[1]
        vx = v[0]
        vy = v[1]
        flame = self.sceneController.CreateNewEntity(TYPE_FLAME2, 0, 0, vx, vy)
        x2, y2 = self.sceneController.GetEntityPosCenter(flame)
        self.sceneController.SetEntityPos(flame, x - x2, y - y2)
        
    def Fire(self, x, y, angle):
        if self.ammo.loading or self.delay_next_fire > 0: return
        Weapon.Fire(self, x, y, angle)
        x = x - 5
        y = y - 5
        vx = 200  * math.cos((angle * math.pi) / 180)
        vy = -200 * math.sin((angle * math.pi) / 180)
        angle = angle % 360
        state = int( (angle + 25) / 45 ) % 8
        bazooka = self.sceneController.CreateNewEntity(TYPE_BAZOOKA, x, y, vx, vy, state)
        self.sceneController.AttachObserverToEntity(bazooka, self)
        
    def NotifyCollisionWithEntity(self, entity_id1, entity_id2):
        if entity_id2 != self.gameController.GetPlayerEntityId(self.player_id):
            self._explode(entity_id1)
        
    def NotifyCollisionWithLand(self, entity_id, x, y, side):
        self._explode(entity_id)

    def _explode(self, bullet_entity_id):
        if self.sceneController.EntityExists(bullet_entity_id) == False:
            return
        x0, y0 = self.sceneController.GetEntityPosCenter(bullet_entity_id)
        x0, y0 = int(x0), int(y0)
        vx0, vy0 = self.sceneController.GetEntityVelocity(bullet_entity_id)
        self.sceneController.DeleteEntity(bullet_entity_id)
        # create parts
        for i in xrange(0, random.randint(0, 4)):
            x = x0
            y = y0
            vx = -vx0 * 0.1 + random.randint(0, 180) - 90
            vy = -vy0 * 0.1 + random.randint(0, 180) - 90
            state = i % 4
            part = self.sceneController.CreateNewEntity(TYPE_WHITE_DOT, x, y, vx, vy, state)
            self.sceneController.AttachObserverToEntity(part, self._parts_observer)
        # create big flame (flame4)
        flame = self.sceneController.CreateNewEntity(TYPE_FLAME4)
        x2, y2 = self.sceneController.GetEntityPosCenter(flame)
        x = x0 - x2
        y = y0 - y2
        self.sceneController.SetEntityPos(flame, x, y)
        #v = (bullet_entity.vx * 0.1, bullet_entity.vy * 0.1)
        v = (0, 0)
        # create small flames at different times
        for i in xrange(0, random.randint(1, 2)):
            time = random.randint(0, 300)
            _pos = (x0 + random.random() * 30 - 15,
                    y0 + random.random() * 30 - 15)
            self.gameController.SetTimer(time * 0.001,
                                         self._create_small_flame, 
                                         (_pos, v) )
        # remove land
        self.sceneController.RemoveLand(x0, y0, self.shape)
        # create explosion
        self.sceneController.CreateExplosion(
            x0, y0,
            vx0, vy0,
            self.radius, self.power)
