import os, sys, math, random
from scene import *
from m_Weapon import *
from game.m_EntityTypes import *
from game.m_Shapes import *


class CannonPartsObserver:
    
    def __init__(self, level):
        self.level = level
    
    def NotifyCollisionWithEntity(self, entity1, entity2):
        return
        
    def NotifyCollisionWithLand(self, entity, x, y, side):
        self.level.RemoveEntity(entity)


class Cannon(Weapon):
    
    radius = 16
    
    def __init__(self, player):
        Weapon.__init__(self, player)
        self.power = 13
        self.ammo.loading_rate = 20
        self.single_fire_ammo = 16
        self.DELAY_NEXT_FIRE = 270
        self.delay_next_fire = 0
        self.name.value = "Cannon"
        self._parts_observer = RemovingObserver()
        self.shape = shapes[SHAPE_CIRCLE_12]
        
    def Fire(self, x, y, angle):
        if self.ammo.loading or self.delay_next_fire > 0: return
        Weapon.Fire(self, x, y, angle)
        level = self.player.game.GetLevel()
        bullet = level.factory.Create(TYPE_CANNON)
        bullet.SetActiveAnimations([0])
        bullet.x = x - 2.5
        bullet.y = y - 2.5
        v = 210
        bullet.vx = v * math.cos((angle * math.pi) / 180)
        bullet.vy = -v * math.sin((angle * math.pi) / 180)
        self.player.charactor.entity.vx -= bullet.vx / 5
        self.player.charactor.entity.vy -= bullet.vy / 5
        bullet.AttachObserver(self)
        level.AddEntity(bullet)
        
    def NotifyCollisionWithEntity(self, entity1, entity2):
        if entity2 != self.player.charactor.entity:
            self._explode(entity1, entity2)
        
    def NotifyCollisionWithLand(self, entity, x, y, side):
        entity.x += x
        entity.y += y
        self._explode(entity)

    def _explode(self, bullet_entity, hitten_entity=None):
        level = self.player.game.GetLevel()
        if bullet_entity not in level.GetEntities(TYPE_CANNON):
            return
        level.RemoveEntity(bullet_entity)
        x0, y0 = bullet_entity.GetCenter()
        x0 += int(bullet_entity.x)
        y0 += int(bullet_entity.y)
        level.RemoveLand(x0,  y0, self.shape)
        # create parts
        for i in range(0, 4):
            part = level.factory.Create(TYPE_CANNON_PARTS)
            part.x = x0
            part.y = y0
            part.vx = -bullet_entity.vx * 0.3 + random.randint(0, 200) - 100
            part.vy = -bullet_entity.vy * 0.3 + random.randint(0, 200) - 100
            part.SetActiveAnimations([i])
            part.AttachObserver(self._parts_observer)
            level.AddEntity(part)
        # create flame
        flame = level.factory.Create(TYPE_FLAME2)
        x2, y2 = flame.GetCenter()
        flame.x = x0 - x2 + random.random() * 10 - 5
        flame.y = y0 - y2 + random.random() * 10 - 5
        level.AddEntity(flame)
        # create an explosion
        self.player.game.physics_engine.CreateExplosion \
            (x0, y0, 
             bullet_entity.vx, bullet_entity.vy,
             self.radius, self.power)
        
        

    