import os, sys, math, random
from scene import *
from m_Weapon import *
from game.m_EntityTypes import *
from game.m_Shapes import *



class Chaingun(Weapon):
    
    radius = 5
    shape = shapes[SHAPE_CIRCLE_5]
    
    def __init__(self, player):
        Weapon.__init__(self, player)
        self.power = 4
        self.ammo.loading_rate = 26
        self.single_fire_ammo = 2.5
        self.DELAY_NEXT_FIRE = 50
        self.delay_next_fire = 0
        self.name.value = "Chaingun"
        
    def Fire(self, x, y, angle):
        if self.ammo.loading or self.delay_next_fire > 0: return
        Weapon.Fire(self, x, y, angle)
        level = self.player.game.GetLevel()
        dot = level.factory.Create(TYPE_BULLET)
        anim = int( (angle + 25) / 45 ) % 8
        dot.SetActiveAnimations([anim])
        dot.x = x - 2.5
        dot.y = y - 2.5
        dot.vx = 200  * math.cos((angle * math.pi) / 180)
        dot.vy = -200 * math.sin((angle * math.pi) / 180)
        self.player.charactor.entity.vx -= dot.vx / 16
        self.player.charactor.entity.vy -= dot.vy / 16
        dot.AttachObserver(self)
        level.AddEntity(dot)

    def NotifyCollisionWithEntity(self, entity1, entity2):
        if entity2 != self.player.charactor.entity:
            self._explode(entity1)
        
    def NotifyCollisionWithLand(self, entity, x, y, side):
        entity.x += x
        entity.y += y
        self._explode(entity)

    def _explode(self, dot_entity):
        level = self.player.game.GetLevel()
        if dot_entity not in level.GetEntities(TYPE_BULLET):
            return
        level.RemoveEntity(dot_entity)
        x0, y0 = int(dot_entity.x), int(dot_entity.y)
        level.RemoveLand(x0,  y0, self.shape)
        # create flame
        flame = level.factory.Create(TYPE_FLAME3)
        x2, y2 = flame.GetCenter()
        flame.x = x0 - x2
        flame.y = y0 - y2
        level.AddEntity(flame)
        # create an explosion
        self.player.game.physics_engine.CreateExplosion \
            (x0, y0, 
             dot_entity.vx, dot_entity.vy, 
             self.radius, self.power)

                