import os, sys, math, random
from scene import *
from m_Weapon import *



class GausGun(Weapon):
    
    radius = 30
    circle = Circle(15)
    
    def __init__(self, player):
        Weapon.__init__(self, player)
        self.power = 49
        self.ammo = 100
        self._loading = False
        self.name = "Gaus Gun"
        self._parts_observer = RemovingObserver()
        
    def Fire(self, x, y, angle):
        if self._loading or self.ammo < 100:
            self._loading = True
            return
        level = self.player.game.GetLevel()
        dot = level.factory.Create(TYPE_GAUS_GUN)
        dot.vx = 700  * math.cos((angle * math.pi) / 180)
        dot.vy = -700 * math.sin((angle * math.pi) / 180)
        dot.x = x
        dot.y = y
        dot.AttachObserver(self)
        level.AddEntity(dot)
        self.ammo = 0
        self._loading = True
        
    def Advance(self, time):
        if self._loading:
            self.ammo += 20 * (float(time)/1000)
            if self.ammo > 100:
                self.ammo = 100
                self._loading = False

    def NotifyCollisionWithEntity(self, entity1, entity2):
        if entity2 != self.player.charactor.entity:
            self._explode(entity1, entity2)
        
    def NotifyCollisionWithLand(self, entity, x, y, side):
        self._explode(entity)

    def _explode(self, bullet_entity, hitten_entity=None):
        level = self.player.game.GetLevel()
        if bullet_entity not in level.GetEntities(TYPE_GAUS_GUN):
            return
        level.RemoveEntity(bullet_entity)
        x0, y0 = int(bullet_entity.x), int(bullet_entity.y)
        # create parts
        for i in xrange(0, 15):
            _type = TYPE_WHITE_DOT
            _vf = 0.0
            if random.randint(0, 1) == 0:
                _type = TYPE_YELLOW_DOT
                _vf = 0.2
            part = level.factory.Create(_type)
            part.x = x0
            part.y = y0
            part.vx = -bullet_entity.vx * _vf + random.randint(0, 200) - 100
            part.vy = -bullet_entity.vy * _vf + random.randint(0, 200) - 100
            part.SetAnimation(i % 4)
            part.gravity = 160
            part.AttachObserver(self._parts_observer)
            level.AddEntity(part)
        # create flame
        flame = level.factory.Create(TYPE_FLAME3)
        x2, y2 = flame.GetCenter()
        flame.x = x0 - x2
        flame.y = y0 - y2
        level.AddEntity(flame)
        level.RemoveLand(x0,  y0, self.circle)
        # create an explosion
        self.player.game.physics_engine.CreateExplosion( \
                level, x0, y0, self.radius, self.power)

    