import os, sys, math, random
from scene import *
from m_Weapon import *



class Pistol(Weapon):
    
    radius = 15
    circle = Circle(3)
    
    def __init__(self, player):
        Weapon.__init__(self, player)
        self.power = 12
        self.ammo = 100
        self.delay_next = 0
        self._loading = False
        self.name = "Pistol"
        
    def Fire(self, x, y, angle):
        if self._loading or self.ammo < 10:
            self._loading = True
            return
        if self.delay_next > 0: return
        self.delay_next = 320
        level = self.player.game.GetLevel()
        dot = level.factory.Create(TYPE_YELLOW_DOT)
        dot.x = x
        dot.y = y
        dot.vx = 200 * math.cos((angle * math.pi) / 180)
        dot.vy = -200 * math.sin((angle * math.pi) / 180)
        dot.SetAnimation(random.randint(0, 5))
        dot.AttachObserver(self)
        level.AddEntity(dot)
        self.ammo -= 10
        if self.ammo < 10:
            self._loading = True
        
    def Advance(self, time):
        if self._loading:
            self.ammo += 40 * (float(time)/1000)
            if self.ammo > 100:
                self.ammo = 100
                self._loading = False
        elif self.delay_next > 0: self.delay_next -= time

    def NotifyCollisionWithEntity(self, entity1, entity2):
        if entity2 != self.player.charactor.entity:
            self._explode(entity1)
        
    def NotifyCollisionWithLand(self, entity, x, y, side):
        self._explode(entity)

    def _explode(self, dot_entity):
        level = self.player.game.GetLevel()
        if dot_entity not in level.GetEntities(TYPE_YELLOW_DOT):
            return
        level.RemoveEntity(dot_entity)
        x0, y0 = int(dot_entity.x), int(dot_entity.y)
        level.RemoveLand(x0,  y0, self.circle)
        # create flame
        flame = level.factory.Create(TYPE_FLAME3)
        x2, y2 = flame.GetCenter()
        flame.x = x0 - x2
        flame.y = y0 - y2
        level.AddEntity(flame)
        # remove land
        level.RemoveLand(x0,  y0, self.circle)
        # create an explosion
        self.player.game.physics_engine.CreateExplosion( \
                level, x0, y0, self.radius, self.power)
