import os, sys, math, random
from scene import *
from m_Weapon import *
from game.m_EntityTypes import *
from game.m_Shapes import *


class Shotgun(Weapon):
    
    radius = 5
    shape = shapes[SHAPE_CIRCLE_5]
    
    def __init__(self, player):
        Weapon.__init__(self, player)
        self.ammo.loading_rate = 20
        self.single_fire_ammo = 30
        self.DELAY_NEXT_FIRE = 350
        self.delay_next_fire = 0
        self.name.value = "Shotgun"
        self.power = 3
        
    def Fire(self, x, y, angle):
        if self.ammo.loading or self.delay_next_fire > 0: return
        Weapon.Fire(self, x, y, angle)
        level = self.player.game.GetLevel()
        for ang in xrange(int(angle) - 5, int(angle) + 5):
            dot = level.factory.Create(TYPE_BULLET)
            dot.x = x + random.randint(0, 2) - 1 - 2.5
            dot.y = y + random.randint(0, 2) - 1 - 2.5
            v = 200 + random.randint(0, 20)
            dot.vx = v * math.cos((ang * math.pi) / 180)
            dot.vy = -v * math.sin((ang * math.pi) / 180)
            self.player.charactor.entity.vx -= dot.vx / 16
            self.player.charactor.entity.vy -= dot.vy / 16
            dot.SetActiveAnimations([int( (ang + 25) / 45 ) % 8])
            dot.AttachObserver(self)
            level.AddEntity(dot)

    def NotifyCollisionWithEntity(self, entity1, entity2):
        if entity2 != self.player.charactor.entity:
            self._explode(entity1)
        
    def NotifyCollisionWithLand(self, entity, x, y, side):
        entity.x += x
        entity.y += y
        self._explode(entity)

    def _explode(self, dot_entity):
        level = self.player.game.GetLevel()
        if dot_entity not in level.GetEntities(TYPE_BULLET):
            return
        level.RemoveEntity(dot_entity)
        x0, y0 = int(dot_entity.x), int(dot_entity.y)
        level.RemoveLand(x0,  y0, self.shape)
        # create flame
        flame = level.factory.Create(TYPE_FLAME3)
        x2, y2 = flame.GetCenter()
        flame.x = x0 - x2
        flame.y = y0 - y2
        level.AddEntity(flame)
        # create an explosion
        self.player.game.physics_engine.CreateExplosion( \
                x0, y0, 
                dot_entity.vx, dot_entity.vy,
                self.radius, self.power)
