import sys, random
from scene import IEntityObserver, Entity
from game.m_Dashboard import LoadableDashboardItem, DashboardItem


class Weapon(IEntityObserver):
    """
    an abstract weapon.
    """

    # methods inherited from IEntityObserver:
        
    def NotifyCollisionWithEntity(self, entity1, entity2):
        pass
        
    def NotifyCollisionWithLand(self, entity, x, y, side):
        pass

    def __init__(self, player_id, gameController, sceneController):
        """
        @param player: the Player which owns the weapon.
        """
        self.gameController = gameController
        self.sceneController = sceneController
        self.player_id = player_id
        self.ammo = LoadableDashboardItem()
        self.ammo.value = 100
        self.ammo.loading_rate = 20
        self.single_fire_ammo = 20
        self.DELAY_NEXT_FIRE = 100
        self.delay_next_fire = 0
        self.name = DashboardItem()
        self.name.value = ""
        
    def Fire(self, x, y, angle):
        # if self.ammo.loading or self.delay_next_fire > 0: return
        self.delay_next_fire = self.DELAY_NEXT_FIRE
        self.ammo.value -= self.single_fire_ammo
        if self.ammo.value < self.single_fire_ammo: self.ammo.loading = True
        
    def Advance(self, time):
        if self.delay_next_fire > 0: self.delay_next_fire -= time



class RemovingObserver(IEntityObserver):
    """
    an observer that removes the collided entity from the scene.
    """

    def __init__(self, sceneController):
        self.sceneController = sceneController

    def NotifyCollisionWithEntity(self, entity_id1, entity_id2):
        pass
        
    def NotifyCollisionWithLand(self, entity_id, x, y, side):
        self.sceneController.DeleteEntity(entity_id)



