import os, sys, math, random
from scene import *
from m_Weapon import *


class ZimmEntity(Entity):
    
    def __init__(self, res=None, level=None, prototype=None):
        Entity.__init__(self, res, level, prototype)
        self.timer = 0
        self.state = 0
        
    def Clone(self):
        return ZimmEntity(prototype=self)
        
    def Disappear(self):
        self.state = 1
        self.SetAnimation(1)
        self.vx = self.vy = 0
        
    def Advance(self, time):
        Entity.Advance(self, time)
        if self.state == 0:
            self.timer += time
            if self.timer > 30000:
                self.Disappear()
        elif self.state == 1:
            if self._anims[self._anim_index].cycles > 0:
                self.level.RemoveEntity(self)


class Zimm(Weapon):
    
    def __init__(self, player):
        Weapon.__init__(self, player)
        self.power = 40
        self.ammo = 100
        self._loading = False
        self.name = "Zimm"
        self.delay_next = 0
        
    def Fire(self, x, y, angle):
        if self._loading or self.ammo < 49:
            self._loading = True
            return
        if self.delay_next > 0: return
        self.delay_next = 350
        level = self.player.game.GetLevel()
        dot = level.factory.Create(TYPE_ZIMM)
        dot.vx = 250  * math.cos((angle * math.pi) / 180)
        dot.vy = -250 * math.sin((angle * math.pi) / 180)
        xc, yc = dot.GetCenter()
        dot.x = x - xc
        dot.y = y - yc
        dot.AttachObserver(self)
        level.AddEntity(dot)
        self.ammo -= 49
        if self.ammo < 49:
            self._loading = True
        
    def Advance(self, time):
        if self._loading:
            self.ammo += 20 * (float(time)/1000)
            if self.ammo > 100:
                self.ammo = 100
                self._loading = False
        elif self.delay_next > 0:
            self.delay_next -= time

    def NotifyCollisionWithEntity(self, zimm_entity, entity2):
        level = self.player.game.GetLevel()
        if zimm_entity not in level.GetEntities(TYPE_ZIMM):
            return
        if entity2 == self.player.charactor.entity and \
           zimm_entity.timer < 75: return
        zimm_entity.Disappear()
        player = self.player.game.GetPlayerByEntity(entity2)
        if player != None:
            player.charactor.DecreaseHealth(self.power)
        
    def NotifyCollisionWithLand(self, entity, x, y, side):
        if side == Entity.SIDE_TOP or side == Entity.SIDE_BOTTOM:
            entity.vy *= -1
        elif side == Entity.SIDE_LEFT or side == Entity.SIDE_RIGHT:
            entity.vx *= -1
        
        

    